Package: checkmate
Type: Package
Title: Fast and Versatile Argument Checks
Description: Tests and assertions to perform frequent argument checks. A
    substantial part of the package was written in C to minimize any worries
    about execution time overhead.
Version: 2.3.4
Authors@R: c(
    person("Michel", "Lang", NULL, "michellang@gmail.com",
      role = c("cre", "aut"), comment = c(ORCID = "0000-0001-9754-0393")),
    person("Bernd", "Bischl", NULL, "bernd_bischl@gmx.net", role = "ctb"),
    person("Dénes", "Tóth", NULL, "toth.denes@kogentum.hu", role = "ctb",
      comment = c(ORCID = "0000-0003-4262-3217"))
    )
URL: https://mllg.github.io/checkmate/,
        https://github.com/mllg/checkmate
URLNote: https://github.com/mllg/checkmate
BugReports: https://github.com/mllg/checkmate/issues
NeedsCompilation: yes
ByteCompile: yes
Encoding: UTF-8
Depends: R (>= 3.0.0)
Imports: backports (>= 1.1.0), utils
Suggests: R6, fastmatch, data.table (>= 1.9.8), devtools, ggplot2,
        knitr, magrittr, microbenchmark, rmarkdown, testthat (>=
        3.0.4), tinytest (>= 1.1.0), tibble
License: BSD_3_clause + file LICENSE
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Collate: 'AssertCollection.R' 'allMissing.R' 'anyInfinite.R'
        'anyMissing.R' 'anyNaN.R' 'asInteger.R' 'assert.R' 'helper.R'
        'makeExpectation.R' 'makeTest.R' 'makeAssertion.R'
        'checkAccess.R' 'checkArray.R' 'checkAtomic.R'
        'checkAtomicVector.R' 'checkCharacter.R' 'checkChoice.R'
        'checkClass.R' 'checkComplex.R' 'checkCount.R'
        'checkDataFrame.R' 'checkDataTable.R' 'checkDate.R'
        'checkDirectoryExists.R' 'checkDisjunct.R' 'checkDouble.R'
        'checkEnvironment.R' 'checkFALSE.R' 'checkFactor.R'
        'checkFileExists.R' 'checkFlag.R' 'checkFormula.R'
        'checkFunction.R' 'checkInt.R' 'checkInteger.R'
        'checkIntegerish.R' 'checkList.R' 'checkLogical.R'
        'checkMatrix.R' 'checkMultiClass.R' 'checkNamed.R'
        'checkNames.R' 'checkNull.R' 'checkNumber.R' 'checkNumeric.R'
        'checkOS.R' 'checkPOSIXct.R' 'checkPathForOutput.R'
        'checkPermutation.R' 'checkR6.R' 'checkRaw.R' 'checkScalar.R'
        'checkScalarNA.R' 'checkSetEqual.R' 'checkString.R'
        'checkSubset.R' 'checkTRUE.R' 'checkTibble.R' 'checkVector.R'
        'coalesce.R' 'isIntegerish.R' 'matchArg.R' 'qassert.R'
        'qassertr.R' 'vname.R' 'wfwl.R' 'zzz.R'
Packaged: 2026-02-02 10:59:55 UTC; michel
Author: Michel Lang [cre, aut] (ORCID: <https://orcid.org/0000-0001-9754-0393>),
  Bernd Bischl [ctb],
  Dénes Tóth [ctb] (ORCID: <https://orcid.org/0000-0003-4262-3217>)
Maintainer: Michel Lang <michellang@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-03 08:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 08:58:20 UTC; unix
Archs: checkmate.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(print,AssertCollection)
export("%??%")
export(allMissing)
export(anyInfinite)
export(anyMissing)
export(anyNaN)
export(asCount)
export(asInt)
export(asInteger)
export(assert)
export(assertAccess)
export(assertArray)
export(assertAtomic)
export(assertAtomicVector)
export(assertCharacter)
export(assertChoice)
export(assertClass)
export(assertComplex)
export(assertCount)
export(assertDataFrame)
export(assertDataTable)
export(assertDate)
export(assertDirectory)
export(assertDirectoryExists)
export(assertDisjunct)
export(assertDouble)
export(assertEnvironment)
export(assertFALSE)
export(assertFactor)
export(assertFile)
export(assertFileExists)
export(assertFlag)
export(assertFormula)
export(assertFunction)
export(assertInt)
export(assertInteger)
export(assertIntegerish)
export(assertList)
export(assertLogical)
export(assertMatrix)
export(assertMultiClass)
export(assertNamed)
export(assertNames)
export(assertNull)
export(assertNumber)
export(assertNumeric)
export(assertOS)
export(assertPOSIXct)
export(assertPathForOutput)
export(assertPermutation)
export(assertR6)
export(assertRaw)
export(assertScalar)
export(assertScalarNA)
export(assertSetEqual)
export(assertString)
export(assertSubset)
export(assertTRUE)
export(assertTibble)
export(assertVector)
export(assert_access)
export(assert_array)
export(assert_atomic)
export(assert_atomic_vector)
export(assert_character)
export(assert_choice)
export(assert_class)
export(assert_complex)
export(assert_count)
export(assert_data_frame)
export(assert_data_table)
export(assert_date)
export(assert_directory)
export(assert_directory_exists)
export(assert_disjunct)
export(assert_double)
export(assert_environment)
export(assert_factor)
export(assert_false)
export(assert_file)
export(assert_file_exists)
export(assert_flag)
export(assert_formula)
export(assert_function)
export(assert_int)
export(assert_integer)
export(assert_integerish)
export(assert_list)
export(assert_logical)
export(assert_matrix)
export(assert_multi_class)
export(assert_named)
export(assert_names)
export(assert_null)
export(assert_number)
export(assert_numeric)
export(assert_os)
export(assert_path_for_output)
export(assert_permutation)
export(assert_posixct)
export(assert_r6)
export(assert_raw)
export(assert_scalar)
export(assert_scalar_na)
export(assert_set_equal)
export(assert_string)
export(assert_subset)
export(assert_tibble)
export(assert_true)
export(assert_vector)
export(checkAccess)
export(checkArray)
export(checkAtomic)
export(checkAtomicVector)
export(checkCharacter)
export(checkChoice)
export(checkClass)
export(checkComplex)
export(checkCount)
export(checkDataFrame)
export(checkDataTable)
export(checkDate)
export(checkDirectory)
export(checkDirectoryExists)
export(checkDisjunct)
export(checkDouble)
export(checkEnvironment)
export(checkFALSE)
export(checkFactor)
export(checkFile)
export(checkFileExists)
export(checkFlag)
export(checkFormula)
export(checkFunction)
export(checkInt)
export(checkInteger)
export(checkIntegerish)
export(checkList)
export(checkLogical)
export(checkMatrix)
export(checkMultiClass)
export(checkNamed)
export(checkNames)
export(checkNull)
export(checkNumber)
export(checkNumeric)
export(checkOS)
export(checkPOSIXct)
export(checkPathForOutput)
export(checkPermutation)
export(checkR6)
export(checkRaw)
export(checkScalar)
export(checkScalarNA)
export(checkSetEqual)
export(checkString)
export(checkSubset)
export(checkTRUE)
export(checkTibble)
export(checkVector)
export(check_access)
export(check_array)
export(check_atomic)
export(check_atomic_vector)
export(check_character)
export(check_choice)
export(check_class)
export(check_complex)
export(check_count)
export(check_data_frame)
export(check_data_table)
export(check_date)
export(check_directory_exists)
export(check_disjunct)
export(check_double)
export(check_environment)
export(check_factor)
export(check_false)
export(check_file_exists)
export(check_flag)
export(check_formula)
export(check_function)
export(check_int)
export(check_integer)
export(check_integerish)
export(check_list)
export(check_logical)
export(check_matrix)
export(check_multi_class)
export(check_named)
export(check_names)
export(check_null)
export(check_number)
export(check_numeric)
export(check_os)
export(check_path_for_output)
export(check_permutation)
export(check_posixct)
export(check_r6)
export(check_raw)
export(check_scalar)
export(check_scalar_na)
export(check_set_equal)
export(check_string)
export(check_subset)
export(check_tibble)
export(check_true)
export(check_vector)
export(expect_access)
export(expect_array)
export(expect_atomic)
export(expect_atomic_vector)
export(expect_character)
export(expect_choice)
export(expect_class)
export(expect_complex)
export(expect_count)
export(expect_data_frame)
export(expect_data_table)
export(expect_date)
export(expect_directory)
export(expect_directory_exists)
export(expect_disjunct)
export(expect_double)
export(expect_environment)
export(expect_factor)
export(expect_file)
export(expect_file_exists)
export(expect_flag)
export(expect_formula)
export(expect_function)
export(expect_int)
export(expect_integer)
export(expect_integerish)
export(expect_list)
export(expect_logical)
export(expect_matrix)
export(expect_multi_class)
export(expect_names)
export(expect_number)
export(expect_numeric)
export(expect_os)
export(expect_path_for_output)
export(expect_permutation)
export(expect_posixct)
export(expect_r6)
export(expect_raw)
export(expect_scalar)
export(expect_scalar_na)
export(expect_set_equal)
export(expect_string)
export(expect_subset)
export(expect_tibble)
export(makeAssertCollection)
export(makeAssertion)
export(makeAssertionFunction)
export(makeExpectation)
export(makeExpectationFunction)
export(makeTest)
export(makeTestFunction)
export(matchArg)
export(qassert)
export(qassertr)
export(qexpect)
export(qexpectr)
export(qtest)
export(qtestr)
export(register_test_backend)
export(reportAssertions)
export(testAccess)
export(testArray)
export(testAtomic)
export(testAtomicVector)
export(testCharacter)
export(testChoice)
export(testClass)
export(testComplex)
export(testCount)
export(testDataFrame)
export(testDataTable)
export(testDate)
export(testDirectory)
export(testDirectoryExists)
export(testDisjunct)
export(testDouble)
export(testEnvironment)
export(testFALSE)
export(testFactor)
export(testFile)
export(testFileExists)
export(testFlag)
export(testFormula)
export(testFunction)
export(testInt)
export(testInteger)
export(testIntegerish)
export(testList)
export(testLogical)
export(testMatrix)
export(testMultiClass)
export(testNamed)
export(testNames)
export(testNull)
export(testNumber)
export(testNumeric)
export(testOS)
export(testPOSIXct)
export(testPathForOutput)
export(testPermutation)
export(testR6)
export(testRaw)
export(testScalar)
export(testScalarNA)
export(testSetEqual)
export(testString)
export(testSubset)
export(testTRUE)
export(testTibble)
export(testVector)
export(test_access)
export(test_array)
export(test_atomic)
export(test_atomic_vector)
export(test_character)
export(test_choice)
export(test_class)
export(test_complex)
export(test_count)
export(test_data_frame)
export(test_data_table)
export(test_date)
export(test_directory)
export(test_directory_exists)
export(test_disjunct)
export(test_double)
export(test_environment)
export(test_factor)
export(test_false)
export(test_file_exists)
export(test_flag)
export(test_formula)
export(test_function)
export(test_int)
export(test_integer)
export(test_integerish)
export(test_list)
export(test_logical)
export(test_matrix)
export(test_multi_class)
export(test_named)
export(test_names)
export(test_null)
export(test_number)
export(test_numeric)
export(test_os)
export(test_path_for_output)
export(test_permutation)
export(test_posixct)
export(test_r6)
export(test_raw)
export(test_scalar)
export(test_scalar_na)
export(test_set_equal)
export(test_string)
export(test_subset)
export(test_tibble)
export(test_true)
export(test_vector)
export(vname)
export(wf)
export(wl)
importFrom(stats,setNames)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
useDynLib(checkmate,c_all_missing)
useDynLib(checkmate,c_any_infinite)
useDynLib(checkmate,c_any_missing)
useDynLib(checkmate,c_any_nan)
useDynLib(checkmate,c_check_array)
useDynLib(checkmate,c_check_atomic)
useDynLib(checkmate,c_check_atomic_vector)
useDynLib(checkmate,c_check_character)
useDynLib(checkmate,c_check_complex)
useDynLib(checkmate,c_check_count)
useDynLib(checkmate,c_check_dataframe)
useDynLib(checkmate,c_check_double)
useDynLib(checkmate,c_check_factor)
useDynLib(checkmate,c_check_flag)
useDynLib(checkmate,c_check_int)
useDynLib(checkmate,c_check_integer)
useDynLib(checkmate,c_check_integerish)
useDynLib(checkmate,c_check_list)
useDynLib(checkmate,c_check_logical)
useDynLib(checkmate,c_check_matrix)
useDynLib(checkmate,c_check_named)
useDynLib(checkmate,c_check_names)
useDynLib(checkmate,c_check_number)
useDynLib(checkmate,c_check_numeric)
useDynLib(checkmate,c_check_posixct)
useDynLib(checkmate,c_check_raw)
useDynLib(checkmate,c_check_scalar)
useDynLib(checkmate,c_check_string)
useDynLib(checkmate,c_check_vector)
useDynLib(checkmate,c_guess_type)
useDynLib(checkmate,c_is_integerish)
useDynLib(checkmate,c_qassert)
useDynLib(checkmate,c_qtest)
useDynLib(checkmate,c_which_first)
useDynLib(checkmate,c_which_last)

# checkmate

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/checkmate)](https://cran.r-project.org/package=checkmate)
[![R build status](https://github.com/mllg/checkmate/workflows/R-CMD-check/badge.svg)](https://github.com/mllg/checkmate)
[![Download Stats](https://cranlogs.r-pkg.org/badges/checkmate)](https://cran.r-project.org/package=checkmate)

Fast and versatile argument checks for R.

Ever used an R function that produced a not-very-helpful error message,
just to discover after minutes of debugging that you simply passed a wrong argument?

Blaming the laziness of the package author for not doing such standard checks
(in a dynamically typed language such as R) is at least partially unfair, as R makes theses types of checks
cumbersome and annoying. Well, that's how it was in the past.

Enter checkmate.

Virtually **every standard type of user error** when passing arguments into function can be
caught with a simple, readable line which produces an **informative error message** in case.
A substantial part of the package was written in C to **minimize any worries about execution time overhead**.
Furthermore, the package provides over 30 expectations to extend the popular [testthat package](https://cran.r-project.org/package=testthat) for unit tests.

## Installation

For the stable release, just install the latest version from [CRAN](https://cran.r-project.org/package=checkmate):

```r
install.packages("checkmate")
```

For the development version, use [devtools](https://cran.r-project.org/package=devtools):

```r
devtools::install_github("mllg/checkmate")
```

## Resources

- [R Journal Paper](https://doi.org/10.32614/RJ-2017-028)
- [NEWS](https://github.com/mllg/checkmate/blob/master/NEWS.md)
- [Documentation/Vignettes](https://mllg.github.io/checkmate/)
- [Grouped function reference](https://mllg.github.io/checkmate/reference/checkmate-package)

#' Check if an argument is a data table
#'
#' @templateVar fn DataTable
#' @template x
#' @inheritParams checkMatrix
#' @inheritParams checkList
#' @inheritParams checkDataFrame
#' @param key [\code{character}]\cr
#'   Expected primary key(s) of the data table.
#' @param index [\code{character}]\cr
#'   Expected secondary key(s) of the data table.
#' @template null.ok
#' @template checker
#' @family compound
#' @export
#' @examples
#' library(data.table)
#' dt = as.data.table(iris)
#' setkeyv(dt, "Species")
#' setkeyv(dt, "Sepal.Length", physical = FALSE)
#' testDataTable(dt)
#' testDataTable(dt, key = "Species", index = "Sepal.Length", any.missing = FALSE)
checkDataTable = function(x, key = NULL, index = NULL, types = character(0L), any.missing = TRUE, all.missing = TRUE, min.rows = NULL, max.rows = NULL, min.cols = NULL, max.cols = NULL, nrows = NULL, ncols = NULL, row.names = NULL, col.names = NULL, null.ok = FALSE) {
  if (!requireNamespace("data.table", quietly = TRUE))
    stop("Install package 'data.table' to perform checks of data tables")

  qassert(null.ok, "B1")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be a data.table, not 'NULL'")
  }

  if (!data.table::is.data.table(x)) {
    return(paste0("Must be a data.table", if (null.ok) " (or 'NULL')" else "", sprintf(", not %s", guessType(x))))
  }

  checkDataFrame(x, types, any.missing, all.missing, min.rows, max.rows, min.cols, max.cols, nrows, ncols, row.names, col.names, null.ok = FALSE) %and%
    checkDataTableProps(x, key, index)
}

checkDataTableProps = function(x, key = NULL, index = NULL) {
  if (!is.null(key)) {
    qassert(key, "S")
    if (!setequal(data.table::key(x) %??% character(0L), key))
      return(sprintf("Must have primary keys: %s", paste0(key, collapse = ",")))
  }
  if (!is.null(index)) {
    qassert(index, "S")
    indices = strsplit(data.table::indices(x) %??% "", "__", fixed = TRUE)[[1L]]
    if (!setequal(indices, index))
      return(sprintf("Must have secondary keys (indices): %s", paste0(index, collapse = ",")))
  }
  return(TRUE)
}

#' @export
#' @rdname checkDataTable
check_data_table = checkDataTable

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkDataTable
assertDataTable = makeAssertionFunction(checkDataTable, use.namespace = FALSE)

#' @export
#' @rdname checkDataTable
assert_data_table = assertDataTable

#' @export
#' @include makeTest.R
#' @rdname checkDataTable
testDataTable = makeTestFunction(checkDataTable)

#' @export
#' @rdname checkDataTable
test_data_table = testDataTable

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkDataTable
expect_data_table = makeExpectationFunction(checkDataTable, use.namespace = FALSE)

#' Check if an object contains missing values
#'
#' @description
#' \code{anyMissing} checks for the presence of at least one missing value,
#' \code{allMissing} checks for the presence of at least one non-missing value.
#' Supported are atomic types (see \code{\link[base]{is.atomic}}), lists and data frames.
#' Missingness is defined as \code{NA} or \code{NaN} for atomic types and data frame columns,
#' \code{NULL} is defined as missing for lists.\cr
#' \code{allMissing} applied to a \code{data.frame} returns \code{TRUE} if at least one column has
#' only non-missing values. If you want to perform the less frequent check that there is at least
#' a single non-missing observation present in the \code{data.frame}, use
#' \code{all(sapply(df, allMissing))} instead.
#'
#' @param x [\code{ANY}]\cr
#'  Object to check.
#' @return [\code{logical(1)}] Returns \code{TRUE} if any (\code{anyMissing}) or all (\code{allMissing})
#'  elements of \code{x} are missing (see details), \code{FALSE} otherwise.
#' @useDynLib checkmate c_any_missing
#' @export
#' @examples
#' anyMissing(c(1, 1))
#' anyMissing(c(1, NA))
#' anyMissing(list(1, NULL))
#'
#' x = iris
#' x[, "Species"] = NA
#' anyMissing(x)
#' allMissing(x)
anyMissing = function(x) {
  .Call(c_any_missing, x)
}

context("ALTREPS")

test_that("ALTREP no na", {
  skip_if(getRversion() < "3.5.0")
  wrapper = function(x, srt = 0, nna = 0) .Internal(wrap_meta(x, srt, nna))

  x = c(1, NA, 3)
  xw = wrapper(x, nna = 1)
  expect_true(anyMissing(x))
  expect_false(anyMissing(xw))

  x = as.integer(c(1, NA, 3))
  xw = wrapper(x, nna = 1)
  expect_true(anyMissing(x))
  expect_false(anyMissing(xw))

  x = c("a", NA, "b")
  xw = wrapper(x, nna = 1)
  expect_true(anyMissing(x))
  expect_false(anyMissing(xw))
})

test_that("ALTREP sorted", {
  skip_if(getRversion() < "3.5.0")
  wrapper = function(x, srt = 0, nna = 0) .Internal(wrap_meta(x, srt, nna))

  x = c(3, 1, 2)
  xw = wrapper(x, srt = 1)
  expect_false(testNumeric(x, sorted = TRUE))
  expect_true(testNumeric(xw, sorted = TRUE))

  x = as.integer(c(3, 1, 2))
  xw = wrapper(x, srt = 1)
  expect_false(testInteger(x, sorted = TRUE))
  expect_true(testInteger(xw, sorted = TRUE))

  x = c("c", "a", "b")
  xw = wrapper(x, srt = 1)
  expect_false(testCharacter(x, sorted = TRUE))
  expect_true(testCharacter(xw, sorted = TRUE))
})

context("anyMissing")

xb = logical(10)
xi = integer(10)
xd = double(10)
xc = complex(10)
xs = letters[1:10]
xl = as.list(1:10)
xm = matrix(1:9, 3)
xf = data.frame(a=1:5, b=1:5)


test_that("anyMissing", {
  expect_false(anyMissing(integer(0)))

  expect_false(anyMissing(xb))
  expect_false(anyMissing(xi))
  expect_false(anyMissing(xd))
  expect_false(anyMissing(xc))
  expect_false(anyMissing(xs))
  expect_false(anyMissing(xl))
  expect_false(anyMissing(xm))
  expect_false(anyMissing(xf))

  xb[5] = xi[5] = xd[5] = xc[5] = xs[5] = xm[2, 2] = xf$b[3] = NA
  xl[5] = list(NULL)
  expect_true(anyMissing(xb))
  expect_true(anyMissing(xi))
  expect_true(anyMissing(xd))
  expect_true(anyMissing(xc))
  expect_true(anyMissing(xs))
  expect_true(anyMissing(xl))
  expect_true(anyMissing(xm))
  expect_true(anyMissing(xf))


  expect_false(anyMissing(as.raw(1)))
  expect_false(anyMissing(NULL))
  expect_error(anyMissing(as.symbol("a")), "supported")
})

test_that("allMissing", {
  expect_true(allMissing(integer(0)))

  expect_false(allMissing(xb))
  expect_false(allMissing(xi))
  expect_false(allMissing(xd))
  expect_false(allMissing(xc))
  expect_false(allMissing(xs))
  expect_false(allMissing(xl))
  expect_false(allMissing(xm))
  expect_false(allMissing(xf))

  xb[5] = xi[5] = xd[5] = xc[5] = xm[2, 2] = xf$b[3] = NA
  xl[5] = list(NULL)

  expect_false(allMissing(xb))
  expect_false(allMissing(xi))
  expect_false(allMissing(xd))
  expect_false(allMissing(xc))
  expect_false(allMissing(xs))
  expect_false(allMissing(xl))
  expect_false(allMissing(xm))
  expect_false(allMissing(xf))

  xb[] = xi[] = xd[] = xc[] = xm[] = xs = NA
  xl = list(NULL, NULL)
  xf$a = xf$b = NA

  expect_true(allMissing(xb))
  expect_true(allMissing(xi))
  expect_true(allMissing(xd))
  expect_true(allMissing(xc))
  expect_true(allMissing(xs))
  expect_true(allMissing(xl))
  expect_true(allMissing(xm))
  expect_true(allMissing(xf))

  expect_false(allMissing(as.raw(1)))
  expect_false(allMissing(NULL))
  expect_error(allMissing(as.symbol("a")), "supported")
})

test_that("anyMissing and allMissing work correctly with data.frames", {
  df = data.frame(a = 1:2, b = 2:1)
  expect_false(anyMissing(df))
  expect_false(allMissing(df))
  df$b[1] = NA
  expect_true(anyMissing(df))
  expect_false(allMissing(df))
  df$b[2] = NA
  expect_true(anyMissing(df))
  expect_true(allMissing(df))
  expect_false(all(vlapply(df, allMissing)))
})

#' Check if an argument is a data frame
#'
#' @templateVar fn DataFrame
#' @template x
#' @param all.missing [\code{logical(1)}]\cr
#'  Are columns with only missing values allowed? Default is \code{TRUE}.
#' @inheritParams checkMatrix
#' @inheritParams checkList
#' @template null.ok
#' @template checker
#' @family compound
#' @family basetypes
#' @export
#' @useDynLib checkmate c_check_dataframe
#' @examples
#' testDataFrame(iris)
#' testDataFrame(iris, types = c("numeric", "factor"), min.rows = 1, col.names = "named")
checkDataFrame = function(x, types = character(0L), any.missing = TRUE, all.missing = TRUE, min.rows = NULL, max.rows = NULL, min.cols = NULL, max.cols = NULL, nrows = NULL, ncols = NULL, row.names = NULL, col.names = NULL, null.ok = FALSE) {
  .Call(c_check_dataframe, x, any.missing, all.missing, min.rows, max.rows, min.cols, max.cols, nrows, ncols, row.names, col.names, null.ok) %and%
  checkListTypes(x, types)
}

#' @export
#' @rdname checkDataFrame
check_data_frame = checkDataFrame

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkDataFrame
assertDataFrame = makeAssertionFunction(checkDataFrame, use.namespace = FALSE)

#' @export
#' @rdname checkDataFrame
assert_data_frame = assertDataFrame

#' @export
#' @include makeTest.R
#' @rdname checkDataFrame
testDataFrame = makeTestFunction(checkDataFrame)

#' @export
#' @rdname checkDataFrame
test_data_frame = testDataFrame

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkDataFrame
expect_data_frame = makeExpectationFunction(checkDataFrame, use.namespace = FALSE)

context("assert")

test_that("assert w/ check*", {
  x = NULL
  expect_true(assert(checkNull(x), checkDataFrame(x)))
  expect_true(assert(checkNull(x)))
  grepme = iris
  expect_true(assert(checkNull(grepme), checkDataFrame(grepme)))
  expect_error(assert(checkNull(grepme), checkNumeric(grepme)), "One of")
  expect_error(assert(checkNull(grepme), checkNumeric(grepme)), "grepme")

  x = 1
  expect_true(assert(checkNumeric(x), checkCount(x)))
  expect_true(assert(checkNumeric(x), checkCount(x), combine = "or"))
  expect_true(assert(checkNumeric(x), checkCount(x), combine = "and"))

  x = 1.1
  expect_true(assert(checkNumeric(x), checkCount(x), combine = "or"))
  expect_error(assert(checkNumeric(x), checkCount(x), combine = "and"))

  x = "a"
  expect_true(assert(checkString(x)))
  expect_error(assert(checkNumeric(x), checkCount(x), combine = "or"))
  expect_error(assert(checkNumeric(x), checkCount(x), combine = "and"))
})

test_that("bug #69", {
  sub = subset = 1:150
  res = assert(checkIntegerish(subset), checkLogical(subset, len = 150))
  expect_true(res)
  res = assert(checkIntegerish(sub), checkLogical(sub, len = 150))
  expect_true(res)
})

test_that("correct variable is reported (#182)", {
  f = function(a, b) assert(checkFunction(a), checkNumeric(b), combine = "and")
  expect_error(f(identity, "a"), "'b'")
})

test_that("can push to collection", {
  coll = makeAssertCollection()
  x = NULL
  expect_error(assert(checkNull(x), .var.name = c("a", "b"), add = coll), "length")
  expect_error(assert(checkNull(x), add = "not_assert_coll"), "AssertCollection")
  expect_true(assert(checkNull(x), add = coll))
  expect_true(coll$isEmpty())
  expect_true(assert(checkNull(x), checkDataFrame(x), add = coll))
  expect_true(coll$isEmpty())
  grepme = iris
  expect_true(assert(checkNull(grepme), checkDataFrame(grepme), add = coll))
  expect_true(coll$isEmpty())
  assert(checkNull(grepme), checkNumeric(grepme), add = coll)
  expect_match(
    coll$getMessages(), "Variable.+One of")
  assert(checkNull(grepme), checkNumeric(x), add = coll)
  expect_match(
    coll$getMessages()[2], "Variables.+grepme.+x.+One of")
  assert(
    checkNull(grepme), checkNumeric(x), .var.name = "TEST", add = coll)
  expect_match(
    coll$getMessages()[3], "Variable.+TEST.+One of")
  assert(
    checkNull(grepme), checkNumeric(x), .var.name = c("V1", "V2"), add = coll)
  expect_match(
    coll$getMessages()[4], "Variable.+V1.+V2.+One of")
})

context("checkDataFrame")

test_that("checkDataFrame", {
  myobj = iris
  expect_succ_all(DataFrame, myobj)
  myobj = TRUE
  expect_fail_all(DataFrame, myobj)

  expect_true(testDataFrame(data.frame()))
  expect_false(testDataFrame(NULL))
  expect_true(testDataFrame(data.frame(1)))
  expect_true(testDataFrame(iris))
  expect_false(testDataFrame(list(1)))

  x = iris
  expect_true(testDataFrame(x, types = c("numeric", "factor")))
  expect_false(testDataFrame(x, types = c("integer", "factor")))
  expect_false(testDataFrame(x, types = c("numeric", "character")))
  expect_true(testDataFrame(data.frame(), types = "NULL"))

  expect_true(testDataFrame(data.frame(), types = "numeric"))
  expect_error(assertDataFrame(1), "data.frame")
  expect_error(assertDataFrame(x, types = "numeric"), "numeric")

  # check nrow and ncol constraints
  expect_true(testDataFrame(x, nrows = 150L, ncols = 5L))
  expect_false(testDataFrame(x, nrows = 150L, ncols = 7L))
  expect_true(testDataFrame(x, min.rows = 2L, min.cols = 4L))
  expect_false(testDataFrame(x, min.rows = 151L, min.cols = 4L))
  expect_false(testDataFrame(x, min.rows = 1L, min.cols = 6L))
  expect_true(testDataFrame(x, max.rows = 200L))
  expect_false(testDataFrame(x, max.rows = 100L))
  expect_true(testDataFrame(x, max.cols = 10L))
  expect_false(testDataFrame(x, max.cols = 2L))
})


test_that("checkDataFrame name checking works", {
  df = data.frame(x = 1:2, y = 1:2)
  names(df) = c("x", "x")
  expect_identical(assertDataFrame(df), df)
  expect_error(assertDataFrame(df, col.names = "unnamed"), "colnames")

  names(df) = c("x", "")
  expect_error(assertDataFrame(df, col.names = "named"), "empty")

  names(df) = c("x", "x")
  expect_identical(assertDataFrame(df, col.names = "named"), df)
  expect_error(assertDataFrame(df, col.names = "unique"), "duplicated")
  expect_error(assertDataFrame(df, col.names = "strict"), "duplicated")
  expect_error(assertDataFrame(df, col.names = "foo"), "type")

  names(df) = c("x", "1")
  expect_identical(assertDataFrame(df, col.names = "named"), df)
  expect_identical(assertDataFrame(df, col.names = "unique"), df)
  expect_error(assertDataFrame(df, col.names = "strict"), "naming conventions")

  rownames(df) = letters[1:2]
  expect_succ_all(DataFrame, df, row.names = "strict")
  expect_succ_all(DataFrame, df, row.names = "unique")
  expect_succ_all(DataFrame, df, row.names = "named")
  expect_fail_all(DataFrame, df, row.names = "unnamed")

  rownames(df) = NULL
  expect_fail_all(DataFrame, df, row.names = "unnamed") # no names defaults to as.character(seq_row(x))
  expect_succ_all(DataFrame, df, row.names = "named")
  expect_succ_all(DataFrame, df, row.names = "unique")
  expect_fail_all(DataFrame, df, row.names = "strict")
})

test_that("dimension checks work for empty frames", {
  x = iris[, -c(1:5)]
  expect_true(testDataFrame(x, min.rows = 5))
  expect_true(testDataFrame(x, max.rows = 200))
  expect_false(testDataFrame(x, max.rows = 100))
  expect_true(testDataFrame(x, nrows = 150))
  expect_false(testDataFrame(x, min.rows = 151))
  expect_false(testDataFrame(x, nrows = 1))

  x = iris[-c(1:150), ]
  expect_true(testDataFrame(x, min.cols = 1))
  expect_true(testDataFrame(x, max.cols = 5))
  expect_false(testDataFrame(x, max.cols = 2))
  expect_true(testDataFrame(x, ncols = 5))
  expect_false(testDataFrame(x, min.cols = 6))
  expect_false(testDataFrame(x, ncols = 1))
})

test_that("missing values are detected", {
  x = data.frame(a = 1:2, b = c(1, 2))
  expect_true(testDataFrame(x, any.missing = FALSE))
  expect_true(testDataFrame(x, all.missing = FALSE))
  x$b[1] = NA
  expect_false(testDataFrame(x, any.missing = FALSE))
  expect_true(testDataFrame(x, all.missing = FALSE))
  x$a[1] = NA
  expect_false(testDataFrame(x, any.missing = FALSE))
  expect_true(testDataFrame(x, all.missing = FALSE))
  x$b[2] = NA
  expect_false(testDataFrame(x, any.missing = FALSE))
  expect_false(testDataFrame(x, all.missing = FALSE))
})

test_that("missing locations are reported correctly", {
  x = data.frame(a = 1:2, b = 1:2); x$a[1] = NA
  expect_true(grepl("column 'a', row 1", checkDataFrame(x, any.missing = FALSE)))
  x = data.frame(a = 1:2, b = 1:2); x$b[1] = NA
  expect_true(grepl("column 'b', row 1", checkDataFrame(x, any.missing = FALSE)))
  x = data.frame(a = 1:2, b = 1:2); x$b[2] = NA
  expect_true(grepl("column 'b', row 2", checkDataFrame(x, any.missing = FALSE)))

  x = data.frame(a = 1:2, b = 1:2, c = 1:2); x$c[2] = NA
  expect_true(grepl("column 'c', row 2", checkDataFrame(x, any.missing = FALSE)))
})

#' Check if an argument is a vector of type complex
#'
#' @templateVar fn Complex
#' @template x
#' @template na-handling
#' @inheritParams checkVector
#' @template typed.missing
#' @template null.ok
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_complex
#' @export
#' @examples
#' testComplex(1)
#' testComplex(1+1i)
checkComplex = function(x, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, names = NULL, typed.missing = FALSE, null.ok = FALSE) {
  .Call(c_check_complex, x, any.missing, all.missing, len, min.len, max.len, unique, names, typed.missing, null.ok)
}

#' @export
#' @rdname checkComplex
check_complex = checkComplex

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkComplex
assertComplex = makeAssertionFunction(checkComplex, c.fun = "c_check_complex", use.namespace = FALSE)

#' @export
#' @rdname checkComplex
assert_complex = assertComplex

#' @export
#' @include makeTest.R
#' @rdname checkComplex
testComplex = makeTestFunction(checkComplex, c.fun = "c_check_complex")

#' @export
#' @rdname checkComplex
test_complex = testComplex

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkComplex
expect_complex = makeExpectationFunction(checkComplex, c.fun = "c_check_complex", use.namespace = FALSE)

#' Check if an argument is a single integerish value
#'
#' @templateVar fn Int
#' @template x
#' @template na-handling
#' @template na.ok
#' @template bounds
#' @template tol
#' @template coerce
#' @template null.ok
#' @template checker
#' @template note-convert
#' @family scalars
#' @useDynLib checkmate c_check_int
#' @export
#' @examples
#' testInt(1)
#' testInt(-1, lower = 0)
checkInt = function(x, na.ok = FALSE, lower = -Inf, upper = Inf, tol = sqrt(.Machine$double.eps), null.ok = FALSE) {
  .Call(c_check_int, x, na.ok, lower, upper, tol, null.ok)
}

#' @export
#' @rdname checkInt
check_int = checkInt

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkInt
assertInt = makeAssertionFunction(checkInt, c.fun = "c_check_int", use.namespace = FALSE, coerce = TRUE)

#' @export
#' @rdname checkInt
assert_int = assertInt

#' @export
#' @include makeTest.R
#' @rdname checkInt
testInt = makeTestFunction(checkInt, c.fun = "c_check_int")

#' @export
#' @rdname checkInt
test_int = testInt

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkInt
expect_int = makeExpectationFunction(checkInt, c.fun = "c_check_int", use.namespace = FALSE)

context("checkInt")

test_that("checkInt", {
  myobj = 1L
  expect_succ_all(Int, myobj)
  myobj = 1.1
  expect_fail_all(Int, myobj)

  expect_false(testInt(integer(0)))
  expect_false(testInt(NULL))
  expect_false(testInt(FALSE))
  expect_false(testInt(TRUE))

  expect_true(testInt(1L))
  expect_true(testInt(1.))
  expect_false(testInt(NA))
  expect_true(testInt(NA_real_, na.ok = TRUE))
  expect_false(testInt(1:2))
  expect_false(testInt(""))

  expect_false(is.integer(assertInt(5)))
  expect_true(is.integer(assertInt(5, coerce = TRUE)))

  expect_error(assertInt(2+3i), "integerish")
})


test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkInt(NA, na.ok = TRUE, lower = 1))
  expect_true(checkInt(NA_character_, na.ok = TRUE, upper = 10))
  expect_fail_all(Int, 0L, lower = 1L)
  expect_fail_all(Int, 100L, upper = 10L)
})

test_that("Coercion works", {
  expect_identical(assertInt(2, coerce = TRUE), 2L)
  expect_error(assertInt("lkajsd"), "single integerish")
  expect_error(assertInt("lkajsd"), "not 'character'")

  # check that names are not dropped (#157)
  x = c("a" = 1)
  expect_identical(assertInt(x, coerce = TRUE), setNames(1L, "a"))
})

#' @title Check if an object is an integerish vector
#'
#' @description
#' An integerish value is defined as value safely convertible to integer.
#' This includes integers and numeric values which sufficiently close to an
#' integer w.r.t. a numeric tolerance `tol`.
#'
#' @note
#' To convert from integerish to integer, use \code{\link{asInteger}}.
#'
#' @templateVar fn Integerish
#' @template x
#' @template na-handling
#' @inheritParams checkInteger
#' @inheritParams checkVector
#' @template sorted
#' @template tol
#' @template coerce
#' @template typed.missing
#' @template null.ok
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_integerish
#' @export
#' @examples
#' testIntegerish(1L)
#' testIntegerish(1.)
#' testIntegerish(1:2, lower = 1L, upper = 2L, any.missing = FALSE)
checkIntegerish = function(x, tol = sqrt(.Machine$double.eps), lower = -Inf, upper = Inf, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, sorted = FALSE, names = NULL, typed.missing = FALSE, null.ok = FALSE) {
  .Call(c_check_integerish, x, tol, lower, upper, any.missing, all.missing, len, min.len, max.len, unique, sorted, names, typed.missing, null.ok)
}

#' @export
#' @rdname checkIntegerish
check_integerish = checkIntegerish

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkIntegerish
assertIntegerish = makeAssertionFunction(checkIntegerish, c.fun = "c_check_integerish", use.namespace = FALSE, coerce = TRUE)

#' @export
#' @rdname checkIntegerish
assert_integerish = assertIntegerish

#' @export
#' @include makeTest.R
#' @rdname checkIntegerish
testIntegerish = makeTestFunction(checkIntegerish, c.fun = "c_check_integerish")

#' @export
#' @rdname checkIntegerish
test_integerish = testIntegerish

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkIntegerish
expect_integerish = makeExpectationFunction(checkIntegerish, c.fun = "c_check_integerish", use.namespace = FALSE)

context("checkIntegerish")

test_that("checkIntegerish", {
  myobj = 1
  expect_succ_all(Integerish, myobj)
  myobj = 3.3
  expect_fail_all(Integerish, myobj)

  x = 1 - 0.9 -.1

  expect_true(testIntegerish(integer(0)))
  expect_false(testIntegerish(NULL))
  expect_false(testIntegerish(TRUE))
  expect_false(testIntegerish(FALSE))
  expect_true(testIntegerish(1L))
  expect_true(testIntegerish(c(-1, 0, 1)))
  expect_true(testIntegerish(1.))
  expect_true(testIntegerish(x))

  expect_true(testIntegerish(NA))
  expect_true(testIntegerish(NaN))
  expect_true(testIntegerish(c(1L, NA)))
  expect_true(testIntegerish(c(1, NA)))
  expect_true(testIntegerish(c(1, NaN)))

  expect_false(testIntegerish(1:2 + 0.0001))
  expect_false(testIntegerish(-Inf))
  expect_false(testIntegerish(Inf))

  expect_true(testIntegerish(3+0i))
  expect_false(testIntegerish(3-1i))
  expect_true(testIntegerish(as.complex(NA)))
  expect_false(testIntegerish(3+2i))
  expect_false(testIntegerish(list()))

  max = as.double(.Machine$integer.max)
  min = as.double(-.Machine$integer.max)
  expect_true(testIntegerish(min))
  expect_true(testIntegerish(max))
  expect_false(testIntegerish(min-1))
  expect_false(testIntegerish(max+1))
  expect_false(testIntegerish(min-.1))
  expect_false(testIntegerish(max+.1))

  expect_false(testIntegerish(NA, any.missing = FALSE))
  expect_false(testIntegerish(NA, all.missing = FALSE))
  expect_error(assertIntegerish(x, tol=0), "integerish")

  expect_false(is.integer(assertIntegerish(5)))
  expect_true(is.integer(assertIntegerish(5, coerce = TRUE)))
})

test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkInteger(NA, lower = 1))
  expect_true(checkInteger(NA_character_, upper = 10))
  expect_fail_all(Integerish, 0, lower = 1L)
  expect_fail_all(Integerish, 100, upper = 10L)
})

test_that("isIntegerish internal function", {
  expect_true(isIntegerish(1))
  expect_true(isIntegerish(1.))
  expect_false(isIntegerish(1.1))
})

test_that("sorted works", {
  expect_true(checkIntegerish(1:3, sorted = TRUE))
  expect_true(grepl("sorted", checkIntegerish(3:1, sorted = TRUE), fixed = TRUE))
})

test_that("informative error messages", {
  x = checkIntegerish((.Machine$integer.max + as.double(-3:3)))
  expect_string(x, fixed = "element 5")
  expect_string(x, fixed = "integer range")

  x = checkIntegerish((-.Machine$integer.max - 1))
  expect_string(x, fixed = "element 1")
  expect_string(x, fixed = "integer range")

  x = checkIntegerish(0.5)
  expect_string(x, fixed = "close to an integer")

  x = checkIntegerish(3 + 1i)
  expect_string(x, fixed = "imaginary part")
})

test_that("factors are detected (#164)", {
  x = factor(letters)
  expect_error(assertIntegerish(x), "factor")
})

test_that("Dates are detected (#223)", {
  x = Sys.Date()
  expect_error(assertIntegerish(x), "Date")
})

test_that("POSIXt is detected (#223)", {
  x = Sys.time()
  expect_error(assertIntegerish(x), "POSIX")
})

test_that("0 tolerance works (#177)", {
  expect_true(isIntegerish(1, tol = 0))
})

test_that("coerce rounds to next integer", {
  x = 1 - sqrt(.Machine$double.eps) / 10
  y = assert_integerish(x, coerce = TRUE)
  expect_identical(y, 1L)
})

test_that("typed.missing", {
  expect_true(testIntegerish(NA_character_))
  expect_true(testIntegerish(NA_character_, typed.missing = FALSE))
  expect_false(testIntegerish(NA_character_, typed.missing = TRUE))

  expect_true(testIntegerish(character()))
  expect_true(testIntegerish(character(), typed.missing = FALSE))
  expect_false(testIntegerish(character(), typed.missing = TRUE))
})

test_that("#226", {
  expect_true(test_integerish(NA_character_, any.missing = TRUE, all.missing = TRUE))
})

#' Check if an argument is a vector of type logical
#'
#' @templateVar fn Logical
#' @template x
#' @template na-handling
#' @inheritParams checkVector
#' @template null.ok
#' @template typed.missing
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_logical
#' @export
#' @examples
#' testLogical(TRUE)
#' testLogical(TRUE, min.len = 1)
checkLogical = function(x, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, names = NULL, typed.missing = FALSE, null.ok = FALSE) {
  .Call(c_check_logical, x, any.missing, all.missing, len, min.len, max.len, unique, names, typed.missing, null.ok)
}

#' @export
#' @rdname checkLogical
check_logical = checkLogical

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkLogical
assertLogical = makeAssertionFunction(checkLogical, c.fun = "c_check_logical", use.namespace = FALSE)

#' @export
#' @rdname checkLogical
assert_logical = assertLogical

#' @export
#' @include makeTest.R
#' @rdname checkLogical
testLogical = makeTestFunction(checkLogical, c.fun = "c_check_logical")

#' @export
#' @rdname checkLogical
test_logical = testLogical

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkLogical
expect_logical = makeExpectationFunction(checkLogical, c.fun = "c_check_logical", use.namespace = FALSE)

context("guessType")

test_that("guessType", {
  xb = logical(10)
  xi = integer(10)
  xd = double(10)
  xc = complex(10)
  xs = letters[1:10]
  xl = as.list(1:10)
  xm = matrix(1:9, 3)
  xa = array(1:3)
  xf = data.frame(a=1:5, b=1:5)
  xx = 1; class(xx) = "Foo"
  xxx = 1; class(xxx) = c("Foo", "Bar")

  expect_equal(guessType(xb), "logical")
  expect_equal(guessType(xi), "integer")
  expect_equal(guessType(xd), "double")
  expect_equal(guessType(xc), "complex")
  expect_equal(guessType(xs), "character")
  expect_equal(guessType(xl), "list")
  expect_equal(guessType(xm), "matrix")
  expect_equal(guessType(xa), "array")
  expect_equal(guessType(xf), "data.frame")
  expect_equal(guessType(xx), "Foo")
  expect_equal(guessType(xxx), "Foo/Bar")

  expect_true(grepl("NULL'$", checkLogical(NULL)))
  expect_true(grepl("logical'$", checkInteger(xb)))
  expect_true(grepl("integer'$", checkLogical(xi)))
  expect_true(grepl("double'$", checkLogical(xd)))
  expect_true(grepl("complex'$", checkLogical(xc)))
  expect_true(grepl("character'$", checkLogical(xs)))
  expect_true(grepl("factor'$", checkLogical(factor(xs))))
  expect_true(grepl("list'$", checkLogical(xl)))
  expect_true(grepl("matrix'$", checkLogical(xm)))
  expect_true(grepl("array'$", checkLogical(xa)))
  expect_true(grepl("frame'$", checkLogical(xf)))
  expect_true(grepl("Foo'$", checkLogical(xx)))
  expect_true(grepl("Foo/Bar'$", checkLogical(xxx)))
})

#' Check if an argument is a count
#'
#' @description
#' A count is defined as non-negative integerish value.
#'
#' @templateVar fn Count
#' @template x
#' @template na-handling
#' @template na.ok
#' @param positive [\code{logical(1)}]\cr
#'  Must \code{x} be positive (>= 1)?
#'  Default is \code{FALSE}, allowing 0.
#' @template tol
#' @template coerce
#' @template null.ok
#' @template checker
#' @template note-convert
#' @family scalars
#' @useDynLib checkmate c_check_count
#' @export
#' @examples
#' testCount(1)
#' testCount(-1)
checkCount = function(x, na.ok = FALSE, positive = FALSE, tol = sqrt(.Machine$double.eps), null.ok = FALSE) {
  .Call(c_check_count, x, na.ok, positive, tol, null.ok)
}

#' @export
#' @rdname checkCount
check_count = checkCount

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkCount
assertCount = makeAssertionFunction(checkCount, c.fun = "c_check_count", coerce = TRUE, use.namespace = FALSE)

#' @export
#' @rdname checkCount
assert_count = assertCount

#' @export
#' @include makeTest.R
#' @rdname checkCount
testCount = makeTestFunction(checkCount, c.fun = "c_check_count")

#' @export
#' @rdname checkCount
test_count = testCount

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkCount
expect_count = makeExpectationFunction(checkCount, c.fun = "c_check_count", use.namespace = FALSE)

#' Check if an argument is a list
#'
#' @note
#' Contrary to R's \code{\link[base]{is.list}}, objects of type \code{\link[base]{data.frame}}
#' and \code{\link[base]{pairlist}} are not recognized as list.
#'
#' Missingness is defined here as elements of the list being \code{NULL}, analogously to \code{\link{anyMissing}}.
#'
#' The test for uniqueness does differentiate between the different NA types which are built-in in R.
#' This is required to be consistent with \code{\link[base]{unique}} while checking
#' scalar missing values. Also see the example.
#'
#' @templateVar fn List
#' @template x
#' @inheritParams checkVector
#' @param types [\code{character}]\cr
#'  Character vector of class names. Each list element must inherit
#'  from at least one of the provided types.
#'  The types \dQuote{logical}, \dQuote{integer}, \dQuote{integerish}, \dQuote{double},
#'  \dQuote{numeric}, \dQuote{complex}, \dQuote{character}, \dQuote{factor}, \dQuote{atomic}, \dQuote{vector}
#'  \dQuote{atomicvector}, \dQuote{array}, \dQuote{matrix}, \dQuote{list}, \dQuote{function},
#'  \dQuote{environment} and \dQuote{null} are supported.
#'  For other types \code{\link[base]{inherits}} is used as a fallback to check \code{x}'s inheritance.
#'  Defaults to \code{character(0)} (no check).
#' @template null.ok
#' @template checker
#' @family basetypes
#' @export
#' @useDynLib checkmate c_check_list
#' @examples
#' testList(list())
#' testList(as.list(iris), types = c("numeric", "factor"))
#'
#' # Missingness
#' testList(list(1, NA), any.missing = FALSE)
#' testList(list(1, NULL), any.missing = FALSE)
#'
#' # Uniqueness differentiates between different NA types:
#' testList(list(NA, NA), unique = TRUE)
#' testList(list(NA, NA_real_), unique = TRUE)
checkList = function(x, types = character(0L), any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, names = NULL, null.ok = FALSE) {
  .Call(c_check_list, x, any.missing, all.missing, len, min.len, max.len, unique, names, null.ok) %and%
  checkListTypes(x, types)
}

checkListTypes = function(x, types = character(0L)) {
  if (length(x) == 0L || length(types) == 0L)
    return(TRUE)
  qassert(types, "S")

  ind = seq_along(x)
  for (type in types) {
    f = checkmate$listtypefuns[[type]] %??% function(x) inherits(x, type)
    ind = ind[!vapply(x[ind], f, FUN.VALUE = NA, USE.NAMES = FALSE)]
    if (length(ind) == 0L)
      return(TRUE)
  }
  sprintf("May only contain the following types: {%s}, but element %i has type '%s'", paste0(types, collapse = ","), ind[1L], paste0(class(x[[ind[1L]]]), collapse = ","))
}

#' @export
#' @rdname checkList
check_list = checkList

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkList
assertList = makeAssertionFunction(checkList, use.namespace = FALSE)

#' @export
#' @rdname checkList
assert_list = assertList

#' @export
#' @include makeTest.R
#' @rdname checkList
testList = makeTestFunction(checkList)

#' @export
#' @rdname checkList
test_list = testList

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkList
expect_list = makeExpectationFunction(checkList, use.namespace = FALSE)

#' @section Check scalars:
#' \itemize{
#'   \item{\code{\link{checkFlag}}}
#'   \item{\code{\link{checkCount}}}
#'   \item{\code{\link{checkNumber}}}
#'   \item{\code{\link{checkInt}}}
#'   \item{\code{\link{checkString}}}
#'   \item{\code{\link{checkScalar}}}
#'   \item{\code{\link{checkScalarNA}}}
#' }
#'
#' @section Check vectors:
#' \itemize{
#'   \item{\code{\link{checkLogical}}}
#'   \item{\code{\link{checkNumeric}}}
#'   \item{\code{\link{checkDouble}}}
#'   \item{\code{\link{checkInteger}}}
#'   \item{\code{\link{checkIntegerish}}}
#'   \item{\code{\link{checkCharacter}}}
#'   \item{\code{\link{checkComplex}}}
#'   \item{\code{\link{checkFactor}}}
#'   \item{\code{\link{checkList}}}
#'   \item{\code{\link{checkPOSIXct}}}
#'   \item{\code{\link{checkVector}}}
#'   \item{\code{\link{checkAtomic}}}
#'   \item{\code{\link{checkAtomicVector}}}
#'   \item{\code{\link{checkRaw}}}
#' }
#'
#' @section Check attributes:
#' \itemize{
#'   \item{\code{\link{checkClass}}}
#'   \item{\code{\link{checkMultiClass}}}
#'   \item{\code{\link{checkNames}}}
#'   \item{\code{\link{checkNamed}}} (deprecated)
#' }
#'
#' @section Check compound types:
#' \itemize{
#'   \item{\code{\link{checkArray}}}
#'   \item{\code{\link{checkDataFrame}}}
#'   \item{\code{\link{checkMatrix}}}
#' }
#'
#' @section Check other built-in R types:
#' \itemize{
#'   \item{\code{\link{checkDate}}}
#'   \item{\code{\link{checkEnvironment}}}
#'   \item{\code{\link{checkFunction}}}
#'   \item{\code{\link{checkFormula}}}
#'   \item{\code{\link{checkNull}}}
#' }
#'
#' @section Check sets:
#' \itemize{
#'   \item{\code{\link{checkChoice}}}
#'   \item{\code{\link{checkSubset}}}
#'   \item{\code{\link{checkSetEqual}}}
#'   \item{\code{\link{checkDisjunct}}}
#'   \item{\code{\link{checkPermutation}}}
#' }
#'
#' @section File IO:
#' \itemize{
#'   \item{\code{\link{checkFileExists}}}
#'   \item{\code{\link{checkDirectoryExists}}}
#'   \item{\code{\link{checkPathForOutput}}}
#'   \item{\code{\link{checkAccess}}}
#' }
#'
#' @section Popular data types of third party packages:
#' \itemize{
#'   \item{\code{\link{checkDataTable}}}
#'   \item{\code{\link{checkR6}}}
#'   \item{\code{\link{checkTibble}}}
#' }
#'
#' @section Safe coercion to integer:
#' \itemize{
#'   \item{\code{\link{asCount}}}
#'   \item{\code{\link{asInt}}}
#'   \item{\code{\link{asInteger}}}
#' }
#'
#' @section Quick argument checks using a DSL:
#' \itemize{
#'   \item{\code{\link{qassert}}}
#'   \item{\code{\link{qassertr}}}
#' }
#'
#' @section Misc:
#' \itemize{
#'   \item{\code{\link{checkOS}} (check operating system)}
#'   \item{\code{\link{assert}} (combine multiple checks into an assertion)}
#'   \item{\code{\link{anyMissing}}}
#'   \item{\code{\link{allMissing}}}
#'   \item{\code{\link{anyNaN}}}
#'   \item{\code{\link{wf}} (which.first and which.last)}
#' }
#'
#' @importFrom utils head tail packageVersion getFromNamespace
#' @importFrom stats setNames
"_PACKAGE"

checkmate = new.env(parent = emptyenv())
checkmate$os = c("windows", "mac", "linux", "solaris")[match(tolower(Sys.info()["sysname"]), c("windows", "darwin", "linux", "sunos"))]
checkmate$listtypefuns = list2env(list(
  "logical"      = is.logical,
  "integer"      = is.integer,
  "integerish"   = isIntegerish,
  "double"       = is.double,
  "numeric"      = is.numeric,
  "complex"      = is.complex,
  "character"    = is.character,
  "factor"       = is.factor,
  "atomic"       = is.atomic,
  "vector"       = is.vector,
  "atomicvector" = function(x) !is.null(x) && is.atomic(x),
  "array"        = is.array,
  "matrix"       = is.matrix,
  "function"     = is.function,
  "environment"  = is.environment,
  "list"         = is.list,
  "null"         = is.null
))


register_tinytest = function() {
  ns = getNamespace("checkmate")
  expectations = names(ns)[grepl("^expect_", names(ns))]
  tinytest::register_tinytest_extension("checkmate", expectations)
}

.onLoad = function(libpath, pkgname) {
  backports::import(pkgname)
  if ("tinytest" %in% loadedNamespaces())
    register_tinytest()
  setHook(packageEvent("tinytest", "onLoad"), function(...) register_tinytest(), action = "append")
}

.onUnload = function (libpath) {
  library.dynam.unload("checkmate", libpath) # nocov
}

#' Combine multiple checks into one assertion
#'
#' @description
#' You can call this function with an arbitrary number of of \code{check*}
#' functions, i.e. functions provided by this package or your own functions which
#' return \code{TRUE} on success and the error message as \code{character(1)} otherwise.
#' The resulting assertion is successful, if \code{combine} is
#' \dQuote{or} (default) and at least one check evaluates to \code{TRUE} or
#' \code{combine} is \dQuote{and} and all checks evaluate to \code{TRUE}.
#' Otherwise, \code{assert} throws an informative error message.
#'
#' @param ... [any]\cr
#'  List of calls to check functions.
#' @param combine [\code{character(1)}]\cr
#'  \dQuote{or} or \dQuote{and} to combine the check functions with an OR
#'  or AND, respectively.
#' @template assert
#' @return Throws an error (or pushes the error message to an 
#'   \code{\link{AssertCollection}} if \code{add} is not \code{NULL}) 
#'   if the checks fail and invisibly returns \code{TRUE} otherwise.
#' @export
#' @examples
#' x = 1:10
#' assert(checkNull(x), checkInteger(x, any.missing = FALSE))
#' collection <- makeAssertCollection()
#' assert(checkChoice(x, c("a", "b")), checkDataFrame(x), add = collection)
#' collection$getMessages()
#' 
assert = function(..., combine = "or", .var.name = NULL, add = NULL) {
  assertChoice(combine, c("or", "and"))
  assertClass(add, "AssertCollection", .var.name = "add", null.ok = TRUE)
  dots = match.call(expand.dots = FALSE)$...
  assertCharacter(.var.name, null.ok = TRUE, min.len = 1L, max.len = length(dots))
  env = parent.frame()
  if (combine == "or") {
    msgs = character(length(dots))
    for (i in seq_along(dots)) {
      val = eval(dots[[i]], envir = env)
      if (isTRUE(val))
        return(invisible(TRUE))
      msgs[i] = as.character(val)
    }
    if (is.null(.var.name))
      .var.name = vapply(dots, function(x) as.character(x)[2L], FUN.VALUE = NA_character_)
    if (length(msgs) > 1L) {
      msgs = sprintf("%s(%s): %s", vapply(dots, function(x) as.character(x)[1L], FUN.VALUE = NA_character_), .var.name, msgs)
      msgs = paste0(c("One of the following must apply:", strwrap(msgs, prefix = " * ")), collapse = "\n")
    }
    mstopOrPush(res = msgs, v_name = .var.name, collection = add)
  } else {
    for (i in seq_along(dots)) {
      val = eval(dots[[i]], envir = env)
      if (!isTRUE(val)) {
        if (is.null(.var.name))
          .var.name = as.character(dots[[i]])[2L]
        mstopOrPush(res = val, v_name = .var.name, collection = add)
      }
    }
  }
  invisible(TRUE)
}

# Error handling in assert()
# 
# Internal helper function to handle errors in assert().
# @param res [character(1)}]\cr
#   error message
# @param v_name [\code{character}]\cr
#   Name(s) of the variable(s) whose assertion failed.
# @param collection [\code{AssertCollection} | \code{NULL}]\cr
#   See AssertCollection.
# @return mstopOrPush() throws an exception by calling 
#   mstop() if 'collection' is NULL, or 
#   pushes the error message to the collection otherwise.
# @keywords internal
mstopOrPush = function(res, v_name, collection = NULL) {
  if (!is.null(collection)) {
    v_name = sort(unique(v_name))
    prefix =
      if (length(v_name) > 1L) {
        sprintf(
          "Variables %s",
          paste0(shQuote(v_name), collapse = ", ")
        )
      } else {
        sprintf("Variable '%s'", v_name)
      }
    collection$push(sprintf("%s: %s.", prefix, res))
  } else if (length(v_name) > 1L) {
    mstop("Assertion failed. %s", res)
  } else {
    mstop("Assertion on '%s' failed: %s.", v_name, res)
  }
}

#' @rdname anyMissing
#' @useDynLib checkmate c_all_missing
#' @export
#' @examples
#' allMissing(1:2)
#' allMissing(c(1, NA))
#' allMissing(c(NA, NA))
#' x = data.frame(a = 1:2, b = NA)
#' # Note how allMissing combines the results for data frames:
#' allMissing(x)
#' all(sapply(x, allMissing))
allMissing = function(x) {
  .Call(c_all_missing, x)
}

#' Check if an argument is a single atomic value
#'
#' @templateVar fn Scalar
#' @template x
#' @template na-handling
#' @template na.ok
#' @template null.ok
#' @template checker
#' @family scalars
#' @useDynLib checkmate c_check_scalar
#' @export
#' @examples
#' testScalar(1)
#' testScalar(1:10)
checkScalar = function(x, na.ok = FALSE, null.ok = FALSE) {
  .Call(c_check_scalar, x, na.ok, null.ok)
}

#' @export
#' @rdname checkScalar
check_scalar = checkScalar

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkScalar
assertScalar = makeAssertionFunction(checkScalar, c.fun = "c_check_scalar", use.namespace = FALSE)

#' @export
#' @rdname checkScalar
assert_scalar = assertScalar

#' @export
#' @include makeTest.R
#' @rdname checkScalar
testScalar = makeTestFunction(checkScalar, c.fun = "c_check_scalar")

#' @export
#' @rdname checkScalar
test_scalar = testScalar

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkScalar
expect_scalar = makeExpectationFunction(checkScalar, c.fun = "c_check_scalar", use.namespace = FALSE)

#' Check if an object contains infinite values
#'
#' @description
#' Supported are atomic types (see \code{\link[base]{is.atomic}}), lists and data frames.
#'
#' @param x [\code{ANY}]\cr
#'  Object to check.
#' @return [\code{logical(1)}] Returns \code{TRUE} if any element is \code{-Inf} or \code{Inf}.
#' @useDynLib checkmate c_any_infinite
#' @export
#' @examples
#' anyInfinite(1:10)
#' anyInfinite(c(1:10, Inf))
#' iris[3, 3] = Inf
#' anyInfinite(iris)
anyInfinite = function(x) {
  .Call(c_any_infinite, x)
}

context("anyInfinite")

test_that("anyInfinite", {
  xb = logical(10)
  xi = integer(10)
  xd = double(10)
  xc = complex(10)
  xl = as.list(1:10)
  xm = matrix(1:9, 3)
  xf = data.frame(a=1:5, b=1:5)

  expect_false(anyInfinite(NULL))
  expect_false(anyInfinite(mean))
  expect_false(anyInfinite(double(0)))

  expect_false(anyInfinite(xb))
  expect_false(anyInfinite(xi))
  expect_false(anyInfinite(xd))
  expect_false(anyInfinite(xc))
  expect_false(anyInfinite(xl))
  expect_false(anyInfinite(xm))
  expect_false(anyInfinite(xf))

  xd[5] = xc[5] = xf$b[3] = Inf
  xl[5] = -Inf
  expect_true(anyInfinite(xd))
  expect_true(anyInfinite(xc))
  expect_true(anyInfinite(xl))
  expect_true(anyInfinite(xf))

  x = list(1, list(1, list(1, Inf)))
  expect_true(anyInfinite(x))
})

#' Check if the arguments are permutations of each other.
#'
#' @description
#' In contrast to \code{\link{checkSetEqual}}, the function tests for a true
#' permutation of the two vectors and also considers duplicated values.
#' Missing values are being treated as actual values by default.
#' Does not work on raw values.
#'
#' @templateVar fn Permutation
#' @template x
#' @param y [\code{atomic}]\cr
#'  Vector to compare with. Atomic vector of type other than raw.
#' @param na.ok [\code{logical(1)}]\cr
#'  Are missing values allowed? Default is \code{TRUE}.
#' @template checker
#' @template set
#' @family set
#' @export
#' @examples
#' testPermutation(letters[1:2], letters[2:1])
#' testPermutation(letters[c(1, 1, 2)], letters[1:2])
#' testPermutation(c(NA, 1, 2), c(1, 2, NA))
#' testPermutation(c(NA, 1, 2), c(1, 2, NA), na.ok = FALSE)
checkPermutation = function(x, y, na.ok = TRUE) {
  qassert(x, "a")
  qassert(y, "a")

  if (is.raw(x) || is.raw(y)) {
    stop("Cannot check permutation on raw vectors")
  }

  # These are the cheap checks that we perform separately
  if (!isSameType(x, y) || length(x) != length(y)) {
    return(sprintf("Must be permutation of %s, but is %s", array_collapse(y), array_collapse(x)))
  }

  if (!na.ok && (anyNA(x) || anyNA(y))) {
    return("The parameter na.ok is set to FALSE but NAs were found.")
  }
  # This drops NAs
  xs = sort(x)
  ys = sort(y)

  # We handle NAs and the remaining non-NAs differently:
  # * NA: If na.ok is TRUE, the two vector must have the same number of NAs, this is checked by
  #       comparing the lengths, because the NAs are dropped in the sort above.
  # * non-NA: We sort the vector and check for equality (without NAs)

  if (length(xs) != length(ys) || any(xs != ys)) {
    return(sprintf("Must be permutation of %s, but is %s", array_collapse(ys), array_collapse(xs)))
  }
  return(TRUE)
}

#' @export
#' @rdname checkPermutation
check_permutation = checkPermutation

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkPermutation
assertPermutation = makeAssertionFunction(checkPermutation, use.namespace = FALSE)

#' @export
#' @rdname checkPermutation
assert_permutation = assertPermutation

#' @export
#' @include makeTest.R
#' @rdname checkPermutation
testPermutation = makeTestFunction(checkPermutation)

#' @export
#' @rdname checkPermutation
test_permutation = testPermutation

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkPermutation
expect_permutation = makeExpectationFunction(checkPermutation, use.namespace = FALSE)

#' Check if an object contains NaN values
#'
#' @description
#' Supported are atomic types (see \code{\link[base]{is.atomic}}), lists and data frames.
#'
#' @param x [\code{ANY}]\cr
#'  Object to check.
#' @return [\code{logical(1)}] Returns \code{TRUE} if any element is \code{NaN}.
#' @useDynLib checkmate c_any_nan
#' @export
#' @examples
#' anyNaN(1:10)
#' anyNaN(c(1:10, NaN))
#' iris[3, 3] = NaN
#' anyNaN(iris)
anyNaN = function(x) {
  .Call(c_any_nan, x)
}

#' @title Check if a path is suited for creating an output file
#'
#' @description
#' Check if a file path can be used safely to create a file and write to it.
#'
#' This is checked:
#' \itemize{
#'  \item{Does \code{dirname(x)} exist?}
#'  \item{Does no file under path \code{x} exist?}
#'  \item{Is \code{dirname(x)} writable?}
#' }
#' Paths are relative to the current working directory.
#'
#' @templateVar fn PathForOutput
#' @template x
#' @param overwrite [\code{logical(1)}]\cr
#'  If \code{TRUE}, an existing file in place is allowed if it
#'  it is both readable and writable.
#'  Default is \code{FALSE}.
#' @param extension [\code{character(1)}]\cr
#'  Extension of the file, e.g. \dQuote{txt} or \dQuote{tar.gz}.
#' @template checker
#' @family filesystem
#' @export
#' @examples
#' # Can we create a file in the tempdir?
#' testPathForOutput(file.path(tempdir(), "process.log"))
checkPathForOutput = function(x, overwrite = FALSE, extension = NULL) {
  if (!qtest(x, "S+"))
    return("No path provided")
  qassert(overwrite, "B1")

  x = normalizePath(x, mustWork = FALSE)
  dn = dirname(x)

  w = wf(!dir.exists(dn))
  if (length(w) > 0L)
    return(sprintf("Path to file (dirname) does not exist: '%s' of '%s'", dn[w], x[w]))

  w = which(file.exists(x))
  if (length(w) > 0L) {
    if (overwrite)
      return(checkAccess(dn, "w") %and% checkAccess(x[w], "rw"))
    return(sprintf("File at path already exists: '%s'", x[w]))
  }

  if (!is.null(extension)) {
    qassert(extension, "S1")
    if (!endsWith(x, paste0(".", extension)))
      return(sprintf("File must have extension '.%s'", extension))
  }
  return(checkAccess(dn, "w"))
}

#' @export
#' @rdname checkPathForOutput
check_path_for_output = checkPathForOutput

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkPathForOutput
assertPathForOutput = makeAssertionFunction(checkPathForOutput, use.namespace = FALSE)

#' @export
#' @rdname checkPathForOutput
assert_path_for_output = assertPathForOutput

#' @export
#' @include makeTest.R
#' @rdname checkPathForOutput
testPathForOutput = makeTestFunction(checkPathForOutput)

#' @export
#' @rdname checkPathForOutput
test_path_for_output = testPathForOutput

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkPathForOutput
expect_path_for_output = makeExpectationFunction(checkPathForOutput, use.namespace = FALSE)

context("checkFile")

td = tempfile("checkFile")
dir.create(td, recursive=TRUE)
fn = file.path(td, "myfile.ext")
dn = file.path(td, "dir")
ff = file.path(td, "xxx")
file.create(fn)
dir.create(dn)

test_that("checkFile", {
  myobj = fn
  expect_succ_all(FileExists, myobj)
  myobj = ff
  expect_fail_all(FileExists, myobj)

  expect_false(testFileExists(character(0)))
  expect_false(testFileExists(NULL))
  expect_false(testFileExists(dn))

  expect_error(assertFileExists(character(0)), "provided")
  expect_error(assertFileExists(ff), "exist")
  expect_error(assertFileExists(dn))

  expect_succ_all(FileExists, fn, extension = "ext")
  expect_succ_all(FileExists, fn, extension = c("foo", "ext"))
  expect_fail_all(FileExists, fn, extension = "foo")
})

test_that("check_directory", {
  myobj = dn
  expect_succ_all(DirectoryExists, myobj)
  myobj = ff
  expect_fail_all(DirectoryExists, myobj)

  expect_false(testDirectoryExists(character(0)))
  expect_false(testDirectoryExists(fn))

  expect_error(assertDirectoryExists(character(0)), "provided")
  expect_error(assertDirectoryExists(ff), "exist")
  expect_error(assertDirectoryExists(fn))
})

test_that("check_access", {
  myobj = R.home()
  expect_succ_all(Access, myobj, "r")

  if (.Platform$OS.type != "windows") {
    Sys.chmod(fn, "0000")
    expect_true(testAccess(fn, ""))
    expect_false(testAccess(fn, "x"))
    if (Sys.info()["user"] == "root") {
      expect_true(testAccess(fn, "r"))
      expect_true(testAccess(fn, "w"))
    } else {
      expect_false(testAccess(fn, "r"))
      expect_false(testAccess(fn, "w"))
    }

    Sys.chmod(fn, "0700")
    expect_true(testAccess(fn, ""))
    expect_true(testAccess(fn, "r"))
    expect_true(testAccess(fn, "w"))
    expect_true(testAccess(fn, "x"))
    Sys.chmod(fn, "0600")
    expect_true(testAccess(fn, ""))
    expect_true(testAccess(fn, "r"))
    expect_true(testAccess(fn, "rw"))
    expect_false(testAccess(fn, "rx"))
    expect_false(testAccess(fn, "wx"))

    expect_error(testAccess(fn, "a"))
    expect_error(testAccess(fn, "rrr"))
  }
})

test_that("check_path_for_output", {
  myobj = ff
  expect_succ_all(PathForOutput, myobj)
  myobj = fn
  expect_fail_all(PathForOutput, myobj)

  expect_false(testPathForOutput(character(0)))
  expect_false(testPathForOutput(NULL))

  expect_error(assertPathForOutput(character(0)), "path provided")
  expect_identical(assertPathForOutput(c("a", "b")), c("a", "b"))
  expect_identical(assertPathForOutput(ff), ff)
  expect_error(assertPathForOutput(fn), "exist")
  expect_identical(assertPathForOutput(fn, overwrite = TRUE), fn)
  expect_true(testPathForOutput(c(fn, ff, dn), overwrite = TRUE))
  expect_false(testPathForOutput(c(fn, ff, dn), overwrite = FALSE))

  expect_true(checkPathForOutput("a.txt", extension = "txt"))
  expect_error(assertPathForOutput("a.R", extension = "txt"), "extension")
})

test_that("#195", {
  x = tempfile(fileext = ".csv.gz")
  file.create(x)
  expect_true(test_file_exists(x, extension = ".gz"))
  expect_true(test_file_exists(x, extension = ".csv.gz"))
})

#' Check the class membership of an argument
#'
#' @templateVar fn MultiClass
#' @template x
#' @param classes [\code{character}]\cr
#'  Class names to check for inheritance with \code{\link[base]{inherits}}.
#'  \code{x} must inherit from any of the specified classes.
#' @template null.ok
#' @template checker
#' @family attributes
#' @family classes
#' @export
#' @examples
#' x = 1
#' class(x) = "bar"
#' checkMultiClass(x, c("foo", "bar"))
#' checkMultiClass(x, c("foo", "foobar"))
checkMultiClass = function(x, classes, null.ok = FALSE) {
  qassert(classes, "S+")
  qassert(null.ok, "B1")
  if (is.null(x) && null.ok)
    return(TRUE)
  if (!inherits(x, classes)) {
    cl = class(x)
    return(sprintf("Must inherit from class '%s', but has class%s '%s'",
        paste0(classes, collapse = "'/'"), if (length(cl) > 1L) "es" else "", paste0(cl, collapse = "','")))
  }
  return(TRUE)
}

#' @export
#' @rdname checkMultiClass
check_multi_class = checkMultiClass

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkMultiClass
assertMultiClass = makeAssertionFunction(checkMultiClass, use.namespace = FALSE)

#' @export
#' @rdname checkMultiClass
assert_multi_class = assertMultiClass

#' @export
#' @include makeTest.R
#' @rdname checkMultiClass
testMultiClass = makeTestFunction(checkMultiClass)

#' @export
#' @rdname checkMultiClass
test_multi_class = testMultiClass

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkMultiClass
expect_multi_class = makeExpectationFunction(checkMultiClass, use.namespace = FALSE)

#' @title Turn a Check into a Test
#'
#' @description
#' \code{makeTest} is the internal function used to evaluate the result of a
#' check and throw an exception if necessary.
#' This function is currently only a stub and just calls \code{\link[base]{isTRUE}}.
#' \code{makeTestFunction} can be used to automatically create an assertion
#' function based on a check function (see example).
#'
#' @param res [\code{TRUE} | \code{character(1)}]\cr
#'  The result of a check function: \code{TRUE} for successful checks,
#'  and an error message as string otherwise.
#' @return \code{makeTest} returns \code{TRUE} if the check is successful and \code{FALSE} otherwise.
#'  \code{makeTestFunction} returns a \code{function}.
#' @export
#' @family CustomConstructors
#' @include helper.R
#' @examples
#' # Simple custom check function
#' checkFalse = function(x) if (!identical(x, FALSE)) "Must be FALSE" else TRUE
#'
#' # Create the respective test function
#' testFalse = function(x) {
#'   res = checkFalse(x)
#'   makeTest(res)
#' }
#'
#' # Alternative: Automatically create such a function
#' testFalse = makeTestFunction(checkFalse)
#' print(testFalse)
makeTest = function(res) {
  isTRUE(res)
}

#' @rdname makeTest
#' @template makeFunction
#' @export
makeTestFunction = function(check.fun, c.fun = NULL, env = parent.frame()) {
  fun.name = if (is.character(check.fun)) check.fun else deparse(substitute(check.fun))
  check.fun = match.fun(check.fun)
  fun.args = formals(args(check.fun))

  new.fun = function() TRUE
  formals(new.fun) = fun.args
  if (is.null(c.fun)) {
    body = paste0("isTRUE(", fun.name, "(", paste0(names(fun.args), collapse = ", "), "))")
  } else {
    body = paste0("isTRUE(.Call(", paste0(c(c.fun, names(fun.args)), collapse = ", "), "))")
  }

  body(new.fun) = parse(text = paste("{", body, "}"))
  environment(new.fun) = env
  return(new.fun)
}

context("makeXFunction")


test_that("makeAssertion", {
  x = assertFlag
  y = makeAssertionFunction(checkFlag, c.fun = "c_check_flag", use.namespace = FALSE)
  expect_identical(formals(x), formals(y))
  if (!isNamespaceLoaded("covr"))
    expect_equal(body(x), body(y))

  x = assertList
  y = makeAssertionFunction(checkList, use.namespace = FALSE)
  expect_identical(formals(x), formals(y))
  if (!isNamespaceLoaded("covr"))
    expect_equal(body(x), body(y))
})

test_that("makeTest", {
  x = testFlag
  y = makeTestFunction(checkFlag, c.fun = "c_check_flag")
  expect_identical(formals(x), formals(y))
  if (!isNamespaceLoaded("covr"))
    expect_equal(body(x), body(y))

  x = testList
  y = makeTestFunction(checkList)
  expect_identical(formals(x), formals(y))
  if (!isNamespaceLoaded("covr"))
    expect_equal(body(x), body(y))

  x = testFlag
  y = function(x) makeTest(checkFlag(x))
  expect_equal(x(TRUE), y(TRUE))
  expect_equal(x(FALSE), y(FALSE))
})

test_that("makeExpectation", {
  x = expect_flag
  y = makeExpectationFunction(checkFlag, c.fun = "c_check_flag", use.namespace = FALSE)
  expect_identical(formals(x), formals(y))
  if (!isNamespaceLoaded("covr"))
    expect_equal(body(x), body(y))

  x = expect_list
  y = makeExpectationFunction(checkList, use.namespace = FALSE)
  expect_identical(formals(x), formals(y))
  if (!isNamespaceLoaded("covr"))
    expect_equal(body(x), body(y))
})

test_that("makeX with name for 'x' not 'x'", {
  checker = function(foo, bar = TRUE) check_numeric(foo)

  achecker = makeAssertionFunction(checker)
  expect_identical(names(formals(achecker)), c("foo", "bar", ".var.name", "add"))
  expect_identical(as.character(formals(achecker)$.var.name)[2], "foo")
  expect_equal(sum(grepl("foo", as.character(body(achecker)))), 3L)
  expect_equal(sum(grepl("bar", as.character(body(achecker)))), 1L)

  tchecker = makeTestFunction(checker)
  expect_identical(names(formals(tchecker)), c("foo", "bar"))
  expect_equal(sum(grepl("foo", as.character(body(tchecker)))), 1L)
  expect_equal(sum(grepl("bar", as.character(body(tchecker)))), 1L)

  echecker = makeExpectationFunction(checker)
  expect_identical(names(formals(echecker)), c("foo", "bar", "info", "label"))
  expect_equal(sum(grepl("foo", as.character(body(echecker)))), 3L)
  expect_equal(sum(grepl("bar", as.character(body(echecker)))), 1L)
})

#' Check if an argument is disjunct from a given set
#'
#' @templateVar fn Disjunct
#' @template x
#' @param y [\code{atomic}]\cr
#'  Other Set.
#' @template fmatch
#' @template checker
#' @template set
#' @family set
#' @export
#' @examples
#' testDisjunct(1L, letters)
#' testDisjunct(c("a", "z"), letters)
#'
#' # x is not converted before the comparison (except for numerics)
#' testDisjunct(factor("a"), "a")
#' testDisjunct(1, "1")
#' testDisjunct(1, as.integer(1))
checkDisjunct = function(x, y, fmatch = FALSE) {
  qassert(x, "a")
  qassert(y, "a")

  if (isTRUE(fmatch) && requireNamespace("fastmatch", quietly = TRUE))
    match = fastmatch::fmatch

  check_disjunct_internal(x, y, match)
}

#' @export
#' @rdname checkDisjunct
check_disjunct = checkDisjunct

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkDisjunct
assertDisjunct = makeAssertionFunction(checkDisjunct, use.namespace = FALSE)

#' @export
#' @rdname checkDisjunct
assert_disjunct = assertDisjunct

#' @export
#' @include makeTest.R
#' @rdname checkDisjunct
testDisjunct = makeTestFunction(checkDisjunct)

#' @export
#' @rdname checkDisjunct
test_disjunct = testDisjunct

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkDisjunct
expect_disjunct = makeExpectationFunction(checkDisjunct, use.namespace = FALSE)

#' @title Check that an argument is a Date
#'
#' @description
#' Checks that an object is of class \code{\link[base]{Date}}.
#'
#' @templateVar fn Atomic
#' @template x
#' @param lower [\code{\link[base]{Date}}]\cr
#'  All non-missing dates in \code{x} must be >= this date. Comparison is done via \code{\link[base]{Ops.Date}}.
#' @param upper [\code{\link[base]{Date}}]\cr
#'  All non-missing dates in \code{x} must be before <= this date. Comparison is done via \code{\link[base]{Ops.Date}}.
#' @template null.ok
#' @inheritParams checkVector
#' @template checker
#' @family basetypes
#' @export
checkDate = function(x, lower = NULL, upper = NULL, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, null.ok = FALSE) {
  qassert(null.ok, "B1")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be of class 'Date', not 'NULL'")
  }
  if (!inherits(x, "Date"))
    return(sprintf("Must be of class 'Date'%s, not '%s'", if (null.ok) " (or 'NULL')" else "", guessType(x)))
  checkInteger(as.integer(x), any.missing = any.missing, all.missing = all.missing, len = len, min.len = min.len, max.len = max.len, unique = unique) %and%
    checkDateBounds(x, lower, upper)
}

checkDateBounds = function(x, lower, upper) {
  if (!is.null(lower)) {
    lower = as.Date(lower, origin = "1970-01-01")
    if (length(lower) != 1L || is.na(lower))
      stop("Argument 'lower' must be a single (non-missing) date")
    if (any(x[!is.na(x)] < lower))
      return(sprintf("Date must be >= %s", lower))
  }

  if (!is.null(upper)) {
    upper = as.Date(upper, origin = "1970-01-01")
    if (length(upper) != 1L || is.na(upper))
      stop("Argument 'upper' must be a single (non-missing) date")
    if (any(x[!is.na(x)] > upper))
      return(sprintf("Date must be <= %s", upper))
  }
  return(TRUE)
}

#' @export
#' @rdname checkDate
check_date = checkDate

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkDate
assertDate = makeAssertionFunction(checkDate, use.namespace = FALSE)

#' @export
#' @rdname checkDate
assert_date = assertDate

#' @export
#' @include makeTest.R
#' @rdname checkDate
testDate = makeTestFunction(checkDate)

#' @export
#' @rdname checkDate
test_date = testDate

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkDate
expect_date = makeExpectationFunction(checkDate, use.namespace = FALSE)

#' Check if an object is an element of a given set
#'
#' @templateVar fn Choice
#' @template x
#' @param choices [\code{atomic}]\cr
#'  Set of possible values.
#' @template null.ok
#' @template fmatch
#' @template checker
#' @template set
#' @family set
#' @export
#' @examples
#' testChoice("x", letters)
#'
#' # x is not converted before the comparison (except for numerics)
#' testChoice(factor("a"), "a")
#' testChoice(1, "1")
#' testChoice(1, as.integer(1))
checkChoice = function(x, choices, null.ok = FALSE, fmatch = FALSE) {
  qassert(null.ok, "B1")

  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    qassert(choices, "a")
    return(sprintf("Must be a subset of %s, not 'NULL'", set_collapse(choices)))
  }

  qassert(choices, "a")
  if (!qtest(x, "a1"))
    return(sprintf("Must be element of set %s, but is not atomic scalar", set_collapse(choices)))
  if (!isSameType(x, choices) && !allMissing(x))
    return(sprintf("Must be element of set %s, but types do not match (%s != %s)", set_collapse(choices), class(x)[1L], class(choices)[1L]))

  if (isTRUE(fmatch) && requireNamespace("fastmatch", quietly = TRUE))
    match = fastmatch::fmatch

  if (match(x, choices, 0L) == 0L)
    return(sprintf("Must be element of set %s, but is '%s'", set_collapse(choices), x))
  return(TRUE)
}

#' @export
#' @rdname checkChoice
check_choice = checkChoice

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkChoice
assertChoice = makeAssertionFunction(checkChoice, use.namespace = FALSE)

#' @export
#' @rdname checkChoice
assert_choice = assertChoice

#' @export
#' @include makeTest.R
#' @rdname checkChoice
testChoice = makeTestFunction(checkChoice)

#' @export
#' @rdname checkChoice
test_choice = testChoice

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkChoice
expect_choice = makeExpectationFunction(checkChoice, use.namespace = FALSE)

context("checkChoice")

test_that("checkChoice", {
  myobj = 1
  expect_succ_all(Choice, myobj, 1:3)
  myobj = 0
  expect_fail_all(Choice, myobj, 1:3)

  expect_false(testChoice(character(0), letters))
  expect_false(testChoice(NULL, letters))
  expect_false(testChoice(1, NULL))
  expect_error(testChoice(list(1), as.list(iris)), "atomic")
  expect_false(testChoice(factor("a"), letters))
  expect_true(testChoice(factor("a"), factor(letters)))
  expect_true(testChoice(1., 1:2))

  expect_false(testChoice(NULL, NULL))
  expect_false(testChoice(NULL, letters, null.ok = FALSE))
  expect_true(checkChoice(NULL, letters, null.ok = TRUE))

  expect_true(testChoice(1L, 1:10))
  expect_false(testChoice("ab", letters))
  expect_false(testChoice(NA_integer_, 1:10))
  expect_false(testChoice(1:2, 1:10))

  expect_error(assertChoice(-1, 1:2), "element of")
  expect_error(assertChoice(1L, list()), "atomic")


  expect_true(grepl("atomic scalar", checkChoice(1:2, 1:10), fixed = TRUE))
  expect_true(grepl("types do not match", checkChoice(factor("a"), letters), fixed = TRUE))
  expect_true(grepl("'foo'", checkChoice("foo", letters), fixed = TRUE))
})


test_that("checkChoice / fastmatch", {
  skip_if_not_installed("fastmatch")
  x = "c"
  y = letters[1:5]

  res = testChoice(x, y)
  expect_true(res)
  expect_null(attr(y, ".match.hash"))

  res = testChoice(x, y, fmatch = TRUE)
  expect_true(res)
  expect_class(attr(y, ".match.hash"), "match.hash")
})

test_that("checkChoice / NAs (#158)", {
  expect_true(test_choice(NA_character_, NA))
  expect_true(test_choice(NA, NA_character_))
})

#' @title Turn a Check into an Assertion
#'
#' @description
#' \code{makeAssertion} is the internal function used to evaluate the result of a
#' check and throw an exception if necessary.
#' \code{makeAssertionFunction} can be used to automatically create an assertion
#' function based on a check function (see example).
#'
#' @template x
#' @param res [\code{TRUE} | \code{character(1)}]\cr
#'  The result of a check function: \code{TRUE} for successful checks,
#'  and an error message as string otherwise.
#' @param var.name [\code{character(1)}]\cr
#'  The custom name for \code{x} as passed to any \code{assert*} function.
#'  Defaults to a heuristic name lookup.
#' @param collection [\code{\link{AssertCollection}}]\cr
#'  If an \code{\link{AssertCollection}} is provided, the error message is stored
#'  in it. If \code{NULL}, an exception is raised if \code{res} is not
#'  \code{TRUE}.
#' @return \code{makeAssertion} invisibly returns the checked object if the check was successful,
#'  and an exception is raised (or its message stored in the collection) otherwise.
#'  \code{makeAssertionFunction} returns a \code{function}.
#' @export
#' @family CustomConstructors
#' @include helper.R
#' @examples
#' # Simple custom check function
#' checkFalse = function(x) if (!identical(x, FALSE)) "Must be FALSE" else TRUE
#'
#' # Create the respective assert function
#' assertFalse = function(x, .var.name = vname(x), add = NULL) {
#'   res = checkFalse(x)
#'   makeAssertion(x, res, .var.name, add)
#' }
#'
#' # Alternative: Automatically create such a function
#' assertFalse = makeAssertionFunction(checkFalse)
#' print(assertFalse)
makeAssertion = function(x, res, var.name, collection) {
  if (!isTRUE(res)) {
    assertString(var.name, .var.name = ".var.name")

    if (is.null(collection)) {
      mstop("Assertion on '%s' failed: %s.", var.name, res, call. = sys.call(-2L))
    }
    assertClass(collection, "AssertCollection", .var.name = "add")
    collection$push(sprintf("Variable '%s': %s.", var.name, res))
  }
  return(invisible(x))
}

#' @rdname makeAssertion
#' @template makeFunction
#' @template use.namespace
#' @param coerce [\code{logical(1)}]\cr
#'  If \code{TRUE}, injects some lines of code to convert numeric values to integer after an successful assertion.
#'  Currently used in \code{\link{assertCount}}, \code{\link{assertInt}} and \code{\link{assertIntegerish}}.
#' @export
makeAssertionFunction = function(check.fun, c.fun = NULL, use.namespace = TRUE, coerce = FALSE, env = parent.frame()) {
  fun.name = if (is.character(check.fun)) check.fun else deparse(substitute(check.fun))
  check.fun = match.fun(check.fun)
  check.args = fun.args = formals(args(check.fun))
  x.name = names(fun.args[1L])
  new.fun = function() TRUE

  body = sprintf("if (missing(%s)) stop(sprintf(\"argument \\\"%%s\\\" is missing, with no default\", .var.name))", x.name)

  if (is.null(c.fun)) {
    body = paste0(body, sprintf("; res = %s(%s)", fun.name, paste0(names(check.args), collapse = ", ")))
  } else {
    body = paste0(body, sprintf("; res = .Call(%s)", paste0(c(c.fun, names(check.args)), collapse = ", ")))
  }

  if (coerce) {
    fun.args = c(fun.args, alist(coerce = FALSE))
  }

  if (use.namespace) {
    fun.args = c(fun.args,  list(.var.name = bquote(checkmate::vname(.(as.name(x.name)))), add = NULL))
    body = paste0(body, "; checkmate::makeAssertion")
  } else {
    fun.args = c(fun.args, list(.var.name = bquote(vname(.(as.name(x.name)))), add = NULL))
    body = paste0(body, "; makeAssertion")
  }
  body = paste0(body, sprintf("(%s, res, .var.name, add)", x.name))

  if (coerce) {
    body = paste0(body, "; if (isTRUE(coerce) && is.double(x)) x = setNames(as.integer(round(x, 0L)), names(x)); invisible(x)")
  }

  formals(new.fun) = fun.args
  body(new.fun) = parse(text = paste("{", body, "}"))
  environment(new.fun) = env
  return(new.fun)
}

#' Check if an argument is a string
#'
#' @description
#' A string is defined as a scalar character vector.
#' To check for vectors of arbitrary length, see \code{\link{checkCharacter}}.
#'
#' @templateVar fn String
#' @template x
#' @template na-handling
#' @template na.ok
#' @inheritParams checkCharacter
#' @template null.ok
#' @template checker
#' @family scalars
#' @export
#' @useDynLib checkmate c_check_string
#' @examples
#' testString("a")
#' testString(letters)
checkString = function(x, na.ok = FALSE, n.chars = NULL, min.chars = NULL, max.chars = NULL, pattern = NULL, fixed = NULL, ignore.case = FALSE, null.ok = FALSE) {
  .Call(c_check_string, x, na.ok, n.chars, min.chars, max.chars, null.ok) %and%
  checkCharacterPattern(x, pattern, fixed, ignore.case)
}

#' @export
#' @rdname checkString
check_string = checkString

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkString
assertString = makeAssertionFunction(checkString, use.namespace = FALSE)

#' @export
#' @rdname checkString
assert_string = assertString

#' @export
#' @include makeTest.R
#' @rdname checkString
testString = makeTestFunction(checkString)

#' @export
#' @rdname checkString
test_string = testString

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkString
expect_string = makeExpectationFunction(checkString, use.namespace = FALSE)

#' @title Quick argument checks on (builtin) R types
#'
#' @description
#' The provided functions parse rules which allow to express some of the most
#' frequent argument checks by typing just a few letters.
#'
#' @param x [any]\cr
#'  Object the check.
#' @param rules [\code{character}]\cr
#'   Set of rules. See details.
#' @template var.name
#' @return
#'  \code{qassert} throws an \code{R} exception if object \code{x} does
#'  not comply to at least one of the \code{rules} and returns the tested object invisibly
#'  otherwise.
#'  \code{qtest} behaves the same way but returns \code{FALSE} if none of the
#'  \code{rules} comply.
#'  \code{qexpect} is intended to be inside the unit test framework \code{\link[testthat]{testthat}} and
#'  returns an \code{\link[testthat]{expectation}}.
#'
#' @details
#' The rule is specified in up to three parts.
#' \enumerate{
#'  \item{
#'    Class and missingness check.
#'    The first letter is an abbreviation for the class. If it is
#'    provided uppercase, missing values are prohibited.
#'    Supported abbreviations:
#'    \tabular{rl}{
#'      \code{[bB]} \tab Bool / logical.\cr
#'      \code{[iI]} \tab Integer.\cr
#'      \code{[xX]} \tab Integerish (numeric convertible to integer, see \code{\link{checkIntegerish}}).\cr
#'      \code{[rR]} \tab Real / double.\cr
#'      \code{[cC]} \tab Complex.\cr
#'      \code{[nN]} \tab Numeric (integer or double).\cr
#'      \code{[sS]} \tab String / character.\cr
#'      \code{[fF]} \tab Factor\cr
#'      \code{[aA]} \tab Atomic.\cr
#'      \code{[vV]} \tab Atomic vector (see \code{\link{checkAtomicVector}}).\cr
#'      \code{[lL]} \tab List. Missingness is defined as \code{NULL} element.\cr
#'      \code{[mM]} \tab Matrix.\cr
#'      \code{[dD]} \tab Data.frame. Missingness is checked recursively on columns.\cr
#'      \code{[pP]} \tab POSIXct date.\cr
#'      \code{[e]}  \tab Environment.\cr
#'      \code{[0]}  \tab \code{NULL}.\cr
#'      \code{[*]}  \tab placeholder to allow any type.
#'    }
#'    Note that the check for missingness does not distinguish between
#'    \code{NaN} and \code{NA}. Infinite values are not treated as missing, but
#'    can be caught using boundary checks (part 3).
#'    }
#'  \item{
#'    Length definition. This can be one of
#'    \tabular{rl}{
#'      \code{[*]} \tab any length,\cr
#'      \code{[?]} \tab length of zero or one,\cr
#'      \code{[+]} \tab length of at least one, or\cr
#'      \code{[0-9]+} \tab exact length specified as integer.
#'    }
#'    Preceding the exact length with one of the comparison operators \code{=}/\code{==},
#'    \code{<}, \code{<=}, \code{>=} or \code{>} is also supported.
#'  }
#'  \item{
#'    Range check as two numbers separated by a comma, enclosed by square brackets
#'    (endpoint included) or parentheses (endpoint excluded).
#'    For example, \dQuote{[0, 3)} results in \code{all(x >= 0 & x < 3)}.
#'    The lower and upper bound may be omitted which is the equivalent of a negative or
#'    positive infinite bound, respectively.
#'    By definition \code{[0,]} contains \code{Inf}, while \code{[0,)} does not.
#'    The same holds for the left (lower) boundary and \code{-Inf}.
#'    E.g., the rule \dQuote{N1()} checks for a single finite numeric which is not NA,
#'    while \dQuote{N1[)} allows \code{-Inf}.
#'  }
#' }
#' @note
#' The functions are inspired by the blog post of Bogumił Kamiński:
#' \url{http://rsnippets.blogspot.de/2013/06/testing-function-agruments-in-gnu-r.html}.
#' The implementation is mostly written in C to minimize the overhead.
#' @seealso \code{\link{qtestr}} and \code{\link{qassertr}} for efficient checks
#' of list elements and data frame columns.
#' @useDynLib checkmate c_qassert
#' @export
#' @examples
#' # logical of length 1
#' qtest(NA, "b1")
#'
#' # logical of length 1, NA not allowed
#' qtest(NA, "B1")
#'
#' # logical of length 0 or 1, NA not allowed
#' qtest(TRUE, "B?")
#'
#' # numeric with length > 0
#' qtest(runif(10), "n+")
#'
#' # integer with length > 0, NAs not allowed, all integers >= 0 and < Inf
#' qtest(1:3, "I+[0,)")
#'
#' # either an emtpy list or a character vector with <=5 elements
#' qtest(1, c("l0", "s<=5"))
#'
#' # data frame with at least one column and no missing value in any column
#' qtest(iris, "D+")
qassert = function(x, rules, .var.name = vname(x)) {
  res = .Call(c_qassert, x, rules, FALSE)
  if (!isTRUE(res))
    mstop(qmsg(res, .var.name), call. = sys.call(-1L))
  invisible(x)
}

#' @useDynLib checkmate c_qtest
#' @rdname qassert
#' @export
qtest = function(x, rules) {
  .Call(c_qtest, x, rules, FALSE, 1L)
}

#' @useDynLib checkmate c_qassert
#' @template expect
#' @rdname qassert
#' @include makeExpectation.R
#' @export
qexpect = function(x, rules, info = NULL, label = vname(x)) {
  res = .Call(c_qassert, x, rules, FALSE)
  if (!isTRUE(res))
    res = qmsg(res, label)
  makeExpectation(x, res, info = info, label = label)
}

qmsg = function(msg, vname) {
  if (length(msg) > 1L)
    msg = paste0(c("One of the following must apply:", strwrap(msg, prefix = " * ")), collapse = "\n")
  sprintf("Assertion on '%s' failed. %s.", vname, msg)
}

## ----include=FALSE------------------------------------------------------------
library(checkmate)

## -----------------------------------------------------------------------------
fact <- function(n, method = "stirling") {
  if (length(n) != 1)
    stop("Argument 'n' must have length 1")
  if (!is.numeric(n))
    stop("Argument 'n' must be numeric")
  if (is.na(n))
    stop("Argument 'n' may not be NA")
  if (is.double(n)) {
    if (is.nan(n))
      stop("Argument 'n' may not be NaN")
    if (is.infinite(n))
      stop("Argument 'n' must be finite")
    if (abs(n - round(n, 0)) > sqrt(.Machine$double.eps))
      stop("Argument 'n' must be an integerish value")
    n <- as.integer(n)
  }
  if (n < 0)
    stop("Argument 'n' must be >= 0")
  if (length(method) != 1)
    stop("Argument 'method' must have length 1")
  if (!is.character(method) || !method %in% c("stirling", "factorial"))
    stop("Argument 'method' must be either 'stirling' or 'factorial'")

  if (method == "factorial")
    factorial(n)
  else
    sqrt(2 * pi * n) * (n / exp(1))^n
}

## -----------------------------------------------------------------------------
fact <- function(n, method = "stirling") {
  assertCount(n)
  assertChoice(method, c("stirling", "factorial"))

  if (method == "factorial")
    factorial(n)
  else
    sqrt(2 * pi * n) * (n / exp(1))^n
}

## -----------------------------------------------------------------------------
f <- function(x) {
  assert(
    checkClass(x, "foo"),
    checkClass(x, "bar")
  )
}

## ----eval=FALSE---------------------------------------------------------------
# # file: tests/test-all.R
# library(testthat)
# library(checkmate) # for testthat extensions
# test_check("mypkg")

## ----eval=FALSE---------------------------------------------------------------
# test_that("checkmate is a sweet extension for testthat", {
#   x = runif(100)
#   expect_numeric(x, len = 100, any.missing = FALSE, lower = 0, upper = 1)
#   # or, equivalent, using the lazy style:
#   qexpect(x, "N100[0,1]")
# })

## ----fig.width=6,fig.height=4,dependson="init",eval=requireNamespace("microbenchmark", quietly = TRUE)----
library(checkmate)
library(ggplot2)
library(microbenchmark)

x = TRUE
r = function(x, na.ok = FALSE) { stopifnot(is.logical(x), length(x) == 1, na.ok || !is.na(x)) }
cm = function(x) assertFlag(x)
cmq = function(x) qassert(x, "B1")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)

## ----fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)----
x = runif(1000)
r = function(x) stopifnot(is.numeric(x), length(x) == 1000, all(!is.na(x) & x >= 0 & x <= 1))
cm = function(x) assertNumeric(x, len = 1000, any.missing = FALSE, lower = 0, upper = 1)
cmq = function(x) qassert(x, "N1000[0,1]")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)

## ----fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)----
x = sample(letters, 10000, replace = TRUE)
r = function(x) stopifnot(is.character(x), !any(is.na(x)), all(nchar(x) > 0))
cm = function(x) assertCharacter(x, any.missing = FALSE, min.chars = 1)
cmq = function(x) qassert(x, "S+[1,]")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)

## ----fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)----
N = 10000
x = data.frame(a = runif(N), b = sample(letters[1:5], N, replace = TRUE), c = sample(c(FALSE, TRUE), N, replace = TRUE))
r = function(x) is.data.frame(x) && !any(sapply(x, function(x) any(is.na(x))))
cm = function(x) testDataFrame(x, any.missing = FALSE)
cmq = function(x) qtest(x, "D")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)

# checkmate tries to stop as early as possible
x$a[1] = NA
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)

## ----fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)----
N = 10000
x.altrep = seq_len(N) # this is an ALTREP in R version >= 3.5.0
x.sexp = c(x.altrep)  # this is a regular SEXP OTOH
r = function(x) stopifnot(is.integer(x), !any(is.na(x)), !is.unsorted(x))
cm = function(x) assertInteger(x, any.missing = FALSE, sorted = TRUE)
mb = microbenchmark(r(x.sexp), cm(x.sexp), r(x.altrep), cm(x.altrep))
print(mb)
autoplot(mb)

## -----------------------------------------------------------------------------
checkSquareMatrix = function(x, mode = NULL) {
  # check functions must return TRUE on success
  # and a custom error message otherwise
  res = checkMatrix(x, mode = mode)
  if (!isTRUE(res))
    return(res)
  if (nrow(x) != ncol(x))
    return("Must be square")
  return(TRUE)
}

# a quick test:
X = matrix(1:9, nrow = 3)
checkSquareMatrix(X)
checkSquareMatrix(X, mode = "character")
checkSquareMatrix(X[1:2, ])

## -----------------------------------------------------------------------------
# For assertions:
assert_square_matrix = assertSquareMatrix = makeAssertionFunction(checkSquareMatrix)
print(assertSquareMatrix)

# For tests:
test_square_matrix = testSquareMatrix = makeTestFunction(checkSquareMatrix)
print(testSquareMatrix)

# For expectations:
expect_square_matrix = makeExpectationFunction(checkSquareMatrix)
print(expect_square_matrix)

## ----eval = FALSE, hilang = "c"-----------------------------------------------
# SEXP qassert(SEXP x, const char *rule, const char *name);
# Rboolean qtest(SEXP x, const char *rule);

## ----eval = FALSE, hilang = "c"-----------------------------------------------
# #include <checkmate.h>
# #include <checkmate_stub.c>

## -----------------------------------------------------------------------------
sessionInfo()

context("qtest")

xb = logical(10); xb[5] = NA
xi = integer(10); xi[5] = NA
xr = double(10); xr[5] = NA
xc = complex(10); xc[5] = NA
xl = as.list(1:10); xl[5] = list(NULL)
xm = matrix(1:9, 3); xm[2, 3] = NA
xd = data.frame(a=1:5, b=1:5); xd$b[3] = NA
xf = factor(letters[1:10]); xf[5] = NA
xe = new.env(); xe$foo = 1
xp = as.POSIXct(Sys.time()) + 1:10; xp[5] = NA

expect_succ_all = function(x, rules) {
  xn = deparse(substitute(x))
  expect_true(qtest(x, rules),
    info = sprintf("rules: %s", paste(rules, collapse=",")), label = xn)
  expect_identical(qassert(x, rules), x,
    info = sprintf("rules: %s", paste(rules, collapse=",")), label = xn)
  expect_expectation_successful(qexpect(x, rules),
    info = sprintf("rules: %s", paste(rules, collapse=",")), label = xn)
}

expect_fail_all = function(x, rules, pattern = NULL) {
  xn = deparse(substitute(x))
  expect_false(qtest(x, rules),
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
  expect_error(qassert(x, rules), regexp = pattern,
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
  expect_expectation_failed(qexpect(x, rules),
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
}

test_that("type and missingness", {
  expect_succ_all(xb, "b")
  expect_fail_all(xb, "B")
  expect_succ_all(xi, "i")
  expect_fail_all(xi, "I")
  expect_succ_all(xr, "r")
  expect_fail_all(xr, "R")
  expect_succ_all(xc, "c")
  expect_fail_all(xc, "C")
  expect_succ_all(xl, "l")
  expect_fail_all(xl, "L")
  expect_succ_all(xm, "m")
  expect_fail_all(xm, "M")
  expect_succ_all(xd, "d")
  expect_fail_all(xd, "D")
  expect_succ_all(xe, "e")
  expect_succ_all(xf, "f")
  expect_fail_all(xf, "F")
  expect_succ_all(xp, "p")
  expect_fail_all(xp, "P")

  expect_fail_all(xd, "b")
  expect_fail_all(xd, "i")
  expect_fail_all(xd, "r")
  expect_fail_all(xd, "c")
  expect_fail_all(xd, "l")
  expect_fail_all(xd, "m")
  expect_fail_all(xl, "e")
  expect_fail_all(xm, "r")
  expect_fail_all(xl, "d")
  expect_fail_all(xl, "f")
  expect_fail_all(xd, "p")

  expect_fail_all(xl, c("f", "n"), "One of")
  expect_error(qassert(1, "O"), "Unknown class identifier")
})

test_that("integerish", {
  expect_succ_all(xb, "x")
  expect_succ_all(xi, "x")
  expect_succ_all(xi, "x")
  expect_fail_all(xi, "X")
  expect_succ_all(xr, "x")
  expect_fail_all(xr, "X")
  expect_fail_all(1:3+.0001, "x")
  expect_fail_all(xd, "x")
})

test_that("length", {
  expect_succ_all(xb, "b+")
  expect_succ_all(xb, "b10")
  expect_succ_all(logical(1), "b+")
  expect_succ_all(logical(1), "b?")
  expect_succ_all(logical(1), "b1")
  expect_fail_all(xb, "b?")
  expect_fail_all(xb, "b5")
  expect_fail_all(xb, "b>=50")
  expect_succ_all(xb, "b<=50")
  expect_succ_all(xe, "e1")
  expect_fail_all(xe, "e>=2")
  expect_fail_all(xe, "f+")

  expect_error(qassert(1, "n9999999999999"), "handle length")
  expect_error(qassert(1, "n-1"), "negative length")
})

test_that("bounds", {
  xx = 1:3
  expect_succ_all(xx, "i+(0,4)")
  expect_succ_all(xx, "i+(0.9999,3.0001)")
  expect_succ_all(xx, "i+(0,1e2)")
  expect_succ_all(1, "n[0, 100]")

  expect_fail_all(xx, "i+[1,2)")
  expect_fail_all(xx, "i+[1,2]")
  expect_fail_all(xx, "i+[1,3)")
  expect_succ_all(xx, "i+[1,3]")

  expect_fail_all(xx, "i+(2,3]")
  expect_fail_all(xx, "i+[2,2]")
  expect_fail_all(xx, "i+(1,3)")
  expect_succ_all(xx, "i+[1,3]")

  expect_succ_all(xx, "i[1,)")
  expect_succ_all(xx, "i[,3]")
  expect_succ_all(Inf, "n(1,]")
  expect_succ_all(-Inf, "n[,1]")
  expect_succ_all(c(-Inf, 0, Inf), "n[,]")
  expect_fail_all(Inf, "n(1,)")
  expect_fail_all(-Inf, "n(,0]")
  expect_fail_all(c(-Inf, 0, Inf), "n(,]")
  expect_fail_all(c(-Inf, 0, Inf), "n(,)")

  xx = letters[1:3]
  expect_succ_all(xx, "s+[1,]")
  expect_succ_all(xx, "s+[1,1]")
  expect_fail_all(xx, "s+[2]")
  expect_fail_all(NA_character_, "s+[1]")
  expect_fail_all(NA, "s+[1]")

  xx = factor(letters[1:3])
  expect_succ_all(xx, "f+[1,]")
  expect_succ_all(xx, "f+[1,1]")
  expect_fail_all(xx, "f+[2]")
  expect_fail_all(NA_integer_, "f+[1]")
  expect_fail_all(NA_character_, "f+[1]")
  expect_fail_all(NA, "f+[1]")

  expect_succ_all(1, "n+()")
  expect_succ_all(1, "n+[]")
  expect_succ_all(Inf, "n+[]")
  expect_succ_all(Inf, "n+(]")
  expect_succ_all(-Inf, "n+[)")
  expect_fail_all(Inf, "n+()")
  expect_fail_all(Inf, "n+[)")
  expect_fail_all(-Inf, "n+(]")

  expect_error(qassert(iris, "d+[1]"), "Bound checks")
})

test_that("non-atomic types", {
  expect_succ_all(function() 1, "*")
  expect_fail_all(function() 1, "b")
  expect_succ_all(function() 1, "*")
  expect_succ_all(NULL, "0?")
  expect_fail_all(xi, "0")
  expect_fail_all(NULL, "0+")
  expect_succ_all(NULL, "00")
  expect_fail_all(xe, "b")
  expect_fail_all(xf, "b")
  expect_fail_all(as.symbol("x"), "n")
  expect_fail_all(xd, "a")
})

test_that("atomic types", {
  expect_succ_all(NULL, "a")
  expect_succ_all(xb, "a+")
  expect_fail_all(xb, "A+")
  expect_succ_all(xi, "a+")
  expect_fail_all(xi, "A+")
  expect_succ_all(xi, "n+")
  expect_fail_all(xi, "N+")
  expect_succ_all(xr, "n+")
  expect_fail_all(xr, "N+")
  expect_succ_all(xr, "a+")
  expect_fail_all(xr, "A+")
  expect_succ_all(xm, "a+")
  expect_fail_all(xm, "A+")
  expect_fail_all(xl, "a+")
  expect_fail_all(xl, "A+")
  expect_fail_all(xe, "a+")
  expect_succ_all(xf, "a+")

  expect_fail_all(NULL, "v")
  expect_succ_all(xb, "v+")
  expect_fail_all(xb, "V+")
  expect_succ_all(xi, "v+")
  expect_fail_all(xi, "V+")
  expect_succ_all(xr, "v+")
  expect_fail_all(xr, "V+")
  expect_fail_all(xm, "v+")
  expect_fail_all(xm, "V+")
  expect_fail_all(xl, "v+")
  expect_fail_all(xl, "V+")
  expect_fail_all(xe, "v+")
  expect_fail_all(xf, "V+")
})

test_that("optional chars", {
  expect_succ_all(TRUE, "b*")
  expect_succ_all(TRUE, "b=1")
  expect_succ_all(TRUE, "b>=0")
  expect_succ_all(TRUE, "b>0")
  expect_succ_all(TRUE, "b<2")
  expect_fail_all(TRUE, "b=2")
  expect_fail_all(TRUE, "b>=2")
  expect_fail_all(TRUE, "b>2")
  expect_fail_all(TRUE, "b<0")
})

test_that("malformated pattern", {
  expect_error(qassert(1, ""), "[Ee]mpty")
  # expect_warning(expect_error(qassert(1, "ä")), "locale")
  expect_error(qassert(1, "nn"), "length definition")
  expect_error(qassert(1, "n="), "length definition")
  expect_error(qassert(1, "n=="), "length definition")
  expect_error(qassert(1, "n==="), "length definition")
  expect_error(qassert(1, "n?1"), "bound definition")
  expect_error(qassert(1, "n>"))
  expect_error(qassert(1, "nö"))
  expect_error(qassert(1, "n\n"))
  expect_error(qassert(1, "n+a"), "opening")
  expect_error(qassert(1, "n+["), "bound")
  expect_error(qassert(1, "n+[1"), "lower")
  expect_error(qassert(1, "n+[x,]"), "lower")
  expect_error(qassert(1, "n+[,y]"), "upper")
  expect_error(qassert(1, "n*("), "bound definition")
  expect_error(qassert(1, "n*]"), "bound definition")
  expect_error(qassert(1, "n*(1)xx"), "Additional chars found")
  expect_error(qassert(1, TRUE), "be a string")
  expect_error(qassert(1, NA_character_), "not be NA")
  expect_error(qtest(1, TRUE), "be a string")
  expect_error(qtest(1, NA_character_), "not be NA")
})

test_that("we get some output", {
  expect_error(qassert(1, "b"), "logical")
  expect_error(qassert(1, "l"), "list")
  expect_error(qassert(1:2, "n?"), "length <=")
})

test_that("empty vectors", {
  expect_succ_all(integer(0), "i*")
  expect_succ_all(integer(0), "i*[0,0]")
  expect_succ_all(integer(0), "n[0,0]")
  expect_fail_all(integer(0), "r[0,0]")
  expect_fail_all(integer(0), "*+")
  expect_succ_all(TRUE, character(0))
})

test_that("logicals are not numeric", {
  expect_fail_all(TRUE, "i")
  expect_fail_all(TRUE, "I")
  expect_fail_all(TRUE, "n")
  expect_fail_all(TRUE, "N")
})

test_that("data frames are not lists", {
  expect_fail_all(iris, "l")
  expect_fail_all(iris, "L")
})

test_that("error messages are properly generated", {
  expect_error(qassert(1, "N22"), "== 22")
  expect_error(qassert(1:3, "N?"), "<= 1")
  expect_error(qassert(integer(0), "N+"), ">= 1")

  expect_error(qassert(1, "N[2,]"), ">= 2")
  expect_error(qassert(1, "N(2,]"), "> 2")
  expect_error(qassert(1, "N[,0]"), "<= 0")
  expect_error(qassert(1, "N[,0)"), "< 0")

  expect_error(qassert(Inf, "N[)"), "!= Inf")
  expect_error(qassert(-Inf, "N(]"), "!= -Inf")
})


test_that("multiple rules are correctly processed", {
  expect_true(qtest(1, c("M", "F", "C", "N")))
})

test_that("integer posix (#175)", {
  intSeq = seq(as.POSIXct("2000-01-01"), length.out = 31, by = "1 day")
  expect_true(qtest(intSeq, "P+"))
})

context("deparse")

test_that("deparse", {
  f = function(num, na.ok) {
    assertNumber(num)
    qassert(na.ok, "B1")
  }

  expect_true(f(1, TRUE))
  expect_error(f(NULL, TRUE), "num")
  expect_error(f(1, NULL), "na.ok")
})

#' Check if an argument is a single missing value
#'
#' @templateVar fn ScalarNA
#' @template x
#' @template null.ok
#' @template checker
#' @family scalars
#' @export
#' @examples
#' testScalarNA(1)
#' testScalarNA(NA_real_)
#' testScalarNA(rep(NA, 2))
checkScalarNA = function(x, null.ok = FALSE) {
  qassert(null.ok, "B1")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be a scalar missing value, not 'NULL'")
  }
  if (!is.atomic(x) || length(x) != 1L || !is.na(x))
    return(paste0("Must be a scalar missing value", if (null.ok) " (or 'NULL')" else ""))
  return(TRUE)
}

#' @export
#' @rdname checkScalarNA
check_scalar_na = checkScalarNA

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkScalarNA
assertScalarNA = makeAssertionFunction(checkScalarNA, use.namespace = FALSE)

#' @export
#' @rdname checkScalarNA
assert_scalar_na = assertScalarNA

#' @export
#' @include makeTest.R
#' @rdname checkScalarNA
testScalarNA = makeTestFunction(checkScalarNA)

#' @export
#' @rdname checkScalarNA
test_scalar_na = testScalarNA

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkScalarNA
expect_scalar_na = makeExpectationFunction(checkScalarNA, use.namespace = FALSE)

#' @title Coalesce operator
#' @rdname coalesce
#'
#' @description
#' Returns the left hand side if not missing nor \code{NULL}, and
#' the right hand side otherwise.
#'
#' @param lhs [any]\cr
#' Left hand side of the operator. Is returned if not missing or \code{NULL}.
#' @param rhs [any]\cr
#' Right hand side of the operator. Is returned if \code{lhs} is missing or \code{NULL}.
#' @return Either \code{lhs} or \code{rhs}.
#' @export
#' @examples
#' print(NULL %??% 1 %??% 2)
#' print(names(iris) %??% letters[seq_len(ncol(iris))])
"%??%" = function(lhs, rhs) {
  if (missing(lhs) || is.null(lhs)) rhs else lhs
}

#' Check if an argument is a single numeric value
#'
#' @templateVar fn Number
#' @template x
#' @template na-handling
#' @template na.ok
#' @template bounds
#' @param finite [\code{logical(1)}]\cr
#'  Check for only finite values? Default is \code{FALSE}.
#' @template null.ok
#' @template checker
#' @useDynLib checkmate c_check_number
#' @family scalars
#' @export
#' @examples
#' testNumber(1)
#' testNumber(1:2)
checkNumber = function(x, na.ok = FALSE, lower = -Inf, upper = Inf, finite = FALSE, null.ok = FALSE) {
  .Call(c_check_number, x, na.ok, lower, upper, finite, null.ok)
}

#' @export
#' @rdname checkNumber
check_number = checkNumber

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkNumber
assertNumber = makeAssertionFunction(checkNumber, c.fun = "c_check_number", use.namespace = FALSE)

#' @export
#' @rdname checkNumber
assert_number = assertNumber

#' @export
#' @include makeTest.R
#' @rdname checkNumber
testNumber = makeTestFunction(checkNumber, c.fun = "c_check_number")

#' @export
#' @rdname checkNumber
test_number = testNumber

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkNumber
expect_number = makeExpectationFunction(checkNumber, c.fun = "c_check_number", use.namespace = FALSE)

context("checkNumber")

test_that("checkNumber", {
  myobj = 1
  expect_succ_all(Number, myobj)
  myobj = "a"
  expect_fail_all(Number, myobj)

  expect_false(testNumber(integer(0)))
  expect_false(testNumber(NULL))

  expect_false(testNumber(TRUE))
  expect_false(testNumber(FALSE))
  expect_true(testNumber(1L))
  expect_true(testNumber(1.))
  expect_false(testNumber(NA))
  expect_false(testNumber(NaN))
  expect_true(testNumber(NaN, na.ok = TRUE))
  expect_true(testNumber(NA_real_, na.ok = TRUE))
  expect_false(testNumber(1:2))
  expect_false(testNumber(""))

  expect_true(testNumber(Inf))
  expect_true(testNumber(-Inf))
  expect_error(assertNumber(Inf, finite = TRUE), "finite")
  expect_error(assertNumber(-Inf, finite = TRUE), "finite")

  expect_false(testNumber(TRUE))

  expect_error(assertNumber(2+3i), "number")
})

test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkNumber(NA, na.ok = TRUE, lower = 1))
  expect_true(checkNumber(NA_character_, na.ok = TRUE, upper = 10))
  expect_fail_all(Number, 0, lower = 1)
  expect_fail_all(Number, 100, upper = 10)
})

#' Collect multiple assertions
#' @name AssertCollection
#'
#' @param collection [\code{AssertCollection}]\cr
#'  Object of type \dQuote{AssertCollection} (constructed via \code{makeAssertCollection}).
#' @description
#' The function \code{makeAssertCollection()} returns a simple stack-like
#' closure you can pass to all functions of the \code{assert*}-family.
#' All messages get collected and can be reported with \code{reportAssertions()}.
#' Alternatively, you can easily write your own report function or customize the the output of
#' the report function to a certain degree.
#' See the example on how to push custom messages or retrieve all stored messages.
#' @return \code{makeAssertCollection()} returns an object of class \dQuote{AssertCollection} and
#'  \code{reportCollection} returns invisibly \code{TRUE} if no error is thrown (i.e., no message was
#'  collected).
#' @examples
#' x = "a"
#' coll = makeAssertCollection()
#'
#' print(coll$isEmpty())
#' assertNumeric(x, add = coll)
#' coll$isEmpty()
#' coll$push("Custom error message")
#' coll$getMessages()
#' \dontrun{
#'   reportAssertions(coll)
#' }
NULL

#' @export
#' @rdname AssertCollection
makeAssertCollection = function() {
  msgs = character(0L)
  x = list(push = function(msg) msgs <<- c(msgs, msg), getMessages = function() msgs, isEmpty = function() length(msgs) == 0L)
  class(x) = "AssertCollection"
  x
}

#' @export
print.AssertCollection = function(x, ...) {
  n = length(x$getMessages())
  if (n == 0L) {
    cat("Empty collection\n")
  } else {
    cat(sprintf("Collection of %i assertion%s.\n", n, ifelse(n > 1L, "s", "")))
  }
}

#' @export
#' @rdname AssertCollection
reportAssertions = function(collection) {
  assertClass(collection, "AssertCollection")
  if (!collection$isEmpty()) {
    msgs = collection$getMessages()
    context = "%i assertions failed:"
    err = c(sprintf(context, length(msgs)), strwrap(msgs, prefix = " * "))
    stop(simpleError(paste0(err, collapse = "\n"), call = sys.call(1L)))
  }
  invisible(TRUE)
}

context("AssertCollection")

test_that("Collection closure wors", {
  coll = makeAssertCollection()
  expect_is(coll, "AssertCollection")
  expect_output(print(coll), "Empty collection")
  expect_equal(coll$getMessages(), character(0L))
  expect_true(coll$isEmpty())
  coll$push("testing")
  expect_equal(coll$getMessages(), "testing")
  expect_false(coll$isEmpty())
  expect_output(print(coll), "Collection of 1 assertion")
  coll$push("foo")
  expect_output(print(coll), "Collection of 2 assertions")
  expect_equal(coll$getMessages(), c("testing", "foo"))
})

test_that("Reporter works", {
  coll = makeAssertCollection()
  expect_true(reportAssertions(coll))
  coll$push("foo")
  coll$push("bar")
  expect_error(reportAssertions(coll), "foo")
  expect_error(reportAssertions(coll), "bar")
})

test_that("asserts push to collection", {
  coll = makeAssertCollection()
  findme = "a"

  assertString(findme, add = coll)
  expect_true(coll$isEmpty())
  expect_true(reportAssertions(coll))

  assertNumeric(findme, add = coll)
  expect_false(coll$isEmpty())
  expect_error(reportAssertions(coll), "findme")
})

mstop = function(msg, ..., call. = NULL) {
  stop(simpleError(sprintf(msg, ...), call.))
}

"%and%" = function(lhs, rhs) {
  if (isTRUE(lhs)) rhs else lhs
}

"%nin%" = function(x, y) {
  !match(x, y, nomatch = 0L)
}

convertCamelCase = function(x) {
  tolower(gsub("((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))", "_\\1", x, perl = TRUE))
}

#' @useDynLib checkmate c_guess_type
guessType = function(x) {
  .Call(c_guess_type, x)
}

isSameType = function(x, y) {
  identical(typeof(x), typeof(y)) || (is.numeric(x) && is.numeric(y))
}

array_collapse = function(x) {
  if (length(x) == 0L)
    return("[]")
  sprintf("['%s']", paste0(x, collapse = "','"))
}

capitalize = function(x) {
  substr(x, 1L, 1L) = toupper(substr(x, 1L, 1L))
  x
}

set_collapse = function(x) {
  if (length(x) == 0L)
    return("{}")
  sprintf("{'%s'}", paste0(unique(x), collapse = "','"))
}

set_msg = function(msg, what, ...) {
  if (is.null(what)) {
    sprintf(capitalize(msg), ...)
  } else {
    paste(capitalize(what), sprintf(msg, ...))
  }
}

check_subset_internal = function(x, choices, match, what = NULL) {
  qassert(choices, "a")
  if (length(choices) == 0L) {
    if (length(x) == 0L)
      return(TRUE)
    return(set_msg("must be a subset of the empty set, i.e. also empty", what))
  }

  if (!is.null(x)) {
    if (!isSameType(x, choices) && !allMissing(x)) {
      return(set_msg("must be a subset of %s, but has different type", what, set_collapse(choices)))
    }

    ii = match(x, choices)
    if (anyMissing(ii)) {
      return(set_msg(
        "must be a subset of %s, but has additional elements %s",
        what, set_collapse(choices), set_collapse(x[is.na(ii)])
      ))
    }
  }

  return(TRUE)
}

check_set_equal_internal = function(x, y, match, what = NULL) {
  if ((!isSameType(x, y) && !allMissing(x))) {
    return(set_msg("Must be setequal to %s, but has different type",
        what, set_collapse(y)))
  }

  ii = match(x, y)
  if (anyMissing(ii)) {
    return(set_msg("must be a permutation of set %s, but has extra elements %s",
        what, set_collapse(y), set_collapse(x[is.na(ii)])
    ))
  }

  ii = match(y, x)
  if (anyMissing(ii)) {
    return(set_msg("must be a set equal to %s, but is missing elements %s",
        what, set_collapse(y), set_collapse(y[is.na(ii)])
    ))
  }

  return(TRUE)
}

check_disjunct_internal = function(x, y, match, what = NULL) {
  if (length(x) == 0L || length(y) == 0L) {
    return(TRUE)
  }

  ii = match(x, y, 0L) > 0L
  if (any(ii)) {
    return(set_msg("must be disjunct from %s, but has elements %s",
      what,
      set_collapse(y),
      set_collapse(x[ii])
    ))
  }

  return(TRUE)
}

library(testthat)

expect_expectation_successful = function(expr, info = NULL, label = NULL) {
  res = tryCatch(expr, expectation = function(e) e)
  expect_is(res, "expectation_success", info = info, label = label)
}

expect_expectation_failed = function(expr, pattern = NULL, info = NULL, label = NULL) {
  x = tryCatch(expr, expectation = function(e) e)
  expect_is(x, "expectation_failure", info = info, label = label)
}

skip_if_not_physically_installed = function(x) {
  loc = find.package(x, quiet = TRUE)
  if (length(loc) == 0L)
    skip(sprintf("Package '%s' is not installed", x))
}

expect_succ_all = function(part, x, ..., cc = as.character(substitute(part)), lc = convertCamelCase(cc)) {
  xn = deparse(substitute(x))

  # check null.ok if it is in formals
  s = paste0("check", cc)
  fun = match.fun(s)
  if ("null.ok" %in% names(formals(args(fun)))) {
    dots = list(...)
    dots["x"] = list(NULL)
    dots$null.ok = TRUE
    expect_true(do.call(fun, dots))
  }

  s = paste0("check", cc)
  fun = match.fun(s)
  expect_true(fun(x, ...), label = s)

  s = paste0("check_", lc)
  fun = match.fun(s)
  expect_true(fun(x, ...), label = s)

  s = paste0("test", cc)
  fun = match.fun(s)
  expect_true(fun(x, ...), info = s, label = xn)

  s = paste0("test_", lc)
  fun = match.fun(s)
  expect_true(fun(x, ...), info = s, label = xn)

  s = paste0("assert", cc)
  fun = match.fun(s)
  expect_identical(fun(x, ...), x, info = s, label = xn)

  s = paste0("assert_", lc)
  fun = match.fun(s)
  expect_identical(fun(x, ...), x, info = s, label = xn)

  s = paste0("expect_", lc)
  fun = get0(s, envir = asNamespace("checkmate"), inherits = FALSE)
  if (!is.null(fun))
    expect_expectation_successful(fun(x, ...), info = s, label = xn)

  invisible(TRUE)
}

expect_fail_all = function(part, x, ..., cc = as.character(substitute(part)), lc = convertCamelCase(cc)) {
  xn = deparse(substitute(x))

  # check null.ok if it is in formals
  s = paste0("check", cc)
  fun = match.fun(s)
  if ("null.ok" %in% names(formals(args(fun)))) {
    dots = list(...)
    dots["x"] = list(NULL)
    dots$null.ok = FALSE
    expect_true(grepl("'NULL'", do.call(fun, dots), fixed = TRUE))
  }

  s = paste0("check", cc)
  fun = match.fun(s)
  res = fun(x, ...)
  expect_true(is.character(res) && nzchar(res), info = s, label = xn)

  s = paste0("test", cc)
  fun = match.fun(s)
  expect_false(fun(x, ...), info = s, label = xn)

  s = paste0("test_", lc)
  fun = match.fun(s)
  expect_false(fun(x, ...), info = s, label = xn)

  s = paste0("assert", cc)
  fun = match.fun(s)
  expect_error(fun(x, ..., .var.name = xn), xn, info = s, label = xn)
  expect_error(fun(x, ...), "'x'", info = s, label = xn)

  s = paste0("assert_", lc)
  fun = match.fun(s)
  expect_error(fun(x, ..., .var.name = xn), xn, info = s, label = xn)
  expect_error(fun(x, ...), "'x'", info = s, label = xn)

  s = paste0("expect_", lc)
  fun = get0(s, envir = asNamespace("checkmate"), inherits = FALSE)
  if (!is.null(fun)) {
    expect_expectation_failed(fun(x, ...), pattern = "x", info = s, label = xn)
    expect_expectation_failed(fun(x, ..., label = xn), pattern = xn, info = s, label = xn)
  }

  invisible(TRUE)
}

vlapply = function (x, fun, ..., use.names = TRUE) {
    vapply(X = x, FUN = fun, ..., FUN.VALUE = NA, USE.NAMES = use.names)
}

#' Check if an argument is an R6 class
#'
#' @templateVar fn Class
#' @template x
#' @inheritParams checkClass
#' @param cloneable [\code{logical(1)}]\cr
#'  If \code{TRUE}, check that \code{x} has a \code{clone} method. If \code{FALSE}, ensure that
#'  \code{x} is not cloneable.
#' @param public [\code{character}]\cr
#'  Names of expected public slots. This includes active bindings.
#' @param private [\code{character}]\cr
#'  Names of expected private slots.
#' @template null.ok
#' @template checker
#' @family classes
#' @export
#' @examples
#' library(R6)
#' generator = R6Class("Bar",
#'   public = list(a = 5),
#'   private = list(b = 42),
#'   active = list(c = function() 99)
#' )
#' x = generator$new()
#' checkR6(x, "Bar", cloneable = TRUE, public = "a")
checkR6 = function(x, classes = NULL, ordered = FALSE, cloneable = NULL, public = NULL, private = NULL, null.ok = FALSE) {
  if (!requireNamespace("R6", quietly = TRUE))
    stop("Install package 'R6' to perform checks of R6 classes")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be an R6 class, not 'NULL'")
  }
  if (!R6::is.R6(x))
    return(paste0("Must be an R6 class", if (null.ok) " (or 'NULL')" else "", sprintf(", not %s", guessType(x))))
  checkClass(x, c(classes, "R6"), ordered) %and% checkR6Props(x, cloneable, public, private)
}

checkR6Props = function(x, cloneable = NULL, public = NULL, private = NULL) {
  if (!is.null(cloneable)) {
    qassert(cloneable, "B1")
    if (cloneable) {
      if (!exists("clone", envir = x))
        return("Must be cloneable")
    } else {
      if (exists("clone", envir = x))
        return("May not be cloneable")
    }
  }

  if (!is.null(public)) {
    qassert(public, "S")
    i = wf(public %nin% ls(x, all.names = TRUE))
    if (length(i) > 0L)
      return(sprintf("Must provide the public slot '%s'", public[i]))
  }

  if (!is.null(private)) {
    qassert(private, "S")
    penv = x$.__enclos_env__[["private"]] %??% new.env()
    if (is.null(penv)) {
      i = if (length(private)) 1L else integer(0L)
    } else {
      i = wf(private %nin% ls(penv, all.names = TRUE))
    }
    if (length(i) > 0L)
      return(sprintf("Must provide the private slot '%s'", private[i]))
  }

  return(TRUE)
}

#' @export
#' @rdname checkR6
check_r6 = checkR6

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkR6
assertR6 = makeAssertionFunction(checkR6, use.namespace = FALSE)

#' @export
#' @rdname checkR6
assert_r6 = assertR6

#' @export
#' @include makeTest.R
#' @rdname checkR6
testR6 = makeTestFunction(checkR6)

#' @export
#' @rdname checkR6
test_r6 = testR6

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkR6
expect_r6 = makeExpectationFunction(checkR6, use.namespace = FALSE)

context("checkR6")

test_that("checkR6", {
  skip_if_not_physically_installed("R6")

  expect_false(testR6(1))
  expect_true("R6" %in% loadedNamespaces())

  x = R6::R6Class("Bar",
    public = list(a = 5),
    private = list(b = 42),
    active = list(c = function() 99)
    )$new()

  y = list(a = 5, b = 42)
  class(y) = "Bar"

  z = R6::R6Class("Bar", cloneable = FALSE)$new()

  expect_succ_all(R6, x)
  expect_fail_all(R6, y)

  expect_true(checkR6(NULL, null.ok = TRUE))
  expect_true(checkR6(x, "Bar", ordered = TRUE))
  expect_true(checkR6(x, cloneable = TRUE))
  expect_true(checkR6(z, cloneable = FALSE))
  expect_true(checkR6(x, public = character(0)))
  expect_true(checkR6(x, public = "a"))
  expect_true(checkR6(x, public = "c"))
  expect_true(checkR6(x, private = "b"))
  expect_true(checkR6(x, private = character(0)))

  expect_error(assertR6(NULL, null.ok = FALSE), "NULL")
  expect_error(assertR6(x, cloneable = FALSE), "cloneable")
  expect_error(assertR6(z, cloneable = TRUE), "cloneable")
  expect_error(assertR6(x, public = "b"), "public")
  expect_error(assertR6(x, private = "a"), "private")
  expect_error(assertR6(x, private = "c"), "private")
})

test_that("check contents of empty private", {
  skip_if_not_physically_installed("R6")

  x = R6::R6Class("Bar",
    public = list(a = 5),
    active = list(c = function() 99)
    )$new()

  expect_false(testR6(x, private = "b"))
  expect_true(testR6(x, private = character(0)))
})

#' @title Check that an argument is a date/time object in POSIXct format
#'
#' @description
#' Checks that an object is of class \code{\link[base]{POSIXct}}.
#'
#' @templateVar fn Atomic
#' @template x
#' @param lower [\code{\link[base]{Date}}]\cr
#'  All non-missing dates in \code{x} must be >= this POSIXct time. Must be provided in the same timezone as \code{x}.
#' @param upper [\code{\link[base]{Date}}]\cr
#'  All non-missing dates in \code{x} must be <= this POSIXct time. Must be provided in the same timezone as \code{x}.
#' @template sorted
#' @template null.ok
#' @inheritParams checkVector
#' @template checker
#' @family basetypes
#' @export
#' @useDynLib checkmate c_check_posixct
#' @export
checkPOSIXct = function(x, lower = NULL, upper = NULL, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, sorted = FALSE, null.ok = FALSE) {
  .Call(c_check_posixct, x, lower, upper, any.missing, all.missing, len, min.len, max.len, unique, sorted, null.ok)
}

#' @export
#' @rdname checkPOSIXct
check_posixct = checkPOSIXct

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkPOSIXct
assertPOSIXct = makeAssertionFunction(checkPOSIXct, c.fun = "c_check_posixct", use.namespace = FALSE)

#' @export
#' @rdname checkPOSIXct
assert_posixct = assertPOSIXct

#' @export
#' @include makeTest.R
#' @rdname checkPOSIXct
testPOSIXct = makeTestFunction(checkPOSIXct, c.fun = "c_check_posixct")

#' @export
#' @rdname checkPOSIXct
test_posixct = testPOSIXct

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkPOSIXct
expect_posixct = makeExpectationFunction(checkPOSIXct, c.fun = "c_check_posixct", use.namespace = FALSE)

context("checkPOSIXct")

test_that("checkPOSIXct", {
  origin = "1970-01-01"
  now = Sys.time()
  yesterday = now - 24 * 60 * 60
  tomorrow = now + 24 * 60 * 60
  now_est = as.POSIXct(as.numeric(now), tz = "EST", origin = origin)
  now_gmt = as.POSIXct(as.numeric(now), tz = "GMT", origin = origin)
  yesterday_gmt = as.POSIXct(as.numeric(now), tz = "GMT", origin = origin)
  tomorrow_gmt = as.POSIXct(as.numeric(now), tz = "GMT", origin = origin)

  expect_succ_all(POSIXct, now, lc = "posixct", cc = "POSIXct")
  expect_fail_all(POSIXct, 1, lc = "posixct", cc = "POSIXct")

  dates = c(yesterday, now, tomorrow, NA)
  expect_true(testPOSIXct(dates, min.len = 1, max.len = 10))
  expect_true(testPOSIXct(dates, len = 4))
  expect_true(testPOSIXct(dates, unique = TRUE))
  expect_true(testPOSIXct(dates, all.missing = FALSE))
  expect_true(testPOSIXct(dates, sorted = TRUE))
  expect_true(testPOSIXct(c(now, now), sorted = TRUE))
  expect_error(assertPOSIXct(c(dates, dates), unique = TRUE))
  expect_error(assertPOSIXct(dates, any.missing = FALSE), "missing")
  expect_error(assertPOSIXct(rev(dates), sorted = TRUE), "sorted")


  expect_true(testPOSIXct(dates, lower = yesterday))
  expect_true(checkPOSIXct(dates, upper = tomorrow))
  expect_error(assertPOSIXct(dates, lower = now), ">=")
  expect_error(assertPOSIXct(dates, upper = now), "<=")

  x = checkPOSIXct(dates, lower = now)
  expect_true(grepl("[0-9]{4}-[0-9]{2}-[0-9]{2}", x))

  x = checkPOSIXct(dates, upper = now)
  expect_true(grepl("[0-9]{4}-[0-9]{2}-[0-9]{2}", x))

  # timezone checks
  expect_error(assertPOSIXct(now_est, lower = yesterday), "Timezones")
  expect_error(assertPOSIXct(now_est, upper = tomorrow), "Timezones")
  expect_error(assertPOSIXct(now, lower = yesterday_gmt), "Timezones")
  expect_error(assertPOSIXct(now, upper = tomorrow_gmt), "Timezones")
  expect_error(assertPOSIXct(now_est, lower = yesterday_gmt), "Timezones")
  expect_error(assertPOSIXct(now_est, upper = tomorrow_gmt), "Timezones")
  expect_true(testPOSIXct(now_gmt, lower = yesterday_gmt, upper = tomorrow_gmt))
})

test_that("integer posix (#175)", {
  intSeq = seq(as.POSIXct("2000-01-01"), length.out = 31, by = "1 day")
  expect_true(test_posixct(intSeq, any.missing = FALSE, lower = intSeq[1], upper = intSeq[31]))
  expect_true(test_posixct(intSeq, any.missing = FALSE, lower = as.POSIXct("2000-01-01"), upper = as.POSIXct("2001-01-01")))
})

#' Check if an argument is FALSE
#'
#' @description
#' Simply checks if an argument is \code{FALSE}.
#'
#' @templateVar fn FALSE.
#' @template x
#' @template na.ok
#' @template checker
#' @export
#' @examples
#' testFALSE(FALSE)
#' testFALSE(TRUE)
checkFALSE = function(x, na.ok = FALSE) {
  qassert(na.ok, "B1")
  if (isFALSE(x) || (na.ok && length(x) == 1L && is.na(x)))
    return(TRUE)
  return("Must be FALSE")
}

#' @export
#' @rdname checkFALSE
check_false = checkFALSE

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkFALSE
assertFALSE = makeAssertionFunction(checkFALSE, use.namespace = FALSE)

#' @export
#' @rdname checkFALSE
assert_false = assertFALSE

#' @export
#' @include makeTest.R
#' @rdname checkFALSE
testFALSE = makeTestFunction(checkFALSE)

#' @export
#' @rdname checkFALSE
test_false = testFALSE

#' Check that an argument is an atomic vector
#'
#' @description
#' For the definition of \dQuote{atomic}, see \code{\link[base]{is.atomic}}.
#'
#' Note that `NULL` is recognized as a valid atomic value, as in R versions up to version 4.3.x.
#' For details, see \url{https://stat.ethz.ch/pipermail/r-devel/2023-September/082892.html}.
#'
#' @templateVar fn Atomic
#' @template x
#' @inheritParams checkVector
#' @template checker
#' @useDynLib checkmate c_check_atomic
#' @export
#' @family basetypes
#' @family atomicvector
#' @examples
#' testAtomic(letters, min.len = 1L, any.missing = FALSE)
checkAtomic = function(x, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, names = NULL) {
  .Call(c_check_atomic, x, any.missing, all.missing, len, min.len, max.len, unique, names)
}

#' @export
#' @rdname checkAtomic
check_atomic = checkAtomic

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkAtomic
assertAtomic = makeAssertionFunction(checkAtomic, c.fun = "c_check_atomic", use.namespace = FALSE)

#' @export
#' @rdname checkAtomic
assert_atomic = assertAtomic

#' @export
#' @include makeTest.R
#' @rdname checkAtomic
testAtomic = makeTestFunction(checkAtomic, c.fun = "c_check_atomic")

#' @export
#' @rdname checkAtomic
test_atomic = testAtomic

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkAtomic
expect_atomic = makeExpectationFunction(checkAtomic, c.fun = "c_check_atomic", use.namespace = FALSE)

context("checkAtomicVector")

test_that("checkAtomicVector", {
  myobj = 1:2
  expect_succ_all(AtomicVector, myobj)
  myobj = NULL
  expect_fail_all(AtomicVector, myobj)

  expect_true(testAtomicVector(integer(0)))
  expect_false(testAtomicVector(NULL))
  expect_true(testAtomicVector(1))
  expect_true(testAtomicVector(integer(0)))
  expect_true(testAtomicVector(factor(1)))

  expect_true(testAtomicVector(NA, any.missing = TRUE))
  expect_false(testAtomicVector(NA, any.missing = FALSE))
  expect_false(testAtomicVector(NA, all.missing = FALSE))

  expect_true(testAtomicVector(1, len=1))
  expect_false(testAtomicVector(1, len=0))

  expect_true(testAtomicVector(1, min.len=0))
  expect_false(testAtomicVector(1, min.len=2))
  expect_true(testAtomicVector(1, max.len=1))
  expect_false(testAtomicVector(1, max.len=0))

  expect_true(testAtomicVector(1, unique=TRUE))
  expect_false(testAtomicVector(1, min.len=2))
  expect_true(testAtomicVector(1, max.len=1))
  expect_false(testAtomicVector(1, max.len=0))

  expect_true(testAtomicVector(1, unique=TRUE))
  expect_true(testAtomicVector(c(1,1), unique=FALSE))
  expect_false(testAtomicVector(c(1,1), unique=TRUE))

  expect_true(testAtomicVector(1, names="unnamed"))
  expect_true(testAtomicVector(setNames(1, "x"), names="named"))
  expect_false(testAtomicVector(1, names="unique"))

  expect_error(assertAtomicVector(iris), "atomic")

  li = list(list = list(1, 2), factor = factor("a"), integer = 1:2, NULL = NULL, data.frame = iris, matrix = matrix(1:9))
  expected = setNames(c(FALSE, TRUE, TRUE, FALSE, FALSE, FALSE), names(li))
  expect_equal(expected, vlapply(li, testAtomicVector))
})

test_that("type guessing works (#48)", {
  x = structure(list(1:4, letters[1:3]), dim = c(2, 1))
  expect_match(checkAtomic(x), "list")
})

#' @title Get the index of the first/last TRUE
#'
#' @description
#' A quick C implementation for \dQuote{which.first} (\code{head(which(x), 1)}) and
#' \dQuote{which.last} (\code{tail(which(x), 1)}).
#'
#' @param x [\code{logical}]\cr
#'  Logical vector.
#' @param use.names [\code{logical(1)}]\cr
#'  If \code{TRUE} and \code{x} is named, the result is also
#'  named.
#' @return [\code{integer(1)} | \code{integer(0)}].
#'  Returns the index of the first/last \code{TRUE} value in \code{x} or
#'  an empty integer vector if none is found. NAs are ignored.
#' @useDynLib checkmate c_which_first
#' @export
#' @examples
#' wf(c(FALSE, TRUE))
#' wl(c(FALSE, FALSE))
#' wf(NA)
wf = function(x, use.names = TRUE) {
  .Call(c_which_first, x, use.names)
}

#' @rdname wf
#' @export
#' @useDynLib checkmate c_which_last
wl = function(x, use.names = TRUE) {
  .Call(c_which_last, x, use.names)
}

#' Check the class membership of an argument
#'
#' @templateVar fn Class
#' @template x
#' @param classes [\code{character}]\cr
#'  Class names to check for inheritance with \code{\link[base]{inherits}}.
#'  \code{x} must inherit from all specified classes.
#' @param ordered [\code{logical(1)}]\cr
#'  Expect \code{x} to be specialized in provided order.
#'  Default is \code{FALSE}.
#' @template null.ok
#' @template checker
#' @family attributes
#' @family classes
#' @export
#' @examples
#' # Create an object with classes "foo" and "bar"
#' x = 1
#' class(x) = c("foo", "bar")
#'
#' # is x of class "foo"?
#' testClass(x, "foo")
#'
#' # is x of class "foo" and "bar"?
#' testClass(x, c("foo", "bar"))
#'
#' # is x of class "foo" or "bar"?
#' \dontrun{
#' assert(
#'   checkClass(x, "foo"),
#'   checkClass(x, "bar")
#' )
#' }
#' # is x most specialized as "bar"?
#' testClass(x, "bar", ordered = TRUE)
checkClass = function(x, classes, ordered = FALSE, null.ok = FALSE) {
  qassert(classes, "S+")
  qassert(ordered, "B1")
  qassert(null.ok, "B1")
  if (is.null(x) && null.ok)
    return(TRUE)
  ord = inherits(x, classes, TRUE)
  w = wf(ord == 0L)

  if (length(w) > 0L) {
    cl = class(x)
    return(sprintf("Must inherit from class '%s', but has class%s '%s'",
        classes[w], if (length(cl) > 1L) "es" else "", paste0(cl, collapse = "','")))
  }
  if (ordered) {
    w = wf(ord != seq_along(ord))
    if (length(w) > 0L) {
      cl = class(x)
      return(sprintf("Must have class '%s' in position %i, but has class%s '%s'",
        classes[w], w, if (length(cl) > 1L) "es" else "", paste0(cl, collapse = "','")))
    }
  }
  return(TRUE)
}

#' @export
#' @rdname checkClass
check_class = checkClass

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkClass
assertClass = makeAssertionFunction(checkClass, use.namespace = FALSE)

#' @export
#' @rdname checkClass
assert_class = assertClass

#' @export
#' @include makeTest.R
#' @rdname checkClass
testClass = makeTestFunction(checkClass)

#' @export
#' @rdname checkClass
test_class = testClass

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkClass
expect_class = makeExpectationFunction(checkClass, use.namespace = FALSE)

#' Partial Argument Matching
#'
#' @description
#' This is an extensions to \code{\link[base]{match.arg}} with support for \code{\link{AssertCollection}}.
#' The behavior is very similar to \code{\link[base]{match.arg}}, except that \code{NULL} is not
#' a valid value for \code{x}.
#'
#' @param x [character]\cr
#'  User provided argument to match.
#' @param choices [character()]\cr
#'  Candidates to match \code{x} with.
#' @param several.ok [logical(1)]\cr
#'  If \code{TRUE}, multiple matches are allowed, cf. \code{\link[base]{match.arg}}.
#' @template add
#' @template var.name
#' @return Subset of \code{choices}.
#' @export
#' @examples
#' matchArg("k", choices = c("kendall", "pearson"))
matchArg = function(x, choices, several.ok = FALSE, .var.name = vname(x), add = NULL) {
  assertCharacter(choices, min.len = 1L)
  assertFlag(several.ok)

  if (several.ok) {
    if (identical(x, choices))
      return(x)
    assertCharacter(x, min.len = 1L, .var.name = .var.name, add = add)
    x = choices[pmatch(x, choices, nomatch = 0L, duplicates.ok = TRUE)]
    assertSubset(x, choices, empty.ok = FALSE, .var.name = .var.name, add = add)
  } else {
    if (identical(x, choices))
      return(x[1L])
    assertCharacter(x, len = 1L, .var.name = .var.name, add = add)
    x = choices[pmatch(x, choices, nomatch = 0L, duplicates.ok = FALSE)]
    assertChoice(x, choices, .var.name = .var.name, add = add)
  }
  x
}

context("matchArg")

test_that("matchArg", {
  x = c("pearson", "kendall", "spearman")
  choices = x
  expect_equal(matchArg(x, choices), choices[1])
  expect_equal(matchArg(x, choices, several.ok = TRUE), choices)

  x = substr(x, 1, 1)
  expect_equal(matchArg(x[2], choices, several.ok = FALSE), choices[2])
  expect_equal(matchArg(x[c(1, 3)], choices, several.ok = TRUE), choices[c(1, 3)])

  expect_error(matchArg(1, 1:10), "character")
  expect_error(matchArg(1, letters), "character")
  expect_error(matchArg(letters, 1:10), "character")
  expect_error(matchArg(x[1:2], choices), "length")
  expect_error(matchArg(x[0], choices), "length 0")
})

#' Check if an argument is a tibble
#'
#' @templateVar fn Tibble
#' @template x
#' @inheritParams checkMatrix
#' @inheritParams checkList
#' @inheritParams checkDataFrame
#' @template null.ok
#' @template checker
#' @family compound
#' @export
#' @examples
#' library(tibble)
#' x = as_tibble(iris)
#' testTibble(x)
#' testTibble(x, nrow = 150, any.missing = FALSE)
checkTibble = function(x, types = character(0L), any.missing = TRUE, all.missing = TRUE, min.rows = NULL, max.rows = NULL, min.cols = NULL, max.cols = NULL, nrows = NULL, ncols = NULL, row.names = NULL, col.names = NULL, null.ok = FALSE) {
  if (!requireNamespace("tibble", quietly = TRUE))
    stop("Install package 'tibble' to perform checks of tibbles")
  qassert(null.ok, "B1")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be a tibble, not 'NULL'")
  }
  if (!tibble::is_tibble(x))
    return(paste0("Must be a tibble", if (null.ok) " (or 'NULL')" else "", sprintf(", not %s", guessType(x))))
  checkDataFrame(x, types, any.missing, all.missing, min.rows, max.rows, min.cols, max.cols, nrows, ncols, row.names, col.names, null.ok)
}

#' @export
#' @rdname checkTibble
check_tibble = checkTibble

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkTibble
assertTibble = makeAssertionFunction(checkTibble, use.namespace = FALSE)

#' @export
#' @rdname checkTibble
assert_tibble = assertTibble

#' @export
#' @include makeTest.R
#' @rdname checkTibble
testTibble = makeTestFunction(checkTibble)

#' @export
#' @rdname checkTibble
test_tibble = testTibble

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkTibble
expect_tibble = makeExpectationFunction(checkTibble, use.namespace = FALSE)

#' Check if an argument is named
#'
#' @templateVar fn Named
#' @template x
#' @param type [character(1)]\cr
#'  Select the check(s) to perform.
#'  \dQuote{unnamed} checks \code{x} to be unnamed.
#'  \dQuote{named} (default) checks \code{x} to be named which excludes names to be \code{NA} or empty (\code{""}).
#'  \dQuote{unique} additionally tests for non-duplicated names.
#'  \dQuote{strict} checks for unique names which comply to R's variable name restrictions.
#'  Note that for zero-length \code{x} every name check evaluates to \code{TRUE}.
#' @template checker
#' @note
#' These function are deprecated and will be removed in a future version.
#' Please use \code{\link{checkNames}} instead.
#' @useDynLib checkmate c_check_named
#' @family attributes
#' @export
#' @examples
#' x = 1:3
#' testNamed(x, "unnamed")
#' names(x) = letters[1:3]
#' testNamed(x, "unique")
checkNamed = function(x, type = "named") {
  .Deprecated(new = "checkNames", old = "checkNamed", package = "checkmate")
  .Call(c_check_named, x, type)
}

#' @export
#' @rdname checkNamed
check_named = checkNamed

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkNamed
assertNamed = makeAssertionFunction(checkNamed, c.fun = "c_check_named", use.namespace = FALSE)

#' @export
#' @rdname checkNamed
assert_named = assertNamed

#' @export
#' @include makeTest.R
#' @rdname checkNamed
testNamed = makeTestFunction(checkNamed, c.fun = "c_check_named")

#' @export
#' @rdname checkNamed
test_named = testNamed

# This function is already provided by testthat
# #' @export
# #' @include makeExpectation.R
# #' @template expect
# #' @rdname checkNamed
expect_named = makeExpectationFunction(checkNamed, c.fun = "c_check_named", use.namespace = FALSE)

#' Check if an argument is a vector of type character
#'
#' @description
#' To check for scalar strings, see \code{\link{checkString}}.
#'
#' @templateVar fn Character
#' @template x
#' @template na-handling
#' @inheritParams checkVector
#' @param pattern [\code{character(1L)}]\cr
#'  Regular expression as used in \code{\link[base]{grepl}}.
#'  All non-missing elements of \code{x} must comply to this pattern.
#' @param fixed [\code{character(1)}]\cr
#'  Substring to detect in \code{x}. Will be used as \code{pattern} in \code{\link[base]{grepl}}
#'  with option \code{fixed} set to \code{TRUE}.
#'  All non-missing elements of \code{x} must contain this substring.
#' @param ignore.case [\code{logical(1)}]\cr
#'  See \code{\link[base]{grepl}}. Default is \code{FALSE}.
#' @param n.chars [\code{integer(1)}]\cr
#'  Exact number of characters for each element of \code{x}.
#' @param min.chars [\code{integer(1)}]\cr
#'  Minimum number of characters for each element of \code{x}.
#' @param max.chars [\code{integer(1)}]\cr
#'  Maximum number of characters for each element of \code{x}.
#' @template sorted
#' @template typed.missing
#' @template null.ok
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_character
#' @export
#' @examples
#' testCharacter(letters, min.len = 1, any.missing = FALSE)
#' testCharacter(letters, min.chars = 2)
#' testCharacter("example", pattern = "xa")
checkCharacter = function(x, n.chars = NULL, min.chars = NULL, max.chars = NULL, pattern = NULL, fixed = NULL, ignore.case = FALSE, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, sorted = FALSE, names = NULL, typed.missing = FALSE, null.ok = FALSE) {
  .Call(c_check_character, x, n.chars, min.chars, max.chars, any.missing, all.missing, len, min.len, max.len, unique, sorted, names, typed.missing, null.ok) %and%
  checkCharacterPattern(x, pattern, fixed, ignore.case)
}

checkCharacterPattern = function(x, pattern = NULL, fixed = NULL, ignore.case = FALSE) {
  if (!is.null(x)) {
    if (!is.null(pattern)) {
      qassert(pattern, "S1")
      ok = grepl(pattern, x[!is.na(x)], fixed = FALSE, ignore.case = ignore.case)
      if (!all(ok))
        return(sprintf("Must comply to pattern '%s'", pattern))
    }
    if (!is.null(fixed)) {
      qassert(fixed, "S1")
      ok = grepl(fixed, x[!is.na(x)], fixed = TRUE, ignore.case = ignore.case)
      if (!all(ok))
        return(sprintf("Must contain substring '%s'", fixed))
    }
  }
  return(TRUE)
}

#' @export
#' @rdname checkCharacter
check_character = checkCharacter

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkCharacter
assertCharacter = makeAssertionFunction(checkCharacter, use.namespace = FALSE)

#' @export
#' @rdname checkCharacter
assert_character = assertCharacter

#' @export
#' @include makeTest.R
#' @rdname checkCharacter
testCharacter = makeTestFunction(checkCharacter)

#' @export
#' @rdname checkCharacter
test_character = testCharacter

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkCharacter
expect_character = makeExpectationFunction(checkCharacter, use.namespace = FALSE)

context("checkArray")

test_that("checkArray", {
  myobj = array(1:2)
  expect_succ_all(Array, myobj)
  myobj = 1:2
  expect_fail_all(Array, myobj)

  x = array(dim = c(2, 3))
  expect_true(testArray(x))
  expect_true(testArray(x, d = 2L))
  expect_false(testArray(x, d = 1L))
  expect_true(testArray(x, min.d = 0L))
  expect_true(testArray(x, min.d = 1L))
  expect_true(testArray(x, max.d = 2L))
  expect_true(testArray(x, max.d = 3L))
  expect_false(testArray(x, min.d = 3L))
  expect_false(testArray(x, max.d = 1L))

  x[2,2] = NA
  expect_true(testMatrix(x))
  expect_false(testMatrix(x, any.missing = FALSE))
  expect_false(testArray(x, any.missing = FALSE))
  expect_error(assertArray(iris))

  x = array(1:27, dim = c(3, 3, 3))
  expect_true(testArray(x, mode = "integer"))
  expect_true(testArray(x, mode = "numeric"))
  expect_true(testArray(x, mode = "atomic"))
  expect_false(testArray(x, mode = "double"))
  expect_false(testArray(x, mode = "character"))
  expect_false(testArray(x, mode = "list"))

  x = array(list(1, 1), dim = c(1, 2))
  expect_true(testArray(x))
  expect_true(testArray(x, mode = "list"))
  expect_false(testArray(x, mode = "atomic"))
  expect_false(testArray(x, mode = "numeric"))

  expect_error(assertArray(1:3), "array")
})

test_that("type guessing works", {
  x = array(1:4)
  expect_match(checkCharacter(x), "array")

  x = array(1:4, dim = c(2, 2))
  expect_match(checkCharacter(x), "matrix")

  x = array(1:9, dim = c(3, 3, 3))
  expect_match(checkCharacter(x), "array")
})

#' Check if an argument is a function
#'
#' @templateVar fn Function
#' @template x
#' @param args [\code{character}]\cr
#'  Expected formal arguments. Checks that a function has no arguments if
#'  set to \code{character(0)}.
#'  Default is \code{NULL} (no check).
#' @param ordered [\code{logical(1)}]\cr
#'  Flag whether the arguments provided in \code{args} must be the first
#'  \code{length(args)} arguments of the function in the specified order.
#'  Default is \code{FALSE}.
#' @param nargs [\code{integer(1)}]\cr
#'  Required number of arguments, without \code{...}.
#'  Default is \code{NULL} (no check).
#' @template null.ok
#' @template checker
#' @family basetypes
#' @export
#' @examples
#' testFunction(mean)
#' testFunction(mean, args = "x")
checkFunction = function(x, args = NULL, ordered = FALSE, nargs = NULL, null.ok = FALSE) {
  qassert(null.ok, "B1")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be a function, not 'NULL'")
  }

  if (!is.function(x))
    return(sprintf("Must be a function%s, not '%s'", if (null.ok) " (or 'NULL')" else "", guessType(x)))

  if (!is.null(args)) {
    qassert(args, "S")
    fargs = names(formals(args(x))) %??% character()

    if (length(args) == 0L) {
      if (length(fargs) > 0L)
        return("May not have any arguments")
      return(TRUE)
    }

    qassert(ordered, "B1")
    if (ordered) {
      if (length(fargs) < length(args) || any(args != head(fargs, length(args)))) {
        return(sprintf("Must have first formal arguments (ordered): %s", paste0(args, collapse = ",")))
      }
    } else {
      tmp = setdiff(args, fargs)
      if (length(tmp))
        return(sprintf("Must have formal arguments: %s", paste0(tmp, collapse = ",")))
    }
  }

  if (!is.null(nargs)) {
    nargs = asCount(nargs)
    fnargs = length(setdiff(names(formals(args(x))) %??% character(0L), "..."))
    if (nargs != fnargs)
      return(sprintf("Must have exactly %i formal arguments, but has %i", nargs, fnargs))
  }

  return(TRUE)
}

#' @export
#' @rdname checkFunction
check_function = checkFunction

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkFunction
assertFunction = makeAssertionFunction(checkFunction, use.namespace = FALSE)

#' @export
#' @rdname checkFunction
assert_function = assertFunction

#' @export
#' @include makeTest.R
#' @rdname checkFunction
testFunction = makeTestFunction(checkFunction)

#' @export
#' @rdname checkFunction
test_function = testFunction

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkFunction
expect_function = makeExpectationFunction(checkFunction, use.namespace = FALSE)

test_backend = new.env(parent = emptyenv())

#' @title Select Backend for Unit Tests
#'
#' @description
#' Allows to explicitly select a backend for the unit tests.
#' Currently supported are \code{"testthat"} and \code{"tinytest"}.
#' The respective package must be installed and are loaded (but not attached).
#'
#' If this function is not explicitly called, defaults to \code{"testthat"} unless
#' the \code{"tinytest"}'s namespace is loaded.
#'
#' @param name [\code{character(1)}]\cr
#'  \code{"testthat"} or \code{"tinytest"}.
#' @return \code{NULL} (invisibly).
#' @export
register_test_backend = function(name) {
  name = match.arg(name, c("testthat", "tinytest"))
  if (name == "testthat") {
    requireNamespace("testthat")
    test_backend$name = "testthat"
  } else {
    requireNamespace("tinytest")
    test_backend$name = "tinytest"
  }
  invisible(NULL)
}

detect_test_backend = function() {
  if ("tinytest" %in% .packages())
    return("tinytest")
  return("testthat")
}

get_test_backend = function() {
  if (is.null(test_backend$name)) {
    test_backend$name = detect_test_backend()
    register_test_backend(test_backend$name)
  }
  test_backend$name
}

#' @title Turn a Check into an Expectation
#'
#' @description
#' \code{makeExpectation} is the internal function used to evaluate the result of a
#' check and turn it into an \code{\link[testthat]{expectation}}.
#' \code{makeExceptionFunction} can be used to automatically create an expectation
#' function based on a check function (see example).
#'
#' @template x
#' @param res [\code{TRUE} | \code{character(1)}]\cr
#'  The result of a check function: \code{TRUE} for successful checks,
#'  and an error message as string otherwise.
#' @param info [\code{character(1)}]\cr
#'   See \code{\link[testthat]{expect_that}}
#' @param label [\code{character(1)}]\cr
#'   See \code{\link[testthat]{expect_that}}
#' @return \code{makeExpectation} invisibly returns the checked object.
#'   \code{makeExpectationFunction} returns a \code{function}.
#' @export
#' @family CustomConstructors
#' @include helper.R
#' @examples
#' # Simple custom check function
#' checkFalse = function(x) if (!identical(x, FALSE)) "Must be FALSE" else TRUE
#'
#' # Create the respective expect function
#' expect_false = function(x, info = NULL, label = vname(x)) {
#'   res = checkFalse(x)
#'   makeExpectation(x, res, info = info, label = label)
#' }
#'
#' # Alternative: Automatically create such a function
#' expect_false = makeExpectationFunction(checkFalse)
#' print(expect_false)
makeExpectation = function(x, res, info, label) {
  backend = get_test_backend()
  if (backend == "testthat") {
    if (!requireNamespace("testthat", quietly = TRUE))
      stop("Package 'testthat' is required for checkmate's 'expect_*' extensions with backend 'testthat'")

    if (!is.null(info)) {
      info = sprintf("Additional info: %s", info)
    }

    if (isTRUE(res)) {
      testthat::succeed(info = info)
    } else {
      testthat::fail(sprintf("Check on '%s' failed: %s", label, res), info = info)
    }
    invisible(x)
  } else {
    if (!requireNamespace("tinytest", quietly = TRUE))
      stop("Package 'tinytest' is required for checkmate's 'expect_*' extensions with backend 'tinytest'")
    call = sys.call(sys.parent(1L))
    if (isTRUE(res)) {
      tinytest::tinytest(TRUE, call = call)
    } else {
      tinytest::tinytest(FALSE,
        call = call,
        diff = if (is.character(res)) res else "",
        info = if (is.null(info)) NA_character_ else info,
        short = "data"
      )
    }
  }
}

#' @rdname makeExpectation
#' @template makeFunction
#' @template use.namespace
#' @export
makeExpectationFunction = function(check.fun, c.fun = NULL, use.namespace = FALSE, env = parent.frame()) {
  fun.name = if (!is.character(check.fun)) deparse(substitute(check.fun)) else check.fun
  check.fun = match.fun(check.fun)
  check.args = fun.args = formals(args(check.fun))
  x.name = names(fun.args[1L])
  x = NULL

  new.fun = function() TRUE
  body = sprintf("if (missing(%s)) stop(sprintf(\"Argument '%%s' is missing\", label))", x.name)

  if (is.null(c.fun)) {
    body = paste0(body, sprintf("; res = %s(%s)", fun.name, paste0(names(check.args), collapse = ", ")))
  } else {
    body = paste0(body, sprintf("; res = .Call(%s)", paste0(c(c.fun, names(check.args)), collapse = ", ")))
  }

  if (use.namespace) {
    formals(new.fun) = c(fun.args, alist(info = NULL, label = checkmate::vname(x)))
    body = paste0(body, "; checkmate::makeExpectation")
  } else {
    formals(new.fun) = c(fun.args, alist(info = NULL, label = vname(x)))
    body = paste0(body, "; makeExpectation")
  }
  body = paste0(body, sprintf("(%s, res, info, label)", x.name))

  body(new.fun) = parse(text = paste("{", body, "}"))
  environment(new.fun) = env
  return(new.fun)
}

#' Check existence and access rights of files
#'
#' @note
#' The functions without the suffix \dQuote{exists} are deprecated and will be removed
#' from the package in a future version due to name clashes.
#' \code{test_file} has been unexported already.
#'
#' @templateVar fn FileExists
#' @template x
#' @inheritParams checkAccess
#' @param extension [\code{character}]\cr
#'  Vector of allowed file extensions, matched case insensitive.
#' @template checker
#' @family filesystem
#' @export
#' @examples
#' # Check if R's COPYING file is readable
#' testFileExists(file.path(R.home(), "COPYING"), access = "r")
#'
#' # Check if R's COPYING file is readable and writable
#' testFileExists(file.path(R.home(), "COPYING"), access = "rw")
checkFileExists = function(x, access = "", extension = NULL) {
  if (!qtest(x, "S+"))
    return("No file provided")

  w = wf(dir.exists(x))
  if (length(w) > 0L)
    return(sprintf("File expected, but directory in place: '%s'", x[w]))

  w = wf(!file.exists(x))
  if (length(w) > 0L)
    return(sprintf("File does not exist: '%s'", x[w]))

  checkAccess(x, access) %and% checkFileExtension(x, extension)
}

checkFileExtension = function(x, extension = NULL) {
  if (!is.null(extension)) {
    qassert(extension, "S+")
    ii = Reduce(`|`, lapply(tolower(extension), endsWith, x = tolower(x)))
    if (!all(ii))
      return(sprintf("File extension must be in {'%s'} (case insensitive), but file name is '%s'",
          paste0(extension, collapse = "','"),
          x[wf(!ii)]))
  }
  return(TRUE)
}

#' @export
#' @rdname checkFileExists
check_file_exists = checkFileExists

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkFileExists
assertFileExists = makeAssertionFunction(checkFileExists, use.namespace = FALSE)

#' @export
#' @rdname checkFileExists
assert_file_exists = assertFileExists

#' @export
#' @include makeTest.R
#' @rdname checkFileExists
testFileExists = makeTestFunction(checkFileExists)

#' @export
#' @rdname checkFileExists
test_file_exists = testFileExists

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkFileExists
expect_file_exists = makeExpectationFunction(checkFileExists, use.namespace = FALSE)

#' @export
#' @rdname checkFileExists
checkFile = checkFileExists

#' @export
#' @rdname checkFileExists
assertFile = assertFileExists

#' @export
#' @rdname checkFileExists
assert_file = assert_file_exists

#' @export
#' @rdname checkFileExists
testFile = testFileExists

#' @export
#' @rdname checkFileExists
expect_file = expect_file_exists

#' Check if an argument is NULL
#'
#' @templateVar fn Null
#' @template x
#' @template checker
#' @family basetypes
#' @export
#' @examples
#' testNull(NULL)
#' testNull(1)
checkNull = function(x) {
  if (!is.null(x))
    return("Must be NULL")
  return(TRUE)
}

#' @export
#' @rdname checkNull
check_null = checkNull

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkNull
assertNull = makeAssertionFunction(checkNull, use.namespace = FALSE)

#' @export
#' @rdname checkNull
assert_null = assertNull

#' @export
#' @include makeTest.R
#' @rdname checkNull
testNull = makeTestFunction(checkNull)

#' @export
#' @rdname checkNull
test_null = testNull

# This function is already provided by testthat
# #' @export
# #' @include makeExpectation.R
# #' @template expect
# #' @rdname checkNull
expect_null = makeExpectationFunction(checkNull, use.namespace = FALSE)

#' Check if an argument is vector of type integer
#'
#' @templateVar fn Integer
#' @template x
#' @template na-handling
#' @inheritParams checkVector
#' @template bounds
#' @template sorted
#' @template null.ok
#' @template typed.missing
#' @template checker
#' @family basetypes
#' @seealso \code{\link{asInteger}}
#' @useDynLib checkmate c_check_integer
#' @export
#' @examples
#' testInteger(1L)
#' testInteger(1.)
#' testInteger(1:2, lower = 1, upper = 2, any.missing = FALSE)
checkInteger = function(x, lower = -Inf, upper = Inf, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, sorted = FALSE, names = NULL, typed.missing = FALSE, null.ok = FALSE) {
  .Call(c_check_integer, x, lower, upper, any.missing, all.missing, len, min.len, max.len, unique, sorted, names, typed.missing, null.ok)
}

#' @export
#' @rdname checkInteger
check_integer = checkInteger

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkInteger
assertInteger = makeAssertionFunction(checkInteger, c.fun = "c_check_integer", use.namespace = FALSE)

#' @export
#' @rdname checkInteger
assert_integer = assertInteger

#' @export
#' @include makeTest.R
#' @rdname checkInteger
testInteger = makeTestFunction(checkInteger, c.fun = "c_check_integer")

#' @export
#' @rdname checkInteger
test_integer = testInteger

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkInteger
expect_integer = makeExpectationFunction(checkInteger, c.fun = "c_check_integer", use.namespace = FALSE)

context("checkNumeric")

test_that("checkNumeric", {
  myobj = 1
  expect_succ_all(Numeric, myobj)
  myobj = "a"
  expect_fail_all(Numeric, myobj)

  expect_true(testNumeric(integer(0)))
  expect_false(testNumeric(NULL))
  expect_false(testNumeric(TRUE))
  expect_false(testNumeric(FALSE))
  expect_true(testNumeric(NA_character_))
  expect_true(testNumeric(NA_real_))
  expect_true(testNumeric(NaN))
  expect_false(testNumeric(NA_real_, any.missing = FALSE))
  expect_false(testNumeric(NA_real_, all.missing = FALSE))
  expect_false(testNumeric(NaN, any.missing = FALSE))
  expect_false(testNumeric(NaN, all.missing = FALSE))
  expect_true(testNumeric(1L))
  expect_true(testNumeric(1))
  expect_true(testNumeric(Inf))
  expect_true(testNumeric(-Inf))
  expect_identical(assertNumeric(1:2, finite = TRUE), 1:2)
  expect_error(assertNumeric(c(1, Inf), finite = TRUE), "finite")
  expect_error(assertNumeric(c(1, -Inf), finite = TRUE), "finite")
  expect_true(testNumeric(1:3, any.missing=FALSE, min.len=1L, max.len=3L))
  expect_false(testNumeric(1:3, any.missing=FALSE, len=5))
  expect_true(testNumeric(1:3, lower = 1L, upper = 3L))
  expect_false(testNumeric(1:3, lower = 5))

  expect_error(assertNumeric("a"), "numeric")
})

test_that("bounds are checked", {
  expect_error(checkNumeric(1, lower = "a"), "number")
  expect_error(checkNumeric(1, lower = 1:2), "length")
  expect_error(checkNumeric(1, lower = NA_real_), "missing")
  expect_error(checkNumeric(1, upper = "a"), "number")
  expect_error(checkNumeric(1, upper = 1:2), "length")
  expect_error(checkNumeric(1, upper = NA_real_), "missing")
})

test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkNumeric(NA, lower = 1))
  expect_true(checkNumeric(NA_character_, upper = 10))
  expect_fail_all(Numeric, 0:5, lower = 1L)
  expect_fail_all(Numeric, 5:15, upper = 10L)
})


test_that("sorted works", {
  xu = runif(10)
  while(!is.unsorted(xu))
    xu = runif(10)
  xs = sort(xu)

  expect_true(checkNumeric(xs, sorted = TRUE))
  expect_true(grepl("sorted", checkNumeric(xu, sorted = TRUE), fixed = TRUE))

  expect_true(checkNumeric(1., sorted = TRUE))
  expect_true(checkNumeric(double(0), sorted = TRUE))
  expect_true(checkNumeric(NA_real_, sorted = TRUE))
  expect_true(checkInteger(rep(NA_real_, 10), sorted = TRUE))

  for (i in 1:10) {
    x = sample(10)
    x[sample(10, sample(7:9, 1))] = NA
    if (is.unsorted(na.omit(x)))
      expect_true(grepl("sorted", checkNumeric(xu, sorted = TRUE), fixed = TRUE))
    else
      expect_true(grepl("sorted", checkNumeric(xu, sorted = TRUE), fixed = TRUE))
  }
  ## see https://github.com/mllg/checkmate/issues/258
  expect_true(grepl("sorted", checkNumeric(c(3, NA, 2), sorted = TRUE), fixed = TRUE))
})

test_that("names check works", {
  expect_true(grepl("names", check_numeric(0:1, names = "unique")))
})


test_that("typed.missing", {
  expect_true(testNumeric(NA))
  expect_true(testNumeric(NA, typed.missing = FALSE))
  expect_false(testNumeric(NA, typed.missing = TRUE))

  expect_true(testNumeric(character()))
  expect_true(testNumeric(character(), typed.missing = FALSE))
  expect_false(testNumeric(character(), typed.missing = TRUE))
})

test_that("Inf bound checks", {
  expect_false(testNumeric(2, lower = Inf))
  expect_true(testNumeric(2, lower = -Inf))
  expect_true(testNumeric(2, upper = Inf))
  expect_false(testNumeric(2, upper = -Inf))
})

context("checkNumeric")

test_that("checkNumeric", {
  myobj = 1
  expect_succ_all(Numeric, myobj)
  myobj = "a"
  expect_fail_all(Numeric, myobj)

  expect_true(testNumeric(integer(0)))
  expect_false(testNumeric(NULL))
  expect_false(testNumeric(TRUE))
  expect_false(testNumeric(FALSE))
  expect_true(testNumeric(NA_character_))
  expect_true(testNumeric(NA_real_))
  expect_true(testNumeric(NaN))
  expect_false(testNumeric(NA_real_, any.missing = FALSE))
  expect_false(testNumeric(NA_real_, all.missing = FALSE))
  expect_false(testNumeric(NaN, any.missing = FALSE))
  expect_false(testNumeric(NaN, all.missing = FALSE))
  expect_true(testNumeric(1L))
  expect_true(testNumeric(1))
  expect_true(testNumeric(Inf))
  expect_true(testNumeric(-Inf))
  expect_identical(assertNumeric(1:2, finite = TRUE), 1:2)
  expect_error(assertNumeric(c(1, Inf), finite = TRUE), "finite")
  expect_error(assertNumeric(c(1, -Inf), finite = TRUE), "finite")
  expect_true(testNumeric(1:3, any.missing=FALSE, min.len=1L, max.len=3L))
  expect_false(testNumeric(1:3, any.missing=FALSE, len=5))
  expect_true(testNumeric(1:3, lower = 1L, upper = 3L))
  expect_false(testNumeric(1:3, lower = 5))

  expect_error(assertNumeric("a"), "numeric")
})

test_that("bounds are checked", {
  expect_error(checkNumeric(1, lower = "a"), "number")
  expect_error(checkNumeric(1, lower = 1:2), "length")
  expect_error(checkNumeric(1, lower = NA_real_), "missing")
  expect_error(checkNumeric(1, upper = "a"), "number")
  expect_error(checkNumeric(1, upper = 1:2), "length")
  expect_error(checkNumeric(1, upper = NA_real_), "missing")
})

test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkNumeric(NA, lower = 1))
  expect_true(checkNumeric(NA_character_, upper = 10))
  expect_fail_all(Numeric, 0:5, lower = 1L)
  expect_fail_all(Numeric, 5:15, upper = 10L)
})


test_that("sorted works", {
  xu = runif(10)
  while(!is.unsorted(xu))
    xu = runif(10)
  xs = sort(xu)

  expect_true(checkNumeric(xs, sorted = TRUE))
  expect_true(grepl("sorted", checkNumeric(xu, sorted = TRUE), fixed = TRUE))

  expect_true(checkNumeric(1., sorted = TRUE))
  expect_true(checkNumeric(double(0), sorted = TRUE))
  expect_true(checkNumeric(NA_real_, sorted = TRUE))
  expect_true(checkInteger(rep(NA_real_, 10), sorted = TRUE))

  for (i in 1:10) {
    x = sample(10)
    x[sample(10, sample(7:9, 1))] = NA
    if (is.unsorted(na.omit(x)))
      expect_true(grepl("sorted", checkNumeric(xu, sorted = TRUE), fixed = TRUE))
    else
      expect_true(grepl("sorted", checkNumeric(xu, sorted = TRUE), fixed = TRUE))
  }
})

test_that("names check works", {
  expect_true(grepl("names", check_numeric(0:1, names = "unique")))
})


test_that("typed.missing", {
  expect_true(testNumeric(NA))
  expect_true(testNumeric(NA, typed.missing = FALSE))
  expect_false(testNumeric(NA, typed.missing = TRUE))

  expect_true(testNumeric(character()))
  expect_true(testNumeric(character(), typed.missing = FALSE))
  expect_false(testNumeric(character(), typed.missing = TRUE))
})

test_that("Inf bound checks", {
  expect_false(testNumeric(2, lower = Inf))
  expect_true(testNumeric(2, lower = -Inf))
  expect_true(testNumeric(2, upper = Inf))
  expect_false(testNumeric(2, upper = -Inf))
})

context("checkDouble")

test_that("checkDouble", {
  myobj = 1
  expect_succ_all(Double, myobj)
  myobj = "a"
  expect_fail_all(Double, myobj)

  expect_true(testDouble(integer(0)))
  expect_false(testDouble(NULL))
  expect_false(testDouble(TRUE))
  expect_false(testDouble(FALSE))
  expect_true(testDouble(NA_character_))
  expect_true(testDouble(NA_real_))
  expect_true(testDouble(NaN))
  expect_false(testDouble(NA_real_, any.missing = FALSE))
  expect_false(testDouble(NA_real_, all.missing = FALSE))
  expect_false(testDouble(NaN, any.missing = FALSE))
  expect_false(testDouble(NaN, all.missing = FALSE))
  expect_false(testDouble(1L))
  expect_true(testDouble(1))
  expect_true(testDouble(Inf))
  expect_true(testDouble(-Inf))
  expect_identical(assertDouble(1:2 + 0.1, finite = TRUE), 1:2 + 0.1)
  expect_error(assertDouble(c(1, Inf), finite = TRUE), "finite")
  expect_error(assertDouble(c(1, -Inf), finite = TRUE), "finite")
  expect_true(testDouble(1:3 + 0.1, any.missing=FALSE, min.len=1L, max.len=3L))
  expect_false(testDouble(1:3 + 0.1, any.missing=FALSE, len=5))
  expect_true(testDouble(1:3 + 0.1, lower = 1L, upper = 3.5))
  expect_false(testDouble(1:3 + 0.1, lower = 5))

  expect_error(assertDouble("a"), "double")
})

test_that("bounds are checked", {
  expect_error(checkDouble(1, lower = "a"), "number")
  expect_error(checkDouble(1, lower = 1:2), "length")
  expect_error(checkDouble(1, lower = NA_real_), "missing")
  expect_error(checkDouble(1, upper = "a"), "number")
  expect_error(checkDouble(1, upper = 1:2), "length")
  expect_error(checkDouble(1, upper = NA_real_), "missing")
})

test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkDouble(NA, lower = 1))
  expect_true(checkDouble(NA_character_, upper = 10))
})

test_that("sorted works", {
  xu = runif(10)
  while(!is.unsorted(xu))
    xu = runif(10)
  xs = sort(xu)

  expect_true(checkDouble(xs, sorted = TRUE))
  expect_true(grepl("sorted", checkDouble(xu, sorted = TRUE), fixed = TRUE))

  expect_true(checkDouble(1., sorted = TRUE))
  expect_true(checkDouble(double(0), sorted = TRUE))
  expect_true(checkDouble(NA_real_, sorted = TRUE))
  expect_true(checkInteger(rep(NA_real_, 10), sorted = TRUE))

  for (i in 1:10) {
    x = sample(10)
    x[sample(10, sample(7:9, 1))] = NA
    if (is.unsorted(na.omit(x)))
      expect_true(grepl("sorted", checkDouble(xu, sorted = TRUE), fixed = TRUE))
    else
      expect_true(grepl("sorted", checkDouble(xu, sorted = TRUE), fixed = TRUE))
  }
})

test_that("typed.missing", {
  expect_true(testDouble(NA_character_))
  expect_true(testDouble(NA_character_, typed.missing = FALSE))
  expect_false(testDouble(NA_character_, typed.missing = TRUE))

  expect_true(testDouble(character()))
  expect_true(testDouble(character(), typed.missing = FALSE))
  expect_false(testDouble(character(), typed.missing = TRUE))
})

context("checkInteger")

test_that("checkInteger", {
  myobj = 1L
  expect_succ_all(Integer, myobj)
  myobj = 1
  expect_fail_all(Integer, myobj)

  expect_true(testInteger(integer(0)))
  expect_false(testInteger(NULL))
  expect_false(testInteger(TRUE))
  expect_false(testInteger(FALSE))
  expect_true(testInteger(NA))
  expect_false(testInteger(NA, any.missing = FALSE))
  expect_false(testInteger(NA, all.missing = FALSE))
  expect_true(testInteger(1L))
  expect_true(testInteger(1:3, any.missing = FALSE, min.len = 1L, max.len = 3L))
  expect_false(testInteger(1:3, any.missing = FALSE, len = 5))
  expect_true(testInteger(1:3, lower = 1L, upper = 3L))
  expect_false(testInteger(1:3, lower = 5))
  expect_false(testInteger(1:3, upper = 1))

  expect_error(assertInteger(1), "integer")
})

test_that("bounds of vectors with only missings are not checked", {
  expect_true(checkInteger(NA, lower = 1))
  expect_true(checkInteger(NA_character_, upper = 10))
  expect_fail_all(Integer, 0L, lower = 1L)
  expect_fail_all(Integer, 100L, upper = 10L)
})

test_that("sorted works", {
  xu = sample(10)
  while(!is.unsorted(xu))
    xu = sample(10)
  xs = sort(xu)

  expect_true(checkInteger(xs, sorted = TRUE))
  expect_true(grepl("sorted", checkInteger(xu, sorted = TRUE), fixed = TRUE))

  expect_true(checkInteger(1L, sorted = TRUE))
  expect_true(checkInteger(integer(0), sorted = TRUE))
  expect_true(checkInteger(NA_integer_, sorted = TRUE))
  expect_true(checkInteger(rep(NA_integer_, 10), sorted = TRUE))

  for (i in 1:10) {
    x = sample(10)
    x[sample(10, sample(7:9, 1))] = NA
    if (is.unsorted(na.omit(x)))
      expect_true(grepl("sorted", checkInteger(xu, sorted = TRUE), fixed = TRUE))
    else
      expect_true(grepl("sorted", checkInteger(xu, sorted = TRUE), fixed = TRUE))
  }
})


test_that("factors are detected (#164)", {
  x = factor(letters)
  expect_error(assertInteger(x), "factor")
})

test_that("typed.missing", {
  expect_true(testInteger(NA_character_))
  expect_true(testInteger(NA_character_, typed.missing = FALSE))
  expect_false(testInteger(NA_character_, typed.missing = TRUE))

  expect_true(testInteger(character()))
  expect_true(testInteger(character(), typed.missing = FALSE))
  expect_false(testInteger(character(), typed.missing = TRUE))
})

#' Check if an argument is a matrix
#'
#' @templateVar fn Matrix
#' @template x
#' @template mode
#' @param any.missing [\code{logical(1)}]\cr
#'  Are missing values allowed? Default is \code{TRUE}.
#' @param all.missing [\code{logical(1)}]\cr
#'  Are matrices with only missing values allowed? Default is \code{TRUE}.
#' @param min.rows [\code{integer(1)}]\cr
#'  Minimum number of rows.
#' @param max.rows [\code{integer(1)}]\cr
#'  Maximum number of rows.
#' @param min.cols [\code{integer(1)}]\cr
#'  Minimum number of columns.
#' @param max.cols [\code{integer(1)}]\cr
#'  Maximum number of columns.
#' @param nrows [\code{integer(1)}]\cr
#'  Exact number of rows.
#' @param ncols [\code{integer(1)}]\cr
#'  Exact number of columns.
#' @param row.names [\code{character(1)}]\cr
#'  Check for row names. Default is \dQuote{NULL} (no check).
#'  See \code{\link{checkNamed}} for possible values.
#'  Note that you can use \code{\link{checkSubset}} to check for a specific set of names.
#' @param col.names [\code{character(1)}]\cr
#'  Check for column names. Default is \dQuote{NULL} (no check).
#'  See \code{\link{checkNamed}} for possible values.
#'  Note that you can use \code{\link{checkSubset}} to test for a specific set of names.
#' @template null.ok
#' @template checker
#' @family basetypes
#' @family compound
#' @useDynLib checkmate c_check_matrix
#' @export
#' @examples
#' x = matrix(1:9, 3)
#' colnames(x) = letters[1:3]
#' testMatrix(x, nrows = 3, min.cols = 1, col.names = "named")
checkMatrix = function(x, mode = NULL, any.missing = TRUE, all.missing = TRUE, min.rows = NULL, max.rows = NULL, min.cols = NULL, max.cols = NULL, nrows = NULL, ncols = NULL, row.names = NULL, col.names = NULL, null.ok = FALSE) {
  .Call(c_check_matrix, x, mode, any.missing, all.missing, min.rows, max.rows, min.cols, max.cols, nrows, ncols, row.names, col.names, null.ok)
}

#' @export
#' @rdname checkMatrix
check_matrix = checkMatrix

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkMatrix
assertMatrix = makeAssertionFunction(checkMatrix, c.fun = "c_check_matrix", use.namespace = FALSE)

#' @export
#' @rdname checkMatrix
assert_matrix = assertMatrix

#' @export
#' @include makeTest.R
#' @rdname checkMatrix
testMatrix = makeTestFunction(checkMatrix, c.fun = "c_check_matrix")

#' @export
#' @rdname checkMatrix
test_matrix = testMatrix

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkMatrix
expect_matrix = makeExpectationFunction(checkMatrix, c.fun = "c_check_matrix", use.namespace = FALSE)

context("checkMatrix")

test_that("checkMatrix", {
  myobj = matrix(1:9, 3)
  expect_succ_all(Matrix, myobj)
  myobj = TRUE
  expect_fail_all(Matrix, myobj)

  x = matrix(1:9, 3)
  expect_true(testMatrix(x))
  expect_true(testMatrix(matrix(nrow=0, ncol=0)))
  expect_false(testMatrix(NULL))
  x[2,2] = NA
  expect_true(testMatrix(x))
  expect_false(testMatrix(x, any.missing = FALSE))

  xl = matrix(TRUE)
  xi = matrix(1L)
  xr = matrix(1.)
  xs = matrix("a")
  xc = matrix(1+1i)
  xx = array(list(1, 1), dim = c(1, 2))
  xe = matrix(nrow = 0, ncol = 0); storage.mode(xe) = "double"
  expect_true(testMatrix(xl, "logical"))
  expect_true(testMatrix(xi, "integer"))
  expect_true(testMatrix(xr, "double"))
  expect_true(testMatrix(xe, "double"))
  expect_true(testMatrix(xr, "numeric"))
  expect_true(testMatrix(xc, "complex"))
  expect_true(testMatrix(xs, "character"))
  expect_true(testMatrix(xx, "list"))
  expect_false(testMatrix(xs, "logical"))
  expect_false(testMatrix(xs, "integer"))
  expect_false(testMatrix(xs, "double"))
  expect_true(testMatrix(xe, "logical")) # empty, so all are missing
  expect_false(testMatrix(xs, "numeric"))
  expect_false(testMatrix(xs, "complex"))
  expect_false(testMatrix(xl, "character"))
  expect_false(testMatrix(xx, "numeric"))
  expect_false(testMatrix(xx, "atomic"))
  expect_true(testMatrix(xi, "integerish"))
  expect_true(testMatrix(xr, "integerish"))
  expect_false(testMatrix(xi+0.1, "integerish"))

  expect_true(testMatrix(x, min.rows = 1, min.cols = 1))
  expect_true(testMatrix(x, max.rows = 5, max.cols = 5))
  expect_true(testMatrix(x, nrows = 3, ncols = 3))
  expect_true(testMatrix(xx, nrows = 1, ncols = 2))
  expect_true(testMatrix(xe, nrows = 0, ncols = 0))
  expect_false(testMatrix(x, max.rows = 1))
  expect_false(testMatrix(x, max.cols = 1))
  expect_false(testMatrix(x, min.rows = 5))
  expect_false(testMatrix(x, min.cols = 5))
  expect_false(testMatrix(x, nrows = 5))
  expect_false(testMatrix(x, ncols = 5))

  expect_false(testMatrix(x, row.names = "named"))
  expect_false(testMatrix(x, col.names = "named"))
  rownames(x) = letters[1:3]; colnames(x) = NULL
  expect_true(testMatrix(x, row.names = "named"))
  expect_false(testMatrix(x, col.names = "named"))
  colnames(x) = letters[1:3]; rownames(x) = NULL
  expect_false(testMatrix(x, row.names = "named"))
  expect_true(testMatrix(x, col.names = "named"))
  colnames(x) = rownames(x) = letters[1:3]
  expect_true(testMatrix(x, row.names = "named"))
  expect_true(testMatrix(x, col.names = "named"))

  # named and unnamed is the same for "empty" matricies
  expect_true(testMatrix(xe, row.names = "unnamed"))
  expect_true(testMatrix(xe, col.names = "unnamed"))
  expect_true(testMatrix(xe, row.names = "strict"))
  expect_true(testMatrix(xe, col.names = "strict"))

  expect_error(assertMatrix(iris), "matrix")
  expect_error(assertMatrix(matrix(), min.rows = 99), "99")
})

test_that("dimension arguments are checked", {
  x = matrix(1)
  expect_error(checkMatrix(x, min.rows = 1.2), "close")
  expect_error(checkMatrix(x, min.rows = NA_integer_), "missing")
  expect_error(checkMatrix(x, min.rows = -1), ">= 0")
})

test_that("dimensions are reported correctly", {
  x = matrix(1:42, ncol = 1)
  expect_true(grepl(42, checkMatrix(x, nrows = 43)))
  expect_true(grepl(42, checkMatrix(x, min.rows = 43)))

  x = t(x)
  expect_true(grepl(42, checkMatrix(x, ncols = 43)))
  expect_true(grepl(42, checkMatrix(x, min.cols = 43)))
})

test_that("missing locations are reported correctly", {
  x = matrix(1:12, ncol = 3); x[1, 1] = NA
  expect_true(grepl("row 1, col 1", checkMatrix(x, any.missing = FALSE)))
  x = matrix(1:12, ncol = 3); x[2, 3] = NA
  expect_true(grepl("row 2, col 3", checkMatrix(x, any.missing = FALSE)))
  x = matrix(1:12, ncol = 3); x[4, 3] = NA
  expect_true(grepl("row 4, col 3", checkMatrix(x, any.missing = FALSE)))
})

test_that("mode accepts all missing values (#184)", {
  x = matrix(NA, nrow = 3, ncol = 3)
  expect_true(test_matrix(x, mode = "double"))
})

#' Check if an argument is equal to a given set
#'
#' @templateVar fn Subset
#' @template x
#' @param y [\code{atomic}]\cr
#'  Set to compare with.
#' @param ordered [\code{logical(1)}]\cr
#' Check \code{x} to have the same length and order as \code{y}, i.e.
#' check using \dQuote{==} while handling \code{NA}s nicely.
#' Default is \code{FALSE}.
#' @template fmatch
#' @template checker
#' @template set
#' @family set
#' @export
#' @examples
#' testSetEqual(c("a", "b"), c("a", "b"))
#' testSetEqual(1:3, 1:4)
#'
#' # x is not converted before the comparison (except for numerics)
#' testSetEqual(factor("a"), "a")
#' testSetEqual(1, "1")
#' testSetEqual(1, as.integer(1))
checkSetEqual = function(x, y, ordered = FALSE, fmatch = FALSE) {
  qassert(x, "a")
  qassert(y, "a")
  qassert(ordered, "B1")

  if (ordered) {
    if (!isSameType(x, y) || length(x) != length(y) || any(xor(is.na(x), is.na(y)) | x != y, na.rm = TRUE))
      return(sprintf("Must be equal to %s, but is %s", array_collapse(y), array_collapse(x)))
    return(TRUE)
  }

  if (isTRUE(fmatch) && requireNamespace("fastmatch", quietly = TRUE))
    match = fastmatch::fmatch

  check_set_equal_internal(x, y, match)
}

#' @export
#' @rdname checkSetEqual
check_set_equal = checkSetEqual

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkSetEqual
assertSetEqual = makeAssertionFunction(checkSetEqual, use.namespace = FALSE)

#' @export
#' @rdname checkSetEqual
assert_set_equal = assertSetEqual

#' @export
#' @include makeTest.R
#' @rdname checkSetEqual
testSetEqual = makeTestFunction(checkSetEqual)

#' @export
#' @rdname checkSetEqual
test_set_equal = testSetEqual

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkSetEqual
expect_set_equal = makeExpectationFunction(checkSetEqual, use.namespace = FALSE)

#' Check if an argument is a subset of a given set
#'
#' @templateVar fn Subset
#' @template x
#' @param choices [\code{atomic}]\cr
#'  Set of possible values. May be empty.
#' @param empty.ok [\code{logical(1)}]\cr
#'  Treat zero-length \code{x} as subset of any set \code{choices} (this includes \code{NULL})?
#'  Default is \code{TRUE}.
#' @template fmatch
#' @template checker
#' @template set
#' @family set
#' @export
#' @examples
#' testSubset(c("a", "z"), letters)
#' testSubset("ab", letters)
#' testSubset("Species", names(iris))
#'
#' # x is not converted before the comparison (except for numerics)
#' testSubset(factor("a"), "a")
#' testSubset(1, "1")
#' testSubset(1, as.integer(1))
checkSubset = function(x, choices, empty.ok = TRUE, fmatch = FALSE) {
  qassert(empty.ok, "B1")
  if (length(x) == 0L) {
    if (!empty.ok)
      return(sprintf("Must be a subset of %s, not empty", set_collapse(choices)))
    return(TRUE)
  }

  if (isTRUE(fmatch) && requireNamespace("fastmatch", quietly = TRUE))
    match = fastmatch::fmatch

  check_subset_internal(x, choices, match)
}

#' @export
#' @rdname checkSubset
check_subset = checkSubset

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkSubset
assertSubset = makeAssertionFunction(checkSubset, use.namespace = FALSE)

#' @export
#' @rdname checkSubset
assert_subset = assertSubset

#' @export
#' @include makeTest.R
#' @rdname checkSubset
testSubset = makeTestFunction(checkSubset)

#' @export
#' @rdname checkSubset
test_subset = testSubset

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkSubset
expect_subset = makeExpectationFunction(checkSubset, use.namespace = FALSE)

#' @title Quick recursive arguments checks on lists and data frames
#'
#' @description
#' These functions are the tuned counterparts of \code{\link{qtest}},
#' \code{\link{qassert}} and \code{\link{qexpect}} tailored for recursive
#' checks of list elements or data frame columns.
#'
#' @param x [\code{list} or \code{data.frame}]\cr
#'   List or data frame to check for compliance with at least one of \code{rules}.
#'   See details of \code{\link{qtest}} for rule explanation.
#' @param rules [\code{character}]\cr
#'   Set of rules. See \code{\link{qtest}}
#' @template var.name
#' @return See \code{\link{qassert}}.
#' @seealso \code{\link{qtest}}, \code{\link{qassert}}
#' @useDynLib checkmate c_qassert
#' @export
#' @examples
#' # All list elements are integers with length >= 1?
#' qtestr(as.list(1:10), "i+")
#'
#' # All list elements (i.e. data frame columns) are numeric?
#' qtestr(iris, "n")
#'
#' # All list elements are numeric, w/o NAs?
#' qtestr(list(a = 1:3, b = rnorm(1), c = letters), "N+")
#'
#' # All list elements are numeric OR character
#' qtestr(list(a = 1:3, b = rnorm(1), c = letters), c("N+", "S+"))
qassertr = function(x, rules, .var.name = vname(x)) {
  res = .Call(c_qassert, x, rules, TRUE)
  if (!isTRUE(res))
    mstop(qrmsg(x, res, .var.name), call. = sys.call(-1L))
  invisible(x)
}

#' @rdname qassertr
#' @param depth [\code{integer(1)}]\cr
#'   Maximum recursion depth. Defaults to \dQuote{1} to directly check list elements or
#'   data frame columns. Set to a higher value to check lists of lists of elements.
#' @useDynLib checkmate c_qtest
#' @export
qtestr = function(x, rules, depth = 1L) {
  .Call(c_qtest, x, rules, TRUE, depth)
}

#' @rdname qassertr
#' @template expect
#' @include makeExpectation.R
#' @useDynLib checkmate c_qassert
#' @export
qexpectr = function(x, rules, info = NULL, label = vname(x)) {
  res = .Call(c_qassert, x, rules, TRUE)
  if (!isTRUE(res))
    res = qrmsg(x, res, label)
  makeExpectation(x, res, info = info, label = label)
}

qrmsg = function(x, msg, var.name) {
  pos = attr(msg, "pos")
  if (testNamed(x)) {
    item = sprintf(", element '%s' (%i),", names(x)[pos], pos)
  } else {
    item = sprintf(", element %i,", pos)
  }
  if (length(msg) > 1L)
    msg = paste0(c("One of the following must apply:", strwrap(msg, prefix = " * ")), collapse = "\n")
  sprintf("Assertion on '%s'%s failed. %s.", var.name, item, msg)
}

context("qtestr")

expect_succ_all = function(x, rules) {
  xn = deparse(substitute(x))
  expect_true(qtestr(x, rules),
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
  expect_identical(qassertr(x, rules), x,
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
  expect_expectation_successful(qexpectr(x, rules),
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
}

expect_fail_all = function(x, rules, pattern = NULL) {
  xn = deparse(substitute(x))
  expect_false(qtestr(x, rules),
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
  expect_error(qassertr(x, rules), regexp = pattern,
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
  expect_expectation_failed(qexpectr(x, rules),
    info = sprintf("rules: %s", paste0(rules, collapse=",")), label = xn)
}

test_that("qassertr / qtestr", {
  x = list(a = 1:10, b = rnorm(10))
  expect_succ_all(x, "n+")
  expect_succ_all(x, "n10")
  expect_succ_all(x, "n>=1")
  expect_fail_all(x, "i+")
  expect_fail_all(x, "l")

  x = list(a = NULL, b = 10)
  expect_succ_all(x, "*")
  expect_fail_all(x, "0")
  expect_fail_all(x, "n")

  x = list(a = NULL, b = NULL)
  expect_succ_all(x, "0")
  expect_fail_all(x, "0+")

  x = list()
  expect_succ_all(x, "n+")
  expect_succ_all(x, "0+")

  x = list(1, 2)
  expect_fail_all(x, "S1", pattern = "string")

  x = list(1:10, NULL)
  expect_succ_all(x, c("v", "l", "0"))
  rules = c("v", "l")
  expect_fail_all(x, c("v", "l"), pattern = "One of")

  expect_succ_all(iris, c("f", "n"))
  expect_fail_all(iris, c("s", "n"), pattern = "One of")

  x = NULL
  expect_error(qassertr(x, "x"), "list or data.frame")
  expect_error(qtestr(x, "x"), "list or data.frame")
})

test_that("qtestr / depth", {
  x = list(letters, 1:10, list(letters, 2:3, runif(10)))
  rules = c("v", "l")
  expect_true(qtestr(x, rules, depth = 1L))
  expect_true(qtestr(x, rules, depth = 2L))
  expect_true(qtestr(x, rules, depth = 3L))

  x[[3]][[2]] = iris
  expect_true(qtestr(x, rules, depth = 1L))
  expect_true(qtestr(x, c(rules, "d"), depth = 1L))
  expect_false(qtestr(x, rules, depth = 2L))
  expect_false(qtestr(x, rules, depth = 3L))
})

#' Check if an argument is a vector
#'
#' @templateVar fn Vector
#' @template x
#' @param strict [\code{logical(1)}]\cr
#'  May the vector have additional attributes? If \code{TRUE}, mimics the behavior of
#'  \code{\link[base]{is.vector}}.
#'  Default is \code{FALSE} which allows e.g. \code{factor}s or \code{data.frame}s
#'  to be recognized as vectors.
#' @param any.missing [\code{logical(1)}]\cr
#'  Are vectors with missing values allowed? Default is \code{TRUE}.
#' @param all.missing [\code{logical(1)}]\cr
#'  Are vectors with no non-missing values allowed? Default is \code{TRUE}.
#'  Note that empty vectors do not have non-missing values.
#' @param len [\code{integer(1)}]\cr
#'  Exact expected length of \code{x}.
#' @param min.len [\code{integer(1)}]\cr
#'  Minimal length of \code{x}.
#' @param max.len [\code{integer(1)}]\cr
#'  Maximal length of \code{x}.
#' @param unique [\code{logical(1)}]\cr
#'  Must all values be unique? Default is \code{FALSE}.
#' @param names [\code{character(1)}]\cr
#'  Check for names. See \code{\link{checkNamed}} for possible values.
#'  Default is \dQuote{any} which performs no check at all.
#'  Note that you can use \code{\link{checkSubset}} to check for a specific set of names.
#' @template null.ok
#' @template checker
#' @family basetypes
#' @family atomicvector
#' @useDynLib checkmate c_check_vector
#' @export
#' @examples
#' testVector(letters, min.len = 1L, any.missing = FALSE)
checkVector = function(x, strict = FALSE, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, names = NULL, null.ok = FALSE) {
  .Call(c_check_vector, x, strict, any.missing, all.missing, len, min.len, max.len, unique, names, null.ok)
}

#' @export
#' @rdname checkVector
check_vector = checkVector

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkVector
assertVector = makeAssertionFunction(checkVector, c.fun = "c_check_vector", use.namespace = FALSE)

#' @export
#' @rdname checkVector
assert_vector = assertVector

#' @export
#' @include makeTest.R
#' @rdname checkVector
testVector = makeTestFunction(checkVector, c.fun = "c_check_vector")

#' @export
#' @rdname checkVector
test_vector = testVector

context("checkVector")

li = list(
  list = list(1, 2),
  factor = factor("a"),
  integer = 1:2,
  NULL = NULL,
  data.frame = iris
)

test_that("checkVector", {
  myobj = 1:3
  expect_succ_all(Vector, myobj)
  myobj = NULL
  expect_fail_all(Vector, myobj)

  expect_true(testVector(integer(0)))
  expect_false(testVector(NULL))
  expect_true(testVector(1))
  expect_true(testVector(integer(0)))
  expect_true(testVector(factor(1), strict = FALSE))
  expect_false(testVector(factor(1), strict = TRUE))

  expect_true(testVector(NA, any.missing = TRUE))
  expect_false(testVector(NA, any.missing = FALSE))
  expect_false(testVector(NA, all.missing = FALSE))

  expect_true(testVector(1, len=1))
  expect_false(testVector(1, len=0))

  expect_true(testVector(1, min.len=0))
  expect_false(testVector(1, min.len=2))
  expect_true(testVector(1, max.len=1))
  expect_false(testVector(1, max.len=0))

  expect_true(testVector(1, unique=TRUE))
  expect_false(testVector(1, min.len=2))
  expect_true(testVector(1, max.len=1))
  expect_false(testVector(1, max.len=0))

  expect_true(testVector(1, unique=TRUE))
  expect_true(testVector(c(1,1), unique=FALSE))
  expect_false(testVector(c(1,1), unique=TRUE))

  expect_true(testVector(1, names="unnamed"))
  expect_true(testVector(setNames(1, "x"), names="named"))
  expect_false(testVector(1, names="unique"))

  expect_equal(vlapply(li, is.vector), vlapply(li, testVector, strict = TRUE))
  expected = setNames(c(TRUE, TRUE, TRUE, FALSE, TRUE), c("list", "factor", "integer", "NULL", "data.frame"))
  expect_equal(expected, vlapply(li, testVector, strict = FALSE))

  expect_error(assertVector(iris, strict = TRUE), "vector")
})

test_that("arguments any.missing and all.missing are checked", {
  x = 1
  expect_error(checkVector(x, any.missing = 1), "flag")
  expect_error(checkVector(x, any.missing = NA), "missing")
  expect_error(checkVector(x, all.missing = 1), "flag")
  expect_error(checkVector(x, all.missing = NA), "missing")
})

test_that("length is correctly reported", {
  x = 1:42
  expect_true(grepl(42, checkVector(x, len = 1), fixed = TRUE))
  expect_true(grepl(42, checkVector(x, min.len = 43), fixed = TRUE))
  expect_true(grepl(42, checkVector(x, max.len = 1), fixed = TRUE))
  expect_true(grepl(43, checkVector(x, len = 43), fixed = TRUE))
  expect_true(grepl(43, checkVector(x, min.len = 43), fixed = TRUE))
  expect_true(grepl(41, checkVector(x, max.len = 41), fixed = TRUE))
})

#' Check if an argument is a formula
#'
#' @templateVar fn Formula
#' @template x
#' @template null.ok
#' @template checker
#' @family basetypes
#' @export
#' @examples
#' f = Species ~ Sepal.Length + Sepal.Width
#' checkFormula(f)
checkFormula = function(x, null.ok = FALSE) {
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be a formula, not 'NULL'")
  }

  if (!inherits(x, "formula"))
    return(sprintf("Must be a formula%s, not %s", if (null.ok) " (or 'NULL')" else "", guessType(x)))

  # if (!is.null(response)) {
  #   qassert(response, "B1")

  #   if (response) {
  #     if (length(lhs.formula(f)) == 0L)
  #       return("Must have response (left hand side)")
  #   } else {
  #     if (length(lhs.formula(f)) > 0L)
  #       return("May not have a response (left hand side)")
  #   }
  # }

  return(TRUE)
}

# lhs.formula = function(x) {
#   if (length(x) == 2L) character(0L) else all.vars(x[[2L]])
# }

# rhs.formula = function(x) {
#   all.vars(x[[length(x)]])
# }


#' @export
#' @rdname checkFormula
check_formula = checkFormula

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkFormula
assertFormula = makeAssertionFunction(checkFormula, use.namespace = FALSE)

#' @export
#' @rdname checkFormula
assert_formula = assertFormula

#' @export
#' @include makeTest.R
#' @rdname checkFormula
testFormula = makeTestFunction(checkFormula)

#' @export
#' @rdname checkFormula
test_formula = testFormula

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkFormula
expect_formula = makeExpectationFunction(checkFormula, use.namespace = FALSE)

context("checkFormula")

test_that("checkFormula", {
  f = ~ x
  expect_succ_all(Formula, f)
  expect_fail_all(Formula, 1)

  f = y ~ x + z
  expect_true(checkFormula(f))

  f = y ~ x:z + I(a)
  expect_true(checkFormula(f))

  expect_true(checkFormula(NULL, null.ok = TRUE))
  expect_error(assertFormula(1, "formula"))
})

#' Check that an argument is a vector of type double
#'
#' @templateVar fn Double
#' @template x
#' @template na-handling
#' @inheritParams checkVector
#' @template bounds
#' @template sorted
#' @param finite [\code{logical(1)}]\cr
#'  Check for only finite values? Default is \code{FALSE}.
#' @template null.ok
#' @template typed.missing
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_double
#' @export
#' @examples
#' testDouble(1)
#' testDouble(1L)
#' testDouble(1, min.len = 1, lower = 0)
checkDouble = function(x, lower = -Inf, upper = Inf, finite = FALSE, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, sorted = FALSE, names = NULL, typed.missing = FALSE, null.ok = FALSE) {
  .Call(c_check_double, x, lower, upper, finite, any.missing, all.missing, len, min.len, max.len, unique, sorted, names, typed.missing, null.ok)
}

#' @export
#' @rdname checkDouble
check_double = checkDouble

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkDouble
assertDouble = makeAssertionFunction(checkDouble, c.fun = "c_check_double", use.namespace = FALSE)

#' @export
#' @rdname checkDouble
assert_double = assertDouble

#' @export
#' @include makeTest.R
#' @rdname checkDouble
testDouble = makeTestFunction(checkDouble, c.fun = "c_check_double")

#' @export
#' @rdname checkDouble
test_double = testDouble

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkDouble
expect_double = makeExpectationFunction(checkDouble, c.fun = "c_check_double", use.namespace = FALSE)

#' Check if an argument is TRUE
#'
#' @description
#' Simply checks if an argument is \code{TRUE}.
#'
#' @templateVar fn TRUE.
#' @template x
#' @template na.ok
#' @template checker
#' @export
#' @examples
#' testTRUE(TRUE)
#' testTRUE(FALSE)
checkTRUE = function(x, na.ok = FALSE) {
  qassert(na.ok, "B1")
  if (isTRUE(x) || (na.ok && length(x) == 1L && is.na(x)))
    return(TRUE)
  return("Must be TRUE")
}

#' @export
#' @rdname checkTRUE
check_true = checkTRUE

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkTRUE
assertTRUE = makeAssertionFunction(checkTRUE, use.namespace = FALSE)

#' @export
#' @rdname checkTRUE
assert_true = assertTRUE

#' @export
#' @include makeTest.R
#' @rdname checkTRUE
testTRUE = makeTestFunction(checkTRUE)

#' @export
#' @rdname checkTRUE
test_true = testTRUE

#' @title Lookup a variable name
#' @description
#' Tries to heuristically determine the variable name of \code{x} in the parent frame
#' with a combination of \code{\link[base]{deparse}} and \code{\link[base]{substitute}}.
#' Used for checkmate's error messages.
#' @param x [ANY]\cr
#'   Object.
#' @return [\code{character(1)}] Variable name.
#' @export
vname = function(x) {
  paste0(deparse(eval.parent(substitute(substitute(x))), width.cutoff = 500L), collapse = "\n")
}

#' Check that an argument is an atomic vector
#'
#' @description
#' An atomic vector is defined slightly different from specifications in
#' \code{\link[base]{is.atomic}} and \code{\link[base]{is.vector}}:
#' An atomic vector is either \code{logical}, \code{integer}, \code{numeric},
#' \code{complex}, \code{character} or \code{raw} and can have any attributes except a
#' dimension attribute (like matrices).
#' I.e., a \code{factor} is an atomic vector, but a matrix or \code{NULL} are not.
#' In short, this is basically equivalent to \code{is.atomic(x) && !is.null(x) && is.null(dim(x))}.
#'
#' @templateVar fn AtomicVector
#' @template x
#' @param any.missing [\code{logical(1)}]\cr
#'  Are vectors with missing values allowed? Default is \code{TRUE}.
#' @param all.missing [\code{logical(1)}]\cr
#'  Are vectors with only missing values allowed? Default is \code{TRUE}.
#' @param len [\code{integer(1)}]\cr
#'  Exact expected length of \code{x}.
#' @param min.len [\code{integer(1)}]\cr
#'  Minimal length of \code{x}.
#' @param max.len [\code{integer(1)}]\cr
#'  Maximal length of \code{x}.
#' @param unique [\code{logical(1)}]\cr
#'  Must all values be unique? Default is \code{FALSE}.
#' @param names [\code{character(1)}]\cr
#'  Check for names. See \code{\link{checkNamed}} for possible values.
#'  Default is \dQuote{any} which performs no check at all.
#' @template checker
#' @family basetypes
#' @family atomicvector
#' @useDynLib checkmate c_check_atomic_vector
#' @export
#' @examples
#' testAtomicVector(letters, min.len = 1L, any.missing = FALSE)
checkAtomicVector = function(x, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, names = NULL) {
  .Call(c_check_atomic_vector, x, any.missing, all.missing, len, min.len, max.len, unique, names)
}

#' @export
#' @rdname checkAtomicVector
check_atomic_vector = checkAtomicVector

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkAtomicVector
assertAtomicVector = makeAssertionFunction(checkAtomicVector, c.fun = "c_check_atomic_vector", use.namespace = FALSE)

#' @export
#' @rdname checkAtomicVector
assert_atomic_vector = assertAtomicVector

#' @export
#' @include makeTest.R
#' @rdname checkAtomicVector
testAtomicVector = makeTestFunction(checkAtomicVector, c.fun = "c_check_atomic_vector")

#' @export
#' @rdname checkAtomicVector
test_atomic_vector = testAtomicVector

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkAtomicVector
expect_atomic_vector = makeExpectationFunction(checkAtomicVector, c.fun = "c_check_atomic_vector", use.namespace = FALSE)

#' Check for existence and access rights of directories
#'
#' @note
#' The functions without the suffix \dQuote{exists} are deprecated and will be removed
#' from the package in a future version due to name clashes.
#'
#' @templateVar fn DirectoryExists
#' @template x
#' @inheritParams checkAccess
#' @inheritParams checkFile
#' @template checker
#' @family filesystem
#' @export
#' @examples
#' # Is R's home directory readable?
#' testDirectory(R.home(), "r")
#'
#' # Is R's home directory readable and writable?
#' testDirectory(R.home(), "rw")
checkDirectoryExists = function(x, access = "") {
  if (!qtest(x, "S+"))
    return("No directory provided")

  w = wf(!file.exists(x))
  if (length(w) > 0L)
    return(sprintf("Directory '%s' does not exist", x[w]))

  w = wf(!dir.exists(x))
  if (length(w) > 0L)
    return(sprintf("Directory expected, but file in place: '%s'", x[w]))

  checkAccess(x, access)
}

#' @export
#' @rdname checkDirectoryExists
check_directory_exists = checkDirectoryExists

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkDirectoryExists
assertDirectoryExists = makeAssertionFunction(checkDirectoryExists, use.namespace = FALSE)

#' @export
#' @rdname checkDirectoryExists
assert_directory_exists = assertDirectoryExists

#' @export
#' @include makeTest.R
#' @rdname checkDirectoryExists
testDirectoryExists = makeTestFunction(checkDirectoryExists)

#' @export
#' @rdname checkDirectoryExists
test_directory_exists = testDirectoryExists

#' @export
#' @rdname checkDirectoryExists
expect_directory_exists = makeExpectationFunction(checkDirectoryExists, use.namespace = FALSE)

#' @export
#' @rdname checkDirectoryExists
checkDirectory = checkDirectoryExists

#' @export
#' @rdname checkDirectoryExists
assertDirectory = assertDirectoryExists

#' @export
#' @rdname checkDirectoryExists
assert_directory = assert_directory_exists

#' @export
#' @rdname checkDirectoryExists
testDirectory = testDirectoryExists

#' @export
#' @rdname checkDirectoryExists
test_directory = test_directory_exists

#' @export
#' @rdname checkDirectoryExists
expect_directory = expect_directory_exists

#' Convert an argument to an integer
#'
#' @description
#' \code{asInteger} is intended to be used for vectors while \code{asInt} is
#' a specialization for scalar integers and \code{asCount} for scalar
#' non-negative integers.
#' Convertible are (a) atomic vectors with all elements \code{NA}
#' and (b) double vectors with all elements being within \code{tol}
#' range of an integer.
#'
#' Note that these functions may be deprecated in the future.
#' Instead, it is advised to use \code{\link{assertCount}},
#' \code{\link{assertInt}} or \code{\link{assertIntegerish}} with
#' argument \code{coerce} set to \code{TRUE} instead.
#'
#' @param x [any]\cr
#'  Object to convert.
#' @template na-handling
#' @inheritParams checkInteger
#' @inheritParams checkVector
#' @template tol
#' @template var.name
#' @return Converted \code{x}.
#' @export
#' @examples
#' asInteger(c(1, 2, 3))
#' asCount(1)
#' asInt(1)
asInteger = function(x, tol = sqrt(.Machine$double.eps), lower = -Inf, upper = Inf, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, unique = FALSE, sorted = FALSE, names = NULL, .var.name = vname(x)) {
  assertIntegerish(x, tol = tol, lower = lower, upper = upper, any.missing = any.missing, all.missing = all.missing, len = len, min.len = min.len, max.len = max.len, unique = unique, sorted = sorted, names = names, null.ok = FALSE, .var.name = .var.name)
  storage.mode(x) = "integer"
  x
}

#' @rdname asInteger
#' @param positive [\code{logical(1)}]\cr
#'  Must \code{x} be positive (>= 1)?
#'  Default is \code{FALSE}.
#' @template na.ok
#' @export
asCount = function(x, na.ok = FALSE, positive = FALSE, tol = sqrt(.Machine$double.eps), .var.name = vname(x)) {
  assertCount(x, na.ok, positive, tol, .var.name = .var.name)
  storage.mode(x) = "integer"
  x
}

#' @rdname asInteger
#' @template bounds
#' @export
asInt = function(x, na.ok = FALSE, lower = -Inf, upper = Inf, tol = sqrt(.Machine$double.eps), .var.name = vname(x)) {
  assertInt(x, na.ok, lower, upper, tol, .var.name = .var.name)
  storage.mode(x) = "integer"
  x
}

context("asType")

test_that("asInteger", {
  xi = 1:5
  xd = as.double(1:5)
  xc = as.complex(1:5)

  expect_equal(asInteger(xi), xi)
  expect_equal(asInteger(xd), xi)
  expect_equal(asInteger(xc), xi)
  expect_equal(asInteger(NA), NA_integer_)

  expect_equal(names(asInteger(xi)), names(xi))
  expect_equal(names(asInteger(xd)), names(xd))
  expect_equal(names(asInteger(xc)), names(xc))
  names(xi) = names(xd) = names(xc) = letters[1:5]
  expect_equal(names(asInteger(xi)), names(xi))
  expect_equal(names(asInteger(xd)), names(xd))
  expect_equal(names(asInteger(xc)), names(xc))

  y = "a"
  expect_error(asInteger(y), "'y'")
  expect_error(asInteger(3+1i))
  expect_error(asInteger(iris))
  expect_error(asInteger(NA, any.missing = FALSE), "missing")
})

test_that("asInt", {
  xi = 1L
  xd = 1.
  xc = as.complex(1)

  expect_equal(names(asInt(xi)), names(xi))
  expect_equal(names(asInt(xd)), names(xd))
  expect_equal(names(asInt(xc)), names(xc))
  names(xi) = names(xd) = names(xc) = "a"
  expect_equal(names(asInt(xi)), names(xi))
  expect_equal(names(asInt(xd)), names(xd))
  expect_equal(names(asInt(xc)), names(xc))

  expect_error(asInt(letters[1:2]), "integerish")
  expect_error(asInt(1:2), "length 1")
  expect_equal(asInt(xi), xi)
  expect_equal(asInt(xd), xi)
  expect_equal(asInt(xc), xi)
  expect_error(asInt(NA), "NA")
  expect_equal(asInt(NA, na.ok = TRUE), NA_integer_)

  y = "a"
  expect_error(asInt(y), "'y'")
  expect_error(asInt(3+1i))
  expect_error(asInt(iris))
  expect_error(asInt(xi, lower = 2), ">=")
})

test_that("asCount", {
  xi = 1L
  xd = 1.
  xc = as.complex(1)

  expect_equal(names(asCount(xi)), names(xi))
  expect_equal(names(asCount(xd)), names(xd))
  expect_equal(names(asCount(xc)), names(xc))
  names(xi) = names(xd) = names(xc) = "a"
  expect_equal(names(asCount(xi)), names(xi))
  expect_equal(names(asCount(xd)), names(xd))
  expect_equal(names(asCount(xc)), names(xc))

  expect_error(asCount(letters[1:2]), "count")
  expect_error(asCount(1:2), "length 1")
  expect_equal(asCount(xi), xi)
  expect_equal(asCount(xd), xi)
  expect_equal(asCount(xc), xi)
  expect_error(asCount(NA), "NA")
  expect_equal(asCount(NA, na.ok = TRUE), NA_integer_)

  y = "a"
  expect_error(asCount(y), "'y'")
  expect_error(asCount(3+1i))
  expect_error(asCount(iris))
  expect_error(asCount(0, positive = TRUE))
  expect_equal(asCount(1, positive = FALSE), 1L)
})

#' Check if an argument is an environment
#'
#' @templateVar fn Environment
#' @template x
#' @param contains [\code{character}]\cr
#'  Vector of object names expected in the environment.
#'  Defaults to \code{character(0)}.
#' @template null.ok
#' @template checker
#' @family basetypes
#' @export
#' @examples
#' ee = as.environment(list(a = 1))
#' testEnvironment(ee)
#' testEnvironment(ee, contains = "a")
checkEnvironment = function(x, contains = character(0L), null.ok = FALSE) {
  qassert(contains, "S")
  qassert(null.ok, "B1")
  if (is.null(x)) {
    if (null.ok)
      return(TRUE)
    return("Must be an environment, not 'NULL'")
  }
  if (!is.environment(x))
    return(sprintf("Must be an environment%s, not '%s'", if (null.ok) " (or 'NULL')" else "", guessType(x)))
  if (length(contains) > 0L) {
    w = wf(contains %nin% ls(x, all.names = TRUE))
    if (length(w) > 0L)
      return(sprintf("Must contain an object with name '%s'", contains[w]))
  }
  return(TRUE)
}

#' @export
#' @rdname checkEnvironment
check_environment = checkEnvironment

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkEnvironment
assertEnvironment = makeAssertionFunction(checkEnvironment, use.namespace = FALSE)

#' @export
#' @rdname checkEnvironment
assert_environment = assertEnvironment

#' @export
#' @include makeTest.R
#' @rdname checkEnvironment
testEnvironment = makeTestFunction(checkEnvironment)

#' @export
#' @rdname checkEnvironment
test_environment = testEnvironment

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkEnvironment
expect_environment = makeExpectationFunction(checkEnvironment, use.namespace = FALSE)

#' Check if an argument is a flag
#'
#' @description
#' A flag is defined as single logical value.
#'
#' @templateVar fn Flag
#' @template x
#' @template na-handling
#' @template na.ok
#' @template null.ok
#' @template checker
#' @family scalars
#' @useDynLib checkmate c_check_flag
#' @export
#' @examples
#' testFlag(TRUE)
#' testFlag(1)
checkFlag = function(x, na.ok = FALSE, null.ok = FALSE) {
  .Call(c_check_flag, x, na.ok, null.ok)
}

#' @export
#' @rdname checkFlag
check_flag = checkFlag

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkFlag
assertFlag = makeAssertionFunction(checkFlag, c.fun = "c_check_flag", use.namespace = FALSE)

#' @export
#' @rdname checkFlag
assert_flag = assertFlag

#' @export
#' @include makeTest.R
#' @rdname checkFlag
testFlag = makeTestFunction(checkFlag, c.fun = "c_check_flag")

#' @export
#' @rdname checkFlag
test_flag = testFlag

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkFlag
expect_flag = makeExpectationFunction(checkFlag, c.fun = "c_check_flag", use.namespace = FALSE)

#' Check if an argument is an array
#'
#' @templateVar fn Array
#' @template x
#' @template mode
#' @param any.missing [\code{logical(1)}]\cr
#'  Are missing values allowed? Default is \code{TRUE}.
#' @param d [\code{integer(1)}]\cr
#'  Exact number of dimensions of array \code{x}.
#'  Default is \code{NULL} (no check).
#' @param min.d [\code{integer(1)}]\cr
#'  Minimum number of dimensions of array \code{x}.
#'  Default is \code{NULL} (no check).
#' @param max.d [\code{integer(1)}]\cr
#'  Maximum number of dimensions of array \code{x}.
#'  Default is \code{NULL} (no check).
#' @template null.ok
#' @template checker
#' @family basetypes
#' @family compound
#' @useDynLib checkmate c_check_array
#' @export
#' @examples
#' checkArray(array(1:27, dim = c(3, 3, 3)), d = 3)
checkArray = function(x, mode = NULL, any.missing = TRUE, d = NULL, min.d = NULL, max.d = NULL, null.ok = FALSE) {
  .Call(c_check_array, x, mode, any.missing, d, min.d, max.d, null.ok)
}

#' @export
#' @rdname checkArray
check_array = checkArray

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkArray
assertArray = makeAssertionFunction(checkArray, c.fun = "c_check_array", use.namespace = FALSE)

#' @export
#' @rdname checkArray
assert_array = assertArray

#' @export
#' @include makeTest.R
#' @rdname checkArray
testArray = makeTestFunction(checkArray, c.fun = "c_check_array")

#' @export
#' @rdname checkArray
test_array = testArray

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkArray
expect_array = makeExpectationFunction(checkArray, c.fun = "c_check_array", use.namespace = FALSE)

#' Check names to comply to specific rules
#'
#' @description
#' Performs various checks on character vectors, usually names.
#'
#' @templateVar fn Names
#' @param x [\code{character} || \code{NULL}]\cr
#'  Names to check using rules defined via \code{type}.
#' @param type [character(1)]\cr
#'  Type of formal check(s) to perform on the names.
#'  \describe{
#'  \item{unnamed:}{Checks \code{x} to be \code{NULL}.}
#'  \item{named:}{Checks \code{x} for regular names which excludes names to be \code{NA} or empty (\code{""}).}
#'  \item{unique:}{Performs checks like with \dQuote{named} and additionally tests for non-duplicated names.}
#'  \item{strict:}{Performs checks like with \dQuote{unique} and additionally fails for names with UTF-8 characters and names which do not comply to R's variable name restrictions. As regular expression, this is \dQuote{^[.]*[a-zA-Z]+[a-zA-Z0-9._]*$}.}
#'  \item{ids:}{Same as \dQuote{strict}, but does not enforce uniqueness.}
#'  }
#'  Note that for zero-length \code{x}, all these name checks evaluate to \code{TRUE}.
#' @param subset.of [\code{character}]\cr
#'  Names provided in \code{x} must be subset of the set \code{subset.of}.
#' @param must.include [\code{character}]\cr
#'  Names provided in \code{x} must be a superset of the set \code{must.include}.
#' @param permutation.of [\code{character}]\cr
#'  Names provided in \code{x} must be a permutation of the set \code{permutation.of}.
#'  Duplicated names in \code{permutation.of} are stripped out and duplicated names in \code{x}
#'  thus lead to a failed check.
#'  Use this argument instead of \code{identical.to} if the order of the names is not relevant.
#' @param identical.to [\code{character}]\cr
#'  Names provided in \code{x} must be identical to the vector \code{identical.to}.
#'  Use this argument instead of \code{permutation.of} if the order of the names is relevant.
#' @param disjunct.from [\code{character}]\cr
#'  Names provided in \code{x} must may not be present in the vector \code{disjunct.from}.
#' @param what [\code{character(1)}]\cr
#'  Type of name vector to check, e.g. \dQuote{names} (default), \dQuote{colnames} or \dQuote{rownames}.
#' @template checker
#' @useDynLib checkmate c_check_names
#' @family attributes
#' @export
#' @examples
#' x = 1:3
#' testNames(names(x), "unnamed")
#' names(x) = letters[1:3]
#' testNames(names(x), "unique")
#'
#' cn = c("Species", "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
#' assertNames(names(iris), permutation.of = cn)
checkNames = function(x, type = "named", subset.of = NULL, must.include = NULL, permutation.of = NULL, identical.to = NULL, disjunct.from = NULL, what = "names") {
  .Call(c_check_names, x, type, what) %and%
    checkNamesCmp(x, subset.of, must.include, permutation.of, identical.to, disjunct.from, what)
}

checkNamesCmp = function(x, subset.of, must.include, permutation.of, identical.to, disjunct.from, what) {
  if (!is.null(subset.of)) {
    qassert(subset.of, "S")
    msg = check_subset_internal(x, subset.of, match, what)
    if (!isTRUE(msg))
      return(msg)
  }

  if (!is.null(must.include)) {
    qassert(must.include, "S")
    ii = match(must.include, x)
    if (anyMissing(ii)) {
      return(set_msg("must include the elements %s, but is missing elements %s",
        what,
        set_collapse(must.include),
        set_collapse(must.include[is.na(ii)])
      ))
    }
  }

  if (!is.null(permutation.of)) {
    permutation.of = unique(qassert(permutation.of, "S"))
    msg = check_set_equal_internal(x, permutation.of, match, what)
    if (!isTRUE(msg))
      return(msg)
  }

  if (!is.null(identical.to)) {
    qassert(identical.to, "S")
    if (!identical(x, identical.to)) {
      return(set_msg("must be a identical to set %s, but is %s",
        what,
        set_collapse(identical.to),
        set_collapse(x)
      ))
    }
  }

  if (!is.null(disjunct.from)) {
    qassert(disjunct.from, "S")
    msg = check_disjunct_internal(x, disjunct.from, match, what)
    if (!isTRUE(msg))
      return(msg)
  }

  return(TRUE)
}

#' @export
#' @rdname checkNames
check_names = checkNames

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkNames
assertNames = makeAssertionFunction(checkNames, use.namespace = FALSE)

#' @export
#' @rdname checkNames
assert_names = assertNames

#' @export
#' @include makeTest.R
#' @rdname checkNames
testNames = makeTestFunction(checkNames)

#' @export
#' @rdname checkNames
test_names = testNames

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkNames
expect_names = makeExpectationFunction(checkNames, use.namespace = FALSE)

context("checkNames")

test_that("checkNames", {
  nn = letters[1:3]
  expect_succ_all(Names, nn)
  expect_fail_all(Names, nn, type = "unnamed")

  expect_true(testNames(character(0)))
  expect_false(testNames(NULL))
  expect_false(testNames(integer(0)))

  x = c("a", ".a")
  expect_true(testNames(x))
  expect_true(testNames(x, "unique"))
  expect_true(testNames(x, "strict"))

  expect_false(testNames(1))
  expect_false(testNames(NA_character_))
  expect_false(testNames(NA_integer_))
  expect_false(testNames(""))

  x = c("a", "a")
  expect_true(testNames(x))
  expect_false(testNames(x, "unique"))

  expect_true(testNames("x", type = "strict"))
  expect_true(testNames("..x", type = "strict"))
  expect_true(testNames("x_1", type = "strict"))
  expect_true(testNames("x.", type = "strict"))
  expect_false(testNames("1", type = "strict"))
  expect_false(testNames(".1", type = "strict"))
  expect_false(testNames("..1", type = "strict"))
  expect_false(testNames("x ", type = "strict"))
  expect_false(testNames("\u00e4", type = "strict"))
  expect_false(testNames("\u00e4", type = "ids"))
  expect_error(assertNames(c("a", "a"), "unique"), "unique")

  expect_error(assertNames(c("a", "a"), "strict"), "unique")
  expect_true(testNames(c("a", "a"), "ids"))

  x = c("a", "1")
  expect_error(assertNames(x, "strict"), "naming conventions")
})

test_that("argument 'type' is checked", {
  expect_error(checkNames("x", type = 1), "string")
  expect_error(checkNames("x", type = NA_character_), "missing")
})


test_that("checkNames / subset.of", {
  x = 1:3
  names(x) = letters[1:3]

  expect_true(testNames(names(x), subset.of = letters[1:3]))
  expect_true(testNames(names(x), subset.of = letters[3:1]))
  expect_true(testNames(names(x), subset.of = letters))
  expect_false(testNames(names(x), subset.of = letters[1:2]))
  expect_false(testNames(names(x), subset.of = character(0)))
  expect_false(testNames(NULL, subset.of = character(0)))
  expect_true(testNames(character(0), subset.of = character(0)))
  expect_true(testNames(character(0), subset.of = NULL))

  expect_error(
    assert_names(names(x), subset.of = c("a", "b")),
    "{'c'}", fixed = TRUE
  )
})

test_that("checkNames / identical.to", {
  x = 1:3
  names(x) = letters[1:3]

  expect_true(testNames(names(x), identical.to = letters[1:3]))
  expect_false(testNames(names(x), identical.to = letters[3:1]))
  expect_false(testNames(names(x), identical.to = letters))
  expect_false(testNames(names(x), identical.to = letters[1:2]))
  expect_false(testNames(names(x), identical.to = character(0)))
  expect_false(testNames(NULL, identical.to = character(0)))
  expect_true(testNames(character(0), identical.to = character(0)))
  expect_true(testNames(character(0), identical.to = NULL))
  expect_false(testNames(NULL, identical.to = NULL))

  expect_error(
    assert_names(names(x), identical.to = c("a", "b")),
    "{'a','b','c'}", fixed = TRUE
  )
})

test_that("checkNames / permutation.of", {
  x = 1:3
  names(x) = letters[1:3]

  expect_true(testNames(names(x), permutation.of = letters[1:3]))
  expect_true(testNames(names(x), permutation.of = letters[3:1]))
  expect_false(testNames(names(x), permutation.of = letters))
  expect_false(testNames(names(x), permutation.of = letters[1:2]))
  expect_false(testNames(names(x), permutation.of = character(0)))
  expect_false(testNames(NULL, permutation.of = character(0)))
  expect_true(testNames(character(0), permutation.of = character(0)))
  expect_true(testNames(character(0), permutation.of = NULL))
  expect_false(testNames(NULL, permutation.of = NULL))

  expect_error(
    assert_names(names(x), permutation.of = c("a", "b")),
    "{'c'}", fixed = TRUE
  )

  expect_error(
    assert_names(names(x), permutation.of = c("a", "b", "c", "d")),
    "{'d'}", fixed = TRUE
  )
})

test_that("checkNames / must.include", {
  x = 1:3
  names(x) = letters[1:3]

  expect_true(testNames(names(x), must.include = "a"))
  expect_true(testNames(names(x), must.include = letters[3:1]))
  expect_false(testNames(names(x), must.include = letters))
  expect_true(testNames(names(x), must.include = character(0)))
  expect_false(testNames(NULL, must.include = character(0)))
  expect_true(testNames(character(0), must.include = character(0)))
  expect_true(testNames(character(0), must.include = NULL))

  expect_error(
    assert_names(names(x), must.include = c("a", "b", "c", "d")),
    "{'d'}", fixed = TRUE
  )
})

test_that("checkNames / disjunct.from", {
  x = 1:3
  names(x) = letters[1:3]

  expect_true(testNames(names(x)))
  expect_true(testNames(names(x), disjunct.from = "d"))
  expect_false(testNames(names(x), disjunct.from = "b"))

  expect_error(
    assert_names(names(x), disjunct.from = c("c")),
    "{'c'}", fixed = TRUE
  )
})

test_that("checkNames / errors are useful", {
  foo = matrix(1:9)
  expect_error(
    assertNames(colnames(foo), permutation.of = letters),
    "colnames\\(foo\\)"
  )
  expect_error(
    assertNames(rownames(foo), permutation.of = letters),
    "rownames\\(foo\\)"
  )
})

test_that("checkNames / NULL (#120)", {
  expect_true(testNames(NULL, type = "unnamed"))
  expect_false(testNames(NULL, type = "named"))
})

#' Check the operating system
#'
#' @templateVar fn OS
#' @param os [\code{character(1)}]\cr
#'  Check the operating system to be in a set with possible elements \dQuote{windows},
#'  \dQuote{mac}, \dQuote{linux} and \dQuote{solaris}.
#' @template checker
#' @export
#' @examples
#' testOS("linux")
checkOS = function(os) {
  ok = match.arg(os, c("windows", "mac", "linux", "solaris"), several.ok = TRUE)
  if (checkmate$os %nin% ok)
    return(sprintf("OS must be %s", paste0(ok, collapse = " or ")))
  return(TRUE)
}

#' @export
#' @rdname checkOS
check_os = checkOS

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkOS
assertOS = function(os, add = NULL, .var.name = NULL) {
  res = checkOS(os)
  makeAssertion(os, res, .var.name %??% "Operating System", add)
}

#' @export
#' @rdname checkOS
assert_os = assertOS

#' @export
#' @include makeTest.R
#' @rdname checkOS
testOS = makeTestFunction(checkOS)

#' @export
#' @rdname checkOS
test_os = testOS

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkOS
expect_os = function(os, info = NULL, label = NULL) {
  res = checkOS(os)
  makeExpectation(checkmate$os, res, info, label = label %??% "Operating System")
}

#' Check file system access rights
#'
#' @templateVar fn Access
#' @template x
#' @param access [\code{character(1)}]\cr
#'  Single string containing possible characters \sQuote{r}, \sQuote{w} and \sQuote{x} to
#'  force a check for read, write or execute access rights, respectively.
#'  Write and executable rights are not checked on Windows.
#' @template checker
#' @family filesystem
#' @export
#' @examples
#' # Is R's home directory readable?
#' testAccess(R.home(), "r")
#'
#' # Is R's home directory writeable?
#' testAccess(R.home(), "w")
checkAccess = function(x, access = "") {
  qassert(access, "S1")
  if (nzchar(access)) {
    access = strsplit(access, "")[[1L]]
    if (anyDuplicated(access) > 0L || !all(access %in% c("r", "w", "x")))
      stop("Access pattern invalid, allowed are 'r', 'w' and 'x'")

    is.win = .Platform$OS.type == "windows"
    is.root = (!is.win && Sys.info()["user"] == "root")

    if ("r" %in% access || is.root) {
      w = wf(file.access(x, 4L) != 0L)
      if (length(w) > 0L)
        return(sprintf("'%s' not readable", x[w]))
    }

    if (!is.win) {
      if ("w" %in% access || is.root) {
        w = wf(file.access(x, 2L) != 0L)
        if (length(w) > 0L)
          return(sprintf("'%s' not writeable", x[w]))
      }

      if ("x" %in% access) {
        w = wf(file.access(x, 1L) != 0L)
        if (length(w) > 0L)
          return(sprintf("'%s' not executable", x[w]))
      }
    }
  }
  return(TRUE)
}

#' @export
#' @rdname checkAccess
check_access = checkAccess

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkAccess
assertAccess = makeAssertionFunction(checkAccess, use.namespace = FALSE)

#' @export
#' @rdname checkAccess
assert_access = assertAccess

#' @export
#' @include makeTest.R
#' @rdname checkAccess
testAccess = makeTestFunction(checkAccess)

#' @export
#' @rdname checkAccess
test_access = testAccess

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkAccess
expect_access = makeExpectationFunction(checkAccess, use.namespace = FALSE)

#' Check if an argument is a raw vector
#'
#' @templateVar fn Raw
#' @template x
#' @param len [\code{integer(1)}]\cr
#'  Exact expected length of \code{x}.
#' @param min.len [\code{integer(1)}]\cr
#'  Minimal length of \code{x}.
#' @param max.len [\code{integer(1)}]\cr
#'  Maximal length of \code{x}.
#' @param names [\code{character(1)}]\cr
#'  Check for names. See \code{\link{checkNamed}} for possible values.
#'  Default is \dQuote{any} which performs no check at all.
#'  Note that you can use \code{\link{checkSubset}} to check for a specific set of names.
#' @template null.ok
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_raw
#' @export
#' @examples
#' testRaw(as.raw(2), min.len = 1L)
checkRaw = function(x, len = NULL, min.len = NULL, max.len = NULL, names = NULL, null.ok = FALSE) {
  .Call(c_check_raw, x, len, min.len, max.len, names, null.ok)
}

#' @export
#' @rdname checkRaw
check_raw = checkRaw

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkRaw
assertRaw = makeAssertionFunction(checkRaw, c.fun = "c_check_raw", use.namespace = FALSE)

#' @export
#' @rdname checkRaw
assert_raw = assertRaw

#' @export
#' @include makeTest.R
#' @rdname checkRaw
testRaw = makeTestFunction(checkRaw, c.fun = "c_check_raw")

#' @export
#' @rdname checkRaw
test_raw = testRaw

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkRaw
expect_raw = makeExpectationFunction(checkRaw, c.fun = "c_check_raw", use.namespace = FALSE)

#' Check if an argument is a factor
#'
#' @templateVar fn Factor
#' @template x
#' @inheritParams checkVector
#' @param levels [\code{character}]\cr
#'  Vector of allowed factor levels.
#' @param ordered [\code{logical(1)}]\cr
#'  Check for an ordered factor? If \code{FALSE} or \code{TRUE}, checks explicitly
#'  for an unordered or ordered factor, respectively.
#'  Default is \code{NA} which does not perform any additional check.
#' @param empty.levels.ok [\code{logical(1)}]\cr
#'  Are empty levels allowed?
#'  Default is \code{TRUE}.
#' @param n.levels [\code{integer(1)}]\cr
#'  Exact number of factor levels.
#'  Default is \code{NULL} (no check).
#' @param min.levels [\code{integer(1)}]\cr
#'  Minimum number of factor levels.
#'  Default is \code{NULL} (no check).
#' @param max.levels [\code{integer(1)}]\cr
#'  Maximum number of factor levels.
#'  Default is \code{NULL} (no check).
#' @template null.ok
#' @template checker
#' @family basetypes
#' @useDynLib checkmate c_check_factor
#' @export
#' @examples
#' x = factor("a", levels = c("a", "b"))
#' testFactor(x)
#' testFactor(x, empty.levels.ok = FALSE)
checkFactor = function(x, levels = NULL, ordered = NA, empty.levels.ok = TRUE, any.missing = TRUE, all.missing = TRUE, len = NULL, min.len = NULL, max.len = NULL, n.levels = NULL, min.levels = NULL, max.levels = NULL, unique = FALSE, names = NULL, null.ok = FALSE) {
  .Call(c_check_factor, x, any.missing, all.missing, len, min.len, max.len, unique, names, null.ok) %and%
    checkFactorLevels(x, levels, ordered, empty.levels.ok, n.levels, min.levels, max.levels)
}

checkFactorLevels = function(x , levels = NULL, ordered = NA, empty.levels.ok = TRUE, n.levels = NULL, min.levels = NULL, max.levels = NULL) {
  if (!is.null(x)) {
    if (!is.null(levels)) {
      qassert(levels, "S")
      if (!setequal(levels(x), levels))
        return(sprintf("Must have levels: %s", paste0(levels, collapse = ",")))
    }
    qassert(ordered, "b1")
    if (!is.na(ordered)) {
      x.ordered = is.ordered(x)
      if (ordered && !x.ordered)
        return("Must be an ordered factor, but is unordered")
      else if (!ordered && x.ordered)
        return("Must be an unordered factor, but is ordered")
    }
    qassert(empty.levels.ok, "B1")
    if (!empty.levels.ok) {
      empty = setdiff(levels(x), x)
      if (length(empty) > 0L)
        return(sprintf("Has has empty levels '%s'", paste0(empty, collapse = "','")))
    }
    if (!is.null(n.levels)) {
      qassert(n.levels, "X1")
      if (length(levels(x)) != n.levels)
        return(sprintf("Must have exactly %i levels", n.levels))
    }
    if (!is.null(min.levels)) {
      qassert(min.levels, "X1")
      if (length(levels(x)) < min.levels)
        return(sprintf("Must have at least %i levels", min.levels))
    }
    if (!is.null(max.levels)) {
      qassert(max.levels, "X1")
      if (length(levels(x)) > max.levels)
        return(sprintf("Must have at most %i levels", max.levels))
    }
  }
  return(TRUE)
}

#' @export
#' @rdname checkFactor
check_factor = checkFactor

#' @export
#' @include makeAssertion.R
#' @template assert
#' @rdname checkFactor
assertFactor = makeAssertionFunction(checkFactor, use.namespace = FALSE)

#' @export
#' @rdname checkFactor
assert_factor = assertFactor

#' @export
#' @include makeTest.R
#' @rdname checkFactor
testFactor = makeTestFunction(checkFactor)

#' @export
#' @rdname checkFactor
test_factor = testFactor

#' @export
#' @include makeExpectation.R
#' @template expect
#' @rdname checkFactor
expect_factor = makeExpectationFunction(checkFactor, use.namespace = FALSE)

context("generated messages")

test_that("No extra strings attached to generated error messages", {
  foo = function(XX) assertFlag(XX)
  x = try(foo(iris), silent = TRUE)
  expect_error(foo(iris), "^Assertion on 'XX'")
  expect_error(foo(iris), "not 'data.frame'\\.$")
})

test_that("Correct frame is reported by assertions", {
  f1a <- function(num) if(!is.numeric(num)) stop("Not numeric")
  f1b <- function(num) assert_numeric(num)
  f2a <- function(num) f1a(num)
  f2b <- function(num) f1b(num)
  ok.r = try(f2a(letters), silent = TRUE)
  ok.cm = try(f2b(letters), silent = TRUE)

  expect_true(grepl("f1a(num)", as.character(ok.r), fixed = TRUE))
  expect_true(grepl("f1b(num)", as.character(ok.cm), fixed = TRUE))
})

context("checkNull")

test_that("checkNull", {
  myobj = NULL
  expect_succ_all(Null, myobj)
  myobj = TRUE
  expect_fail_all(Null, myobj)

  expect_false(testNull(integer(0)))
  expect_true(testNull(NULL))

  expect_error(assertNull(-1), "NULL")
})

context("checkList")

test_that("checkList", {
  myobj = list(1, 2, 3)
  expect_succ_all(List, myobj)
  myobj = TRUE
  expect_fail_all(List, myobj)

  expect_true(testList(list()))
  expect_false(testList(NULL))
  expect_true(testList(list(1)))
  expect_false(testList(iris))

  x = as.list(iris)
  expect_true(testList(x, types = c("numeric", "factor")))
  expect_false(testList(x, types = c("integer", "factor")))
  expect_false(testList(x, types = c("numeric", "character")))
  expect_true(testList(x, types = c("vector", "factor")))
  expect_true(testList(list(NULL), types = "NULL"))
  expect_true(testList(list(), types = "numeric"))
  expect_false(testList(list(TRUE), types = "numeric"))
  expect_error(assertList(x, types = "numeric"), "numeric")
  expect_error(assertList(x, len = 33), "Must have length 33")

  expect_true(testList(list(), names = "named"))

  x = list(1, NULL)
  expect_true(testList(x))
  expect_false(testList(x, any.missing = FALSE))

  x = 1:3
  class(x) = "foo"
  x = list(x, 1:3)
  expect_true(testList(x, types = c("foo", "integerish")))
  expect_error(assertList(1), "list")
})

test_that("missingness in lists", {
  x = list(1, NULL)
  expect_true(testList(x))
  expect_true(testList(x, all.missing = FALSE))
  expect_false(testList(x, any.missing = FALSE))

  x = list(NULL)
  expect_true(testList(x))
  expect_false(testList(x, all.missing = FALSE))
  expect_false(testList(x, any.missing = FALSE))
})

context("checkClass")

test_that("checkClass", {
  myobj = 1
  expect_succ_all(Class, myobj, "numeric")
  expect_fail_all(Class, myobj, "integer")

  expect_true(testClass(NULL, "NULL"))
  expect_false(testClass(NULL, ""))
  expect_true(testClass(1, "numeric"))
  expect_true(testClass(1L, "integer"))
  expect_false(testClass(1, "integer"))

  foo = 1
  class(foo) = c("a", "b")
  expect_true(testClass(foo, "a"))
  expect_true(testClass(foo, "b"))
  expect_false(testClass(foo, "c"))
  expect_true(testClass(foo, "a", ordered=TRUE))
  expect_false(testClass(foo, "b", ordered=TRUE))
  expect_true(testClass(foo, c("a", "b"), ordered=TRUE))
  expect_false(testClass(foo, c("b", "a"), ordered=TRUE))

  foo = 1
  class(foo) = c("a", "b")
  expect_error(assertClass(foo, "c"), "class 'c', but has classes 'a','b'")
  expect_error(assertClass(foo, "b", ordered=TRUE), "Must have class 'b' in position 1, but has classes 'a','b'")

  foo = 1
  class(foo) = "a"
  expect_error(assertClass(foo, "c"), "class 'c', but has class 'a'")
})

context("checkDataTable")

test_that("checkDataTable", {
  skip_if_not_physically_installed("data.table")

  expect_false(testDataTable(iris))
  expect_true("data.table" %in% loadedNamespaces())

  dt = data.table::as.data.table(iris)
  expect_succ_all("DataFrame", dt)
  expect_succ_all("DataTable", dt)
  expect_fail_all("DataTable", iris)

  expect_true(testDataTable(dt, min.rows = 1, ncols = 5))
  expect_false(testDataTable(dt, min.rows = 1000, ncols = 5))

  expect_true(testDataTable(dt, key = character(0)))
  expect_true(testDataTable(dt, index = character(0)))

  data.table::setkeyv(dt, "Species")
  expect_true(testDataTable(dt, key = "Species"))
  expect_false(testDataTable(dt, index = "Species"))

  dt = data.table::as.data.table(iris)
  data.table::setkeyv(dt, "Species", physical = FALSE)
  expect_false(testDataTable(dt, key = "Species"))
  expect_true(testDataTable(dt, index = "Species"))

  dt = data.table::as.data.table(iris)
  data.table::setkeyv(dt, c("Petal.Width", "Petal.Length"), physical = TRUE)
  data.table::setkeyv(dt, c("Sepal.Length", "Sepal.Width"), physical = FALSE)
  expect_true(testDataTable(dt, key = c("Petal.Width", "Petal.Length"), index = c("Sepal.Width", "Sepal.Length")))

  expect_error(testDataTable(dt, key = 1), "string")
  expect_error(testDataTable(dt, index = 1), "string")
  expect_error(assertDataTable(dt, key = "Species"), "primary keys")
  expect_error(assertDataTable(dt, index = "Species"), "secondary keys")

  x = data.table::as.data.table(iris)
  expect_true(testDataTable(x, max.rows = 200, max.cols = 5))
  expect_false(testDataTable(x, max.rows = 100))
  expect_false(testDataTable(x, max.cols = 3))
})

test_that("list columns", {
  skip_if_not_physically_installed("data.table")

  x = data.table::data.table(a = 1:2, b = list(3, 4))
  expect_true(testDataTable(x, any.missing = FALSE))

  x$b = list(3, NULL)
  expect_true(testDataTable(x, any.missing = FALSE))
  expect_true(testDataTable(x, all.missing = FALSE))

  x$b = list(NULL, NULL)
  expect_true(testDataTable(x, any.missing = FALSE))
  expect_true(testDataTable(x, all.missing = FALSE))
})

test_that("nrow for null data tables", {
  # c.f. https://github.com/Rdatatable/data.table/issues/3149
  M = matrix(1:3, nrow = 3)
  M = M[, integer(0)]
  DT = data.table::as.data.table(M) # null data table

  expect_true(testDataTable(DT, nrows = 0))
})

context("checkComplex")

test_that("checkComplex", {
  myobj = 1+1i
  expect_succ_all(Complex, myobj)
  myobj = 1
  expect_fail_all(Complex, myobj)

  expect_true(testComplex(complex(0L)))
  expect_false(testComplex(NULL))
  expect_false(testComplex(TRUE))
  expect_true(testComplex(NA))
  expect_false(testComplex(NA, any.missing = FALSE))
  expect_false(testComplex(NA, all.missing = FALSE))
  expect_true(testComplex(NA_complex_))
  expect_true(testComplex(1+1i))
  expect_true(testComplex(as.complex(Inf)))
  expect_true(testComplex(c(1+1i, 2+1i), any.missing = FALSE, min.len = 1L, max.len = 3L))

  expect_error(assertComplex(1), "complex")
})

test_that("typed.missing", {
  expect_true(testComplex(NA))
  expect_true(testComplex(NA, typed.missing = FALSE))
  expect_false(testComplex(NA, typed.missing = TRUE))

  expect_true(testComplex(character()))
  expect_true(testComplex(character(), typed.missing = FALSE))
  expect_false(testComplex(character(), typed.missing = TRUE))
})

context("checkAtomic")

li = list(
  list = list(1, 2),
  factor = factor("a"),
  integer = 1:2,
  NULL = NULL,
  data.frame = iris
)

test_that("checkAtomic", {
  myobj = 1:2
  expect_succ_all(Atomic, myobj)
  myobj = iris
  expect_fail_all(Atomic, myobj)

  expect_true(testAtomic(integer(0)))
  expect_true(testAtomic(NULL))
  expect_true(testAtomic(1))
  expect_true(testAtomic(integer(0)))
  expect_true(testAtomic(factor(1)))

  expect_true(testAtomic(NA, any.missing = TRUE))
  expect_false(testAtomic(NA, any.missing = FALSE))
  expect_false(testAtomic(NA, all.missing = FALSE))

  expect_true(testAtomic(1, len=1))
  expect_false(testAtomic(1, len=0))

  expect_true(testAtomic(1, min.len=0))
  expect_false(testAtomic(1, min.len=2))
  expect_true(testAtomic(1, max.len=1))
  expect_false(testAtomic(1, max.len=0))

  expect_true(testAtomic(1, unique=TRUE))
  expect_false(testAtomic(1, min.len=2))
  expect_true(testAtomic(1, max.len=1))
  expect_false(testAtomic(1, max.len=0))

  expect_true(testAtomic(1, unique=TRUE))
  expect_true(testAtomic(c(1,1), unique=FALSE))
  expect_false(testAtomic(c(1,1), unique=TRUE))

  expect_true(testAtomic(1, names="unnamed"))
  expect_true(testAtomic(setNames(1, "x"), names="named"))
  expect_false(testAtomic(1, names="unique"))

  expect_error(assertAtomic(iris), "atomic")

  # handling of is.null for future versions of R, see
  # https://stat.ethz.ch/pipermail/r-devel/2023-September/082892.html
  expect_equal(vlapply(li, function(x) is.atomic(x) || is.null(x)), vlapply(li, testAtomic))
})

context("checkScalarNA")

test_that("checkScalarNA", {
  expect_succ_all("ScalarNA", NA)
  expect_fail_all("ScalarNA", 1)
  expect_true(testScalarNA(NA_real_))
  expect_false(testScalarNA(1))
  expect_false(testScalarNA(rep(NA_character_, 2)))
  expect_expectation_successful(expect_scalar_na(NA), label = NULL)

  expect_error(assertScalarNA(integer(0)), "missing value")
})

test_that("checkScalarNA on data.frame/data.table (#245)", {
  df = data.frame(x = 1:2)
  expect_false(testScalarNA(df))
})

context("checkLogical")

test_that("checkLogical", {
  myobj = TRUE
  expect_succ_all(Logical, myobj)
  myobj = 1
  expect_fail_all(Logical, myobj)

  expect_true(testLogical(logical(0)))
  expect_false(testLogical(NULL))
  expect_true(testLogical(TRUE))
  expect_true(testLogical(NA))
  expect_true(testLogical(NA_real_))
  expect_true(testLogical(FALSE))
  expect_false(testLogical(NA, any.missing=FALSE))
  expect_false(testLogical(NA, all.missing=FALSE))
  expect_false(testLogical(iris))
  expect_true(testLogical(c(TRUE, FALSE), min.len = 2))

  expect_error(assertLogical(1), "logical")
})

test_that("typed.missing", {
  expect_true(testLogical(NA_character_))
  expect_true(testLogical(NA_character_, typed.missing = FALSE))
  expect_false(testLogical(NA_character_, typed.missing = TRUE))

  expect_true(testLogical(character()))
  expect_true(testLogical(character(), typed.missing = FALSE))
  expect_false(testLogical(character(), typed.missing = TRUE))
})

context("checkString")

test_that("checkString", {
  expect_succ_all(String, "a")
  expect_succ_all(String, "")
  expect_fail_all(String, "", min.chars = 1)
  expect_fail_all(String, 1L)
  expect_succ_all(String, NA_character_, na.ok = TRUE, min.chars = 99)
  expect_succ_all(String, "xxxfooxxx", pattern = "foo")
  expect_succ_all(String, "xxxfooxxx", fixed = "foo")
  expect_fail_all(String, "xxxfooxxx", pattern = "bar")
  expect_fail_all(String, "xxxfooxxx", fixed = "bar")
  expect_succ_all(String, "xxxfooxxx", pattern = "FOO", ignore.case = TRUE)
  expect_fail_all(String, "xxxfooxxx", fixed = "FOO", ignore.case = FALSE)

  expect_false(testString(character(0)))
  expect_false(testString(NULL))
  expect_true(testString(""))
  expect_true(testString("foo"))
  expect_true(testString(NA, na.ok = TRUE))
  expect_false(testString(NA_character_))
  expect_true(testString(NA_character_, na.ok = TRUE))
  expect_true(testString(NA, na.ok = TRUE))
  expect_true(testString("a", min.chars = 1))
  expect_false(testString("", min.chars = 1))
  expect_true(testString(NA_character_, min.chars = 1, na.ok = TRUE))
  expect_true(testString(NA_real_, min.chars = 1, na.ok = TRUE))
  expect_error(assertString(1))
})

context("Interoperability")

test_that("data.table is supported", {
  skip_if_not_installed("data.table")
  library(data.table)
  myobj = as.data.table(iris)
  expect_succ_all(DataFrame, myobj)
  expect_true(testDataFrame(myobj, nrows = 150, min.cols = 2, any.missing = FALSE, col.names = "strict"))
  expect_true(testDataFrame(data.table()))
})

test_that("tibble is supported", {
  skip_if_not_installed("tibble")
  library(tibble)
  myobj = as_tibble(iris)
  expect_succ_all(DataFrame, myobj)
  expect_true(testDataFrame(myobj, nrows = 150, min.cols = 2, any.missing = FALSE, col.names = "strict"))
  expect_true(testDataFrame(tibble()))
})

test_that("magrittr is supported", {
  skip_if_not_installed("magrittr")
  library(magrittr)
  x = runif(10)
  expect_identical(x %>% assert_numeric(lower = 0, upper = 1), x)
  expect_identical(iris %>% assert_data_frame(min.rows = 1) %>% ncol, 5L)
})

context("checkEnvironment")

test_that("checkEnvironment", {
  myobj = new.env()
  expect_succ_all(Environment, myobj)
  myobj = list()
  expect_fail_all(Environment, myobj)

  ee = new.env(parent = emptyenv())
  ee$yyy = 1
  ee$zzz = 1

  expect_false(testEnvironment(NULL))
  expect_false(testEnvironment(list()))
  expect_true(testEnvironment(ee))

  expect_false(testEnvironment(ee, contains = "xxx"))
  expect_true(testEnvironment(ee, contains = "yyy"))
  expect_true(testEnvironment(ee, contains = c("yyy", "zzz")))

  expect_error(assertEnvironment(list()), "environment")
  expect_error(assertEnvironment(ee, "xxx"), "with name")
  expect_error(assertEnvironment(letters), "character")
})

context("checkScalar")

test_that("checkScalar", {
  myobj = "a"
  expect_succ_all(Scalar, myobj)
  myobj = 1:2
  expect_fail_all(Scalar, myobj)

  expect_true(testScalar(TRUE))
  expect_true(testScalar(1L))
  expect_true(testScalar(1))
  expect_true(testScalar(1+1i))
  expect_false(testScalar(list(1)))
  expect_false(testScalar(NA, na.ok = FALSE))
  expect_true(testScalar(NA, na.ok = TRUE))

  expect_error(assertScalar(integer(0)), "length 1")
  expect_error(assertScalar(iris), "scalar")
})

context("checkMultiClass")

test_that("checkMultiClass", {
  myobj = 1
  expect_succ_all(MultiClass, myobj, "numeric", cc = "MultiClass", lc = "multi_class")
  expect_fail_all(MultiClass, myobj, "integer")

  foo = 1
  class(foo) = c("a", "b")
  expect_true(testMultiClass(foo, "a"))
  expect_true(testMultiClass(foo, "b"))
  expect_false(testClass(foo, "c"))
  expect_true(testMultiClass(foo, c("c", "b")))
})

context("checkFactor")

test_that("checkFactor", {
  myobj = factor(letters[1:2])
  expect_succ_all(Factor, myobj)
  myobj = letters[1:2]
  expect_fail_all(Factor, myobj)

  x = factor(c("a", "b"), levels = c("a", "b"))
  expect_true(testFactor(x))
  expect_false(testFactor(integer(1)))
  expect_false(testFactor("a"))
  expect_true(testFactor(factor()))
  # expect_false(testFactor(integer(0)))
  expect_false(testFactor(NULL))
  expect_true(testFactor(x, levels = rev(levels(x))))
  expect_true(testFactor(x, empty.levels.ok = FALSE))
  expect_true(testFactor(x, ordered = FALSE))

  expect_false(testFactor(x, levels = c("a")))
  expect_false(testFactor(x, levels = c("a", "b", "c")))

  x = factor(c("a", "b"), levels = c("a", "b", "c"), ordered = TRUE)
  expect_true(testFactor(x, empty.levels.ok = TRUE))
  expect_false(testFactor(x, empty.levels.ok = FALSE))
  expect_true(testFactor(x, ordered = TRUE))
  expect_false(testFactor(x, ordered = FALSE))


  x = factor(c("a", "b"), levels = c("a", "b", "c"))
  expect_error(assertFactor(1), "factor")
  expect_error(assertFactor(x, levels = c("a")), "levels")
  expect_error(assertFactor(x, empty.levels.ok = FALSE), "empty")
  expect_error(assertFactor(x, ordered = TRUE), "ordered")
  x = as.ordered(x)
  expect_error(assertFactor(x, ordered = FALSE), "unordered")


  x = factor(c("a", "b"))
  expect_true(testFactor(x, n.levels = 2))
  expect_true(testFactor(x, min.levels = 2))
  expect_true(testFactor(x, max.levels = 2))
  expect_false(testFactor(x, n.levels = 1))
  expect_false(testFactor(x, min.levels = 3))
  expect_false(testFactor(x, max.levels = 1))

  expect_error(testFactor(x, n.levels = NA))
  expect_error(assertFactor(x, n.levels = 1), "exactly 1 level")
})

test_that("typed.missing", {
  expect_false(testFactor(NA))
  expect_false(testFactor(NA_integer_))

  expect_true(testFactor(factor(levels = "a")))
  expect_false(testFactor(character(0)))
})

context("checkTibble")

test_that("checkTibble", {
  skip_if_not_physically_installed("tibble")

  expect_false(testTibble(iris))
  expect_true("tibble" %in% loadedNamespaces())

  x = tibble::as_tibble(iris)
  expect_succ_all("DataFrame", x)
  expect_succ_all("Tibble", x)
  expect_fail_all("Tibble", iris)

  expect_true(testTibble(x, min.rows = 1, ncols = 5))
  expect_false(testTibble(x, min.rows = 1000, ncols = 5))

  expect_true(testTibble(x, max.rows = 200, max.cols = 5))
  expect_false(testTibble(x, max.rows = 100))
  expect_false(testTibble(x, max.cols = 3))
})

context("checkSubset")

test_that("checkSubset", {
  myobj = letters[1:3]
  expect_succ_all(Subset, myobj, letters)
  myobj = 1:2
  expect_fail_all(Subset, myobj, letters)

  expect_false(testSubset(character(0), letters, empty.ok = FALSE))
  expect_true(testSubset(character(0), letters, empty.ok = TRUE))
  expect_false(testSubset(NULL, letters, empty.ok = FALSE))
  expect_true(testSubset(character(0), letters, empty.ok = TRUE))
  expect_false(testSubset(NULL, letters, empty.ok = FALSE))
  expect_true(testSubset(NULL, letters, empty.ok = TRUE))
  expect_false(testSubset(factor("a"), letters))
  expect_true(testSubset(1., 1:2))
  expect_true(testSubset(factor("a"), factor(letters)))

  expect_true(testSubset(1L, 1:10))
  expect_true(testSubset(3:4, 1:10))
  expect_false(testSubset("ab", letters))
  expect_false(testSubset(NA_integer_, 1:10))

  expect_error(assertSubset(-1, 1:2), "subset of")
  expect_error(assertSubset(1L, list()), "atomic")

  # issue #109
  expect_true(testSubset(character(0), character(0)))
  expect_true(testSubset(integer(0), character(0)))
  expect_error(assertSubset(1, integer(0)), "empty set")
})


test_that("checkSubset / fastmatch", {
  skip_if_not_installed("fastmatch")
  x = "c"
  y = letters[1:5]

  res = testSubset(x, y)
  expect_true(res)
  expect_null(attr(y, ".match.hash"))

  res = testSubset(x, y, fmatch = TRUE)
  expect_true(res)
  expect_class(attr(y, ".match.hash"), "match.hash")
})

test_that("checkSubset / NAs (#158)", {
  expect_true(test_subset(NA_character_, NA))
  expect_true(test_subset(NA, NA_character_))
})

context("checkSetEqual")

test_that("checkSetEqual", {
  myobj = letters[3:1]
  expect_succ_all(SetEqual, myobj, letters[1:3])
  expect_fail_all(SetEqual, myobj, letters[1:3], ordered = TRUE)
  myobj = letters[1:2]
  expect_fail_all(String, myobj, letters[1:3])

  expect_true(testSetEqual(character(0), character(0)))
  expect_true(testSetEqual(character(0), character(0), ordered = TRUE))
  expect_false(testSetEqual(character(0), letters))
  expect_false(testSetEqual(letters, character(0)))
  expect_false(testSetEqual(NULL, letters))
  expect_false(testSetEqual(NULL, letters, ordered = TRUE))
  expect_false(testSetEqual(factor("a"), letters))
  expect_true(testSetEqual(factor(letters), factor(letters)))
  expect_false(testSetEqual(letters, factor(letters)))

  expect_true(testSetEqual(1L, 1L))
  expect_true(testSetEqual(1, 1L))
  expect_true(testSetEqual(3:4, 3:4))
  expect_true(testSetEqual(NA_integer_, NA_integer_))

  expect_true(testSetEqual(1:2, 1:2, ordered = TRUE))
  expect_false(testSetEqual(1:2, 2:1, ordered = TRUE))
  expect_true(testSetEqual(NA, NA, ordered = TRUE))
  expect_false(testSetEqual(NA_integer_, 1L, ordered = TRUE))
  expect_false(testSetEqual(1L, NA_integer_, ordered = TRUE))
  expect_false(testSetEqual(c(NA_integer_, 2L), 1:2, ordered = TRUE))
  expect_true(testSetEqual(c(NA_integer_, 2L), c(NA_real_, 2), ordered = TRUE))

  expect_error(assertSetEqual(1, 1:2), "equal to")
  expect_error(assertSetEqual(1L, list()), "atomic")
})


test_that("checkSetEqual / fastmatch", {
  skip_if_not_installed("fastmatch")
  x = letters[5:1]
  y = letters[1:5]

  res = testSetEqual(x, y)
  expect_true(res)
  expect_null(attr(y, ".match.hash"))

  res = testSetEqual(x, y, fmatch = TRUE)
  expect_true(res)
  expect_class(attr(y, ".match.hash"), "match.hash")
})

test_that("checkSetEqual / NAs (#158)", {
  expect_true(test_set_equal(NA_character_, NA))
  expect_true(test_set_equal(NA, NA_character_))
})

context("checkDate")

test_that("checkDate", {
  x = Sys.Date()
  expect_succ_all(Date, x)
  expect_fail_all(Date, 1)

  expect_true(testDate(x, lower = 1))
  expect_true(testDate(x, upper = as.integer(x + 2)))
  expect_error(assertDate(x, lower = x + 2), ">=")
  expect_error(assertDate(x, upper = x - 2), "<=")

  expect_true(testDate(x, upper = x + 2))
  expect_false(testDate(x, upper = x - 2))
  expect_true(testDate(x, lower = x - 2))
  expect_false(testDate(x, lower = x + 2))

  expect_error(assertDate(x, lower = 1:2), "single")
  expect_error(assertDate(x, lower = NA), "single")
  expect_error(assertDate(x, lower = integer(0)), "single")
  expect_error(assertDate(x, upper = 1:2), "single")
  expect_error(assertDate(x, upper = NA), "single")
  expect_error(assertDate(x, upper = integer(0)), "single")


  x = as.Date(NA)
  expect_error(assertDate(x, any.missing = FALSE), "missing")
  x = rep(Sys.Date(), 2)
  expect_error(assertDate(x, unique = TRUE), "duplicated")

  expect_error(assertDate(letters, unique = TRUE), "character")
})

test_that("NAs are ignored for dates' lower-bound", {
  # Define and test a nomal date vector, and an empty date vector.
  d <- as.Date(c("2015-01-01", "2016-01-01", NA_character_, "2017-01-01"))
  empty <- as.Date(character(0))
  nas <- as.Date(NA_character_, NA_character_, NA_character_)

  # Bounds pass/fail appropriately when missing values are legal.
  expect_true( testDate(d    , lower = "1980-01-01", any.missing = TRUE ))
  expect_false(testDate(d    , lower = "2016-01-01", any.missing = TRUE ))

  # Bounds are ignored when missing values are illegal (and the vector contains missing values).
  expect_false(testDate(d    , lower = "1980-01-01", any.missing = FALSE))
  expect_false(testDate(d    , lower = "2016-01-01", any.missing = FALSE))

  # Zero-length date vectors never fail with a lower bound.
  expect_true( testDate(empty, lower  ="2030-01-01", any.missing = TRUE ))
  expect_true( testDate(empty, lower  ="2030-01-01", any.missing = FALSE))

  # NA date vectors
  expect_true( testDate(nas  , lower  ="2030-01-01", any.missing = TRUE ))
  expect_false(testDate(nas  , lower  ="2030-01-01", any.missing = FALSE))
})

test_that("NAs are ignored for dates' upper-bound", {
  # Define and test a nomal date vector, and an empty date vector.
  d <- as.Date(c("2015-01-01", "2016-01-01", NA_character_, "2017-01-01"))
  empty <- as.Date(character(0))
  nas <- as.Date(NA_character_, NA_character_, NA_character_)

  # Bounds pass/fail appropriately when missing values are legal.
  expect_true( testDate(d    , upper = "2020-01-01", any.missing = TRUE ))
  expect_false(testDate(d    , upper = "2016-01-01", any.missing = TRUE ))

  # Bounds are ignored when missing values are illegal (and the vector contains missing values).
  expect_false(testDate(d    , upper = "2020-01-01", any.missing = FALSE))
  expect_false(testDate(d    , upper = "2016-01-01", any.missing = FALSE))

  # Zero-length date vectors never fail with a upper bound.
  expect_true( testDate(empty, upper = "2000-01-01", any.missing = FALSE))
  expect_true( testDate(empty, upper = "2000-01-01", any.missing = FALSE))

  # NA date vectors
  expect_true( testDate(nas  , lower  ="2030-01-01", any.missing = TRUE ))
  expect_false(testDate(nas  , lower  ="2030-01-01", any.missing = FALSE))
})

context("registered c functions")

test_that("include of registered C functions works", {
  skip_on_cran()
  skip_on_travis()
  devtools::install_github("mllg/checkmate-test-include")
  library(checkmate.test.include)

  expect_true(reexported_qtest(1, "N1"))
  expect_false(reexported_qtest(1, "b"))

  x = pi
  expect_identical(reexported_qassert(x, "N1"), x)
  expect_error(reexported_qassert(x, "b", "foo"), "foo")
})

context("checkCharacter")

test_that("checkCharacter", {
  myobj = c("a", "b")
  expect_succ_all(Character, myobj)
  myobj = 0
  expect_fail_all(Character, myobj)

  expect_true(testCharacter(character(0)))
  expect_false(testCharacter(NULL))
  expect_true(testCharacter("a"))
  expect_false(testCharacter(1))
  expect_true(testCharacter(NA))
  expect_true(testCharacter(NA_character_))

  expect_true(testCharacter("a", min.chars = 1))
  expect_false(testCharacter("a", min.chars = 2))

  # treat NA_character_ as zero-length string
  expect_true(testCharacter(NA_character_, min.chars = 0))
  expect_true(testCharacter(NA_character_, min.chars = 1))
  expect_false(testCharacter(NA_character_, min.chars = 1, any.missing = FALSE))
  expect_false(testCharacter(c("", NA_character_), min.chars = 1))
  expect_true(testCharacter(NA, min.chars = 1))
  expect_true(testCharacter(character(0), min.chars = 1))

  x = c("abba", "baab")
  expect_true(testCharacter(x, pattern="a"))
  expect_true(testCharacter(x, pattern="ab"))
  expect_false(testCharacter(x, pattern="aa"))
  expect_false(testCharacter(x, pattern="^ab"))
  expect_true(testCharacter(x, pattern="AB", ignore.case=TRUE))
  expect_true(testCharacter(x, pattern="AB", ignore.case=TRUE))
  expect_false(testCharacter(x, pattern="AB", ignore.case=FALSE))
  expect_false(testCharacter(x, pattern="AB", ignore.case=FALSE))
  expect_true(testCharacter(x, pattern="a+"))
  expect_false(testCharacter(x, fixed="a+"))

  x = letters[1:3]
  expect_true(testCharacter(x, any.missing=FALSE, min.len=1L, max.len=3L))
  expect_false(testCharacter(x, any.missing=FALSE, len=5))

  expect_error(assertCharacter(1), "character")
})


test_that("NAs are ignored for regexp matching (#106)", {
  expect_true(testCharacter(c("a", NA, "b"), pattern = "^[ab]$", any.missing = TRUE))
  expect_false(testCharacter(c("a", NA, "b"), pattern = "^[cd]$", any.missing = TRUE))
  expect_true(testCharacter(c("a", NA, "bbbabbb"), fixed = "a", any.missing = TRUE))
  expect_false(testCharacter(c("a", NA, "bbbabbb"), fixed = "b", any.missing = TRUE))
})

test_that("Sorted strings", {
  x = letters[1:3]
  expect_true(testCharacter(x, sorted = TRUE))
  expect_error(assertCharacter(rev(x), sorted = TRUE), "sorted")

  x = c("aaa", "aab", "aac")
  expect_true(testCharacter(x, sorted = TRUE))
  expect_error(assertCharacter(rev(x), sorted = TRUE), "sorted")
})

test_that("typed.missing", {
  expect_true(testCharacter(NA))
  expect_true(testCharacter(NA, typed.missing = FALSE))
  expect_false(testCharacter(NA, typed.missing = TRUE))

  expect_true(testCharacter(logical()))
  expect_true(testCharacter(logical(), typed.missing = FALSE))
  expect_false(testCharacter(logical(), typed.missing = TRUE))
})

context("checkCount")

test_that("checkCount", {
  myobj = 1
  expect_succ_all(Count, myobj)
  myobj = -1
  expect_fail_all(Count, myobj)

  expect_false(testCount(integer(0)))
  expect_false(testCount(NULL))
  expect_false(testCount(FALSE))
  expect_false(testCount(TRUE))

  expect_true(testCount(0L))
  expect_false(testCount(0L, positive = TRUE))
  expect_true(testCount(1L, positive = TRUE))
  expect_true(testCount(1))
  expect_true(testCount(0))
  expect_false(testCount(-1))
  expect_false(testCount(0.5))
  expect_false(testCount(NA_integer_))
  expect_true(testCount(NA, na.ok = TRUE))
  expect_true(testCount(NA_integer_, na.ok = TRUE))
  expect_false(testCount(1:2))

  expect_false(is.integer(assertCount(5)))
  expect_true(is.integer(assertCount(5, coerce = TRUE)))

  expect_error(assertCount(-1), ">= 0")
})

context("wf / wl")

test_that("wf / wl", {
  x = c(FALSE, TRUE, FALSE, TRUE)
  expect_equal(wf(x), 2L)
  expect_equal(wl(x), 4L)

  x = c(NA, TRUE, NA, TRUE, NA)
  expect_equal(wf(x), 2L)
  expect_equal(wl(x), 4L)

  x = logical(0L)
  expect_equal(wf(x), integer(0L))
  expect_equal(wl(x), integer(0L))
  expect_equal(wf(x), integer(0L))
  expect_equal(wl(x), integer(0L))

  x = c(NA, NA)
  expect_equal(wf(x), integer(0L))
  expect_equal(wl(x), integer(0L))

  x = setNames(c(NA, FALSE, TRUE, FALSE, TRUE, FALSE, NA), letters[1:7])
  expect_identical(wf(x, TRUE), setNames(3L, "c"))
  expect_identical(wf(x, FALSE), 3L)
  expect_identical(wl(x), setNames(5L, "e"))
  expect_identical(wl(x, FALSE), 5L)
  expect_equal(wf(logical(0)), integer(0))
  expect_equal(wl(logical(0)), integer(0))

  expect_error(wf(42), "logical")
  expect_error(wl(42), "logical")
  expect_error(wf(NA, iris), "use.names")
  expect_error(wl(NA, iris), "use.names")
})

context("checkDisjunct")

test_that("checkDisjunct", {
  myobj = 1:3
  expect_succ_all(Disjunct, myobj, letters)
  myobj = "b"
  expect_fail_all(Disjunct, myobj, letters)

  expect_true(testDisjunct(character(0), letters))
  expect_true(testDisjunct(letters, character(0)))
  expect_true(testDisjunct(character(0L), character(0)))

  expect_false(testDisjunct(factor("a"), letters))
  expect_false(testDisjunct(1., 1:2))
  expect_false(testDisjunct(factor("a"), factor(letters)))

  expect_true(testDisjunct(NA_integer_, 1:10))
  expect_false(testDisjunct(NA_integer_, c(1:10, NA_integer_)))

  expect_error(assertSubset(1L, list()), "atomic")

  expect_true(testDisjunct(integer(0), character(0)))
  expect_error(assert_disjunct("a", "a"), "disjunct from")
})


test_that("checkSubset / fastmatch", {
  skip_if_not_installed("fastmatch")
  x = "c"
  y = letters[1:5]

  res = testDisjunct(x, y)
  expect_false(res)
  expect_null(attr(y, ".match.hash"))

  res = testDisjunct(x, y, fmatch = TRUE)
  expect_false(res)
  expect_class(attr(y, ".match.hash"), "match.hash")
})

test_that("checkDisjunct / NAs (#158)", {
  expect_false(test_disjunct(NA_character_, NA))
  expect_false(test_disjunct(NA, NA_character_))
})

context("checkRaw")

test_that("checkRaw", {
  myobj = as.raw(1)
  expect_succ_all(Raw, myobj)
  myobj = 1
  expect_fail_all(Raw, myobj)

  expect_true(testRaw(as.raw(NULL)))
  expect_false(testRaw(NULL))
  expect_true(testRaw(as.raw(NULL), len = 0))
  expect_true(testRaw(as.raw(1), len = 1))
  expect_true(testRaw(as.raw(1), min.len = 1, max.len = 1))

  x = as.raw(1:2)
  names(x) = letters[1:2]

  expect_true(testRaw(x, names = "unique"))

  expect_error(assertRaw(1), "raw")
  expect_error(assertRaw(as.raw(1), len = 2), "length")
})

context("checkFunction")

test_that("checkFunction", {
  myobj = mean
  expect_succ_all(Function, myobj)
  myobj = TRUE
  expect_fail_all(Function, myobj)

  myfun = function(x, y, ...) x + y

  expect_false(testFunction(NULL))
  expect_true(testFunction(identity))
  expect_true(testFunction(myfun))
  expect_false(testFunction("myfun"))
  expect_false(testFunction("myfun"))

  expect_true(testFunction(myfun, args = "x"))
  expect_true(testFunction(myfun, args = "..."))
  expect_true(testFunction(myfun, args = "x", ordered = TRUE))
  expect_true(testFunction(myfun, args = "y"))
  expect_true(testFunction(myfun, args = c("x", "y")))
  expect_true(testFunction(myfun, args = c("x", "y", "...")))
  expect_true(testFunction(myfun, args = c("y", "x")))
  expect_true(testFunction(myfun, args = c("x", "y"), ordered = TRUE))
  expect_false(testFunction(myfun, args = "z"))
  expect_false(testFunction(myfun, args = c("y"), ordered = TRUE))
  expect_false(testFunction(myfun, args = c("y", "x"), ordered = TRUE))

  expect_true(testFunction(myfun, nargs = 2))
  expect_true(testFunction(myfun, args = "x", nargs = 2))
  expect_true(testFunction(function() 1, nargs = 0))
  expect_true(testFunction(function(...) 1, nargs = 0))
  expect_false(testFunction(function(...) 1, nargs = 1))

  expect_error(assertFunction(fff), "not found")
  expect_error(assertFunction(myfun, "z"), "formal arguments")
  expect_error(assertFunction(myfun, "y", ordered=TRUE), "first formal arguments")

  expect_false(testFunction(function(x) x^2, args = character(0)))
  expect_true(testFunction(function() x^2, args = character(0)))
  expect_error(assertFunction(letters), "character")
})

test_that("checkFunction with Primitives", {
  expect_true(testFunction(sqrt))
  expect_true(testFunction(sqrt, args = "x"))
  expect_true(testFunction(as.logical, nargs = 1)) # bug 144
})

test_that("ordered and no args (#204)", {
  f = function() 0
  expect_error(assertFunction(f, args = c("a", "b"), ordered = FALSE), "formal arguments")
  expect_error(assertFunction(f, args = c("a", "b"), ordered = TRUE), "formal arguments")
})

context("checkPermutation")

test_that("checkPermutation", {
  expect_false(testPermutation(1L, c(1L, 2L)))
  expect_true(testPermutation(2:1, 1:2))

  expect_false(testPermutation(1, c(1, 2)))
  expect_true(testPermutation(c(2, 1), c(1, 2)))

  expect_true(testPermutation(letters[1:2], letters[2:1]))
  expect_false(testPermutation(letters[c(1, 1, 2)], letters[1:2]))

  expect_true(testPermutation(as.factor(letters[1:2]), as.factor(letters[2:1])))
  expect_false(testPermutation(as.factor(letters[c(1, 1, 2)]), as.factor(letters[1:2])))

  expect_true(testPermutation(as.ordered(letters[1:2]), as.ordered(letters[2:1])))
  expect_false(testPermutation(as.ordered(letters[c(1, 1, 2)]), as.ordered(letters[1:2])))

  expect_true(testPermutation(c(1 + 2i, 1), c(1, 1 + 2i)))
  expect_false(testPermutation(c(1 + 2i, 1 + 3i), c(1, 1 + 2i)))

  expect_true(testPermutation(c(FALSE, TRUE, NA), c(NA, TRUE, FALSE)))
  expect_false(testPermutation(c(FALSE, TRUE, NA, NA), c(NA, TRUE, FALSE)))

  expect_true(testPermutation(c(NA, 1), c(1, NA)))
  expect_false(testPermutation(c(1, 1, 1), c(1)))
  expect_true(testPermutation(1, 1L))
  expect_false(testPermutation(c(1, 1, 2), c(2, 2, 1)))
  expect_false(testPermutation(c("a", NA, "b", "b", NA), c(NA, NA, "a", "b", "a")))
  expect_true(testPermutation(c("a", NA, "b", "b", NA), c(NA, NA, "b", "b", "a")))
  expect_false(testPermutation(c("a", NA, "b", "b"), c(NA, NA, "b", "b", "a")))

  expect_false(testPermutation(c(1, NA), c(NA, 1), na.ok = FALSE))
  expect_false(testPermutation(c(1, NA), c(1)))
})

context("checkFlag")

test_that("checkFlag", {
  myobj = TRUE
  expect_succ_all(Flag, myobj)
  myobj = NA
  expect_fail_all(Flag, myobj)

  expect_false(testFlag(logical(0)))
  expect_false(testFlag(NULL))
  expect_true(testFlag(TRUE))
  expect_true(testFlag(FALSE))
  expect_false(testFlag(NA))
  expect_true(testFlag(NA, na.ok = TRUE))
  expect_true(testFlag(NA_character_, na.ok = TRUE))
  expect_false(testFlag(iris))

  expect_error(assertFlag(1), "logical flag")
})

---
title: "Checkmate"
author: "Michel Lang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{checkmate}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---
```{r,include=FALSE}
library(checkmate)
```

Ever used an R function that produced a not-very-helpful error message, just to discover after minutes of debugging that you simply passed a wrong argument?

Blaming the laziness of the package author for not doing such standard checks (in a dynamically typed language such as R) is at least partially unfair, as R makes these types of checks cumbersome and annoying. Well, that's how it was in the past.

Enter checkmate.

Virtually **every standard type of user error** when passing arguments into function can be caught with a simple, readable line which produces an **informative error message** in case.
A substantial part of the package was written in C to **minimize any worries about execution time overhead**.

## Intro
As a motivational example, consider you have a function to calculate the faculty of a natural number and the user may choose between using either the stirling approximation or R's `factorial` function (which internally uses the gamma function).
Thus, you have two arguments, `n` and `method`.
Argument `n` must obviously be a positive natural number and `method` must be either `"stirling"` or `"factorial"`.
Here is a version of all the hoops you need to jump through to ensure that these simple requirements are met:
```{r}
fact <- function(n, method = "stirling") {
  if (length(n) != 1)
    stop("Argument 'n' must have length 1")
  if (!is.numeric(n))
    stop("Argument 'n' must be numeric")
  if (is.na(n))
    stop("Argument 'n' may not be NA")
  if (is.double(n)) {
    if (is.nan(n))
      stop("Argument 'n' may not be NaN")
    if (is.infinite(n))
      stop("Argument 'n' must be finite")
    if (abs(n - round(n, 0)) > sqrt(.Machine$double.eps))
      stop("Argument 'n' must be an integerish value")
    n <- as.integer(n)
  }
  if (n < 0)
    stop("Argument 'n' must be >= 0")
  if (length(method) != 1)
    stop("Argument 'method' must have length 1")
  if (!is.character(method) || !method %in% c("stirling", "factorial"))
    stop("Argument 'method' must be either 'stirling' or 'factorial'")

  if (method == "factorial")
    factorial(n)
  else
    sqrt(2 * pi * n) * (n / exp(1))^n
}
```
And for comparison, here is the same function using checkmate:
```{r}
fact <- function(n, method = "stirling") {
  assertCount(n)
  assertChoice(method, c("stirling", "factorial"))

  if (method == "factorial")
    factorial(n)
  else
    sqrt(2 * pi * n) * (n / exp(1))^n
}
```

## Function overview
The functions can be split into four functional groups, indicated by their prefix.

If prefixed with `assert`, an error is thrown if the corresponding check fails.
Otherwise, the checked object is returned invisibly.
There are many different coding styles out there in the wild, but most R programmers stick to either `camelBack` or `underscore_case`.
Therefore, `checkmate` offers all functions in both flavors: `assert_count` is just an alias for `assertCount` but allows you to retain your favorite style.

The family of functions prefixed with `test` always return the check result as logical value.
Again, you can use `test_count` and `testCount` interchangeably.

Functions starting with `check` return the error message as a string (or `TRUE` otherwise) and can be used if you need more control and, e.g., want to grep on the returned error message.

`expect` is the last family of functions and is intended to be used with the [testthat package](https://cran.r-project.org/package=testthat).
All performed checks are logged into the `testthat` reporter.
Because `testthat` uses the `underscore_case`, the extension functions only come in the underscore style.

All functions are categorized into objects to check on the [package help page](https://mllg.github.io/checkmate/reference/checkmate-package).

## In case you miss flexibility

You can use [assert](https://mllg.github.io/checkmate/reference/assert) to perform multiple checks at once and throw an assertion if all checks fail.

Here is an example where we check that x is either of class `foo` or class `bar`:

```{r}
f <- function(x) {
  assert(
    checkClass(x, "foo"),
    checkClass(x, "bar")
  )
}
```

Note that `assert(, combine = "or")` and `assert(, combine = "and")` allow to control the logical
combination of the specified checks, and that the former is the default.


## Argument Checks for the Lazy

The following functions allow a special syntax to define argument checks using a special format specification.
E.g., `qassert(x, "I+")` asserts that `x` is an integer vector with at least one element and no missing values.
This very simple domain specific language covers a large variety of frequent argument checks with only a few keystrokes.
You choose what you like best.

* [qassert](https://mllg.github.io/checkmate/reference/qassert)
* [qassertr](https://mllg.github.io/checkmate/reference/qassertr)


## checkmate as testthat extension
To extend [testthat](https://cran.r-project.org/package=testthat), you need to IMPORT, DEPEND or SUGGEST on the `checkmate` package.
Here is a minimal example:
```{r,eval=FALSE}
# file: tests/test-all.R
library(testthat)
library(checkmate) # for testthat extensions
test_check("mypkg")
```
Now you are all set and can use more than 30 new expectations in your tests.
```{r,eval=FALSE}
test_that("checkmate is a sweet extension for testthat", {
  x = runif(100)
  expect_numeric(x, len = 100, any.missing = FALSE, lower = 0, upper = 1)
  # or, equivalent, using the lazy style:
  qexpect(x, "N100[0,1]")
})
```

## Speed considerations

In comparison with tediously writing the checks yourself in R (c.f. factorial example at the beginning of the vignette), R is sometimes a tad faster while performing checks on scalars.
This seems odd at first, because checkmate is mostly written in C and should be comparably fast.
Yet many of the functions in the `base` package are not regular functions, but primitives.
While primitives jump directly into the C code, checkmate has to use the considerably slower `.Call` interface.
As a result, it is possible to write (very simple) checks using only the base functions which, under some circumstances, slightly outperform checkmate.
However, if you go one step further and wrap the custom check into a function to convenient re-use it, the performance gain is often lost (see benchmark 1).

For larger objects the tide has turned because checkmate avoids many unnecessary intermediate variables.
Also note that the quick/lazy implementation in `qassert`/`qtest`/`qexpect` is often a tad faster because only two arguments have to be evaluated (the object and the rule) to determine the set of checks to perform.

Below you find some (probably unrepresentative) benchmark.
But also note that this one here has been executed from inside `knitr` which is often the cause for outliers in the measured execution time.
Better run the benchmark yourself to get unbiased results.


### Benchmark 1: Assert that `x` is a flag

```{r,fig.width=6,fig.height=4,dependson="init",eval=requireNamespace("microbenchmark", quietly = TRUE)}
library(checkmate)
library(ggplot2)
library(microbenchmark)

x = TRUE
r = function(x, na.ok = FALSE) { stopifnot(is.logical(x), length(x) == 1, na.ok || !is.na(x)) }
cm = function(x) assertFlag(x)
cmq = function(x) qassert(x, "B1")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)
```


### Benchmark 2: Assert that `x` is a numeric of length 1000 with no missing nor NaN values

```{r,fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)}
x = runif(1000)
r = function(x) stopifnot(is.numeric(x), length(x) == 1000, all(!is.na(x) & x >= 0 & x <= 1))
cm = function(x) assertNumeric(x, len = 1000, any.missing = FALSE, lower = 0, upper = 1)
cmq = function(x) qassert(x, "N1000[0,1]")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)
```


### Benchmark 3: Assert that `x` is a character vector with no missing values nor empty strings

```{r,fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)}
x = sample(letters, 10000, replace = TRUE)
r = function(x) stopifnot(is.character(x), !any(is.na(x)), all(nchar(x) > 0))
cm = function(x) assertCharacter(x, any.missing = FALSE, min.chars = 1)
cmq = function(x) qassert(x, "S+[1,]")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)
```


### Benchmark 4: Test that `x` is a data frame with no missing values

```{r,fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)}
N = 10000
x = data.frame(a = runif(N), b = sample(letters[1:5], N, replace = TRUE), c = sample(c(FALSE, TRUE), N, replace = TRUE))
r = function(x) is.data.frame(x) && !any(sapply(x, function(x) any(is.na(x))))
cm = function(x) testDataFrame(x, any.missing = FALSE)
cmq = function(x) qtest(x, "D")
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)

# checkmate tries to stop as early as possible
x$a[1] = NA
mb = microbenchmark(r(x), cm(x), cmq(x))
print(mb)
autoplot(mb)
```

### Benchmark 5: Assert that `x` is an increasing sequence of integers with no missing values

```{r,fig.width=6,fig.height=4,eval=requireNamespace("microbenchmark", quietly = TRUE)}
N = 10000
x.altrep = seq_len(N) # this is an ALTREP in R version >= 3.5.0
x.sexp = c(x.altrep)  # this is a regular SEXP OTOH
r = function(x) stopifnot(is.integer(x), !any(is.na(x)), !is.unsorted(x))
cm = function(x) assertInteger(x, any.missing = FALSE, sorted = TRUE)
mb = microbenchmark(r(x.sexp), cm(x.sexp), r(x.altrep), cm(x.altrep))
print(mb)
autoplot(mb)
```

## Extending checkmate

To extend checkmate a custom `check*` function has to be written.
For example, to check for a square matrix one can re-use parts of checkmate and extend the check with additional functionality:
```{r}
checkSquareMatrix = function(x, mode = NULL) {
  # check functions must return TRUE on success
  # and a custom error message otherwise
  res = checkMatrix(x, mode = mode)
  if (!isTRUE(res))
    return(res)
  if (nrow(x) != ncol(x))
    return("Must be square")
  return(TRUE)
}

# a quick test:
X = matrix(1:9, nrow = 3)
checkSquareMatrix(X)
checkSquareMatrix(X, mode = "character")
checkSquareMatrix(X[1:2, ])
```
The respective counterparts to the `check`-function can be created using the constructors [makeAssertionFunction](https://mllg.github.io/checkmate/reference/makeAssertion), [makeTestFunction](https://mllg.github.io/checkmate/reference/makeTest) and [makeExpectationFunction](https://mllg.github.io/checkmate/reference/makeExpectation):
```{r}
# For assertions:
assert_square_matrix = assertSquareMatrix = makeAssertionFunction(checkSquareMatrix)
print(assertSquareMatrix)

# For tests:
test_square_matrix = testSquareMatrix = makeTestFunction(checkSquareMatrix)
print(testSquareMatrix)

# For expectations:
expect_square_matrix = makeExpectationFunction(checkSquareMatrix)
print(expect_square_matrix)
```
Note that all the additional arguments `.var.name`, `add`, `info` and `label` are automatically joined with the function arguments of your custom check function.
Also note that if you define these functions inside an R package, the constructors are called at build-time (thus, there is no negative impact on the runtime).

## Calling checkmate from C/C++

The package registers two functions which can be used in other packages' C/C++ code for argument checks.
```{r, eval = FALSE, hilang = "c"}
SEXP qassert(SEXP x, const char *rule, const char *name);
Rboolean qtest(SEXP x, const char *rule);
```
These are the counterparts to [qassert](https://mllg.github.io/checkmate/reference/qassert) and [qtest](https://mllg.github.io/checkmate/reference/qassert).
Due to their simplistic interface, they perfectly suit the requirements of most type checks in C/C++.

For detailed background information on the register mechanism, see the [Exporting C Code](https://r-pkgs.org/src.html#c-export) section in Hadley's Book "R Packages" or [WRE](https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Registering-native-routines).
Here is a step-by-step guide to get you started:

1. Add `checkmate` to your "Imports" and "LinkingTo" sections in your DESCRIPTION file.
2. Create a stub C source file `"checkmate_stub.c"`, see below.
3. Include the provided header file `<checkmate.h>` in each compilation unit where you want to use checkmate.


File contents for (2):
```{r, eval = FALSE, hilang = "c"}
#include <checkmate.h>
#include <checkmate_stub.c>
```

## Session Info
For the sake of completeness, here the `sessionInfo()` for the benchmark (but remember the note before on `knitr` possibly biasing the results).
```{r}
sessionInfo()
```

---
title: "Setup tinytest"
author: "Michel Lang"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Setup tinytest}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Setup `tinytest`

Setting up the `tinytest` framework is covered in detail in the documentation of [tinytest](https://cran.r-project.org/package=tinytest).
Quick summary:

1. In the package root directory, call `tinytest::setup_tinytest(".")`.
2. Edit/add R scripts with unit tests to the directory `inst/tinytest`.


## Setup `checkmate`

1. Edit your `DESCRIPTION` and add `checkmate` to `Suggests` unless it is already listed in `Imports` for its assertions.
2. In **each** test file which calls an expectation from `checkmate`, you must include the following lines at the beginning of the file:
    ```{r,eval=FALSE}
    library("tinytest")
    library("checkmate")
    using("checkmate")
    ```
If you forget the last line, expectations will not get reported correctly in the summary.
Also note that sourcing a file which includes `using("checkmate")` is **not** sufficient.
3. Check if everything works with `R CMD check`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anyNaN.R
\name{anyNaN}
\alias{anyNaN}
\title{Check if an object contains NaN values}
\usage{
anyNaN(x)
}
\arguments{
\item{x}{[\code{ANY}]\cr
Object to check.}
}
\value{
[\code{logical(1)}] Returns \code{TRUE} if any element is \code{NaN}.
}
\description{
Supported are atomic types (see \code{\link[base]{is.atomic}}), lists and data frames.
}
\examples{
anyNaN(1:10)
anyNaN(c(1:10, NaN))
iris[3, 3] = NaN
anyNaN(iris)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkOS.R
\name{checkOS}
\alias{checkOS}
\alias{check_os}
\alias{assertOS}
\alias{assert_os}
\alias{testOS}
\alias{test_os}
\alias{expect_os}
\title{Check the operating system}
\usage{
checkOS(os)

check_os(os)

assertOS(os, add = NULL, .var.name = NULL)

assert_os(os, add = NULL, .var.name = NULL)

testOS(os)

test_os(os)

expect_os(os, info = NULL, label = NULL)
}
\arguments{
\item{os}{[\code{character(1)}]\cr
Check the operating system to be in a set with possible elements \dQuote{windows},
\dQuote{mac}, \dQuote{linux} and \dQuote{solaris}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertOS}/\code{assert_os} return 
 \code{x} invisibly, whereas
 \code{checkOS}/\code{check_os} and 
 \code{testOS}/\code{test_os} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertOS}/\code{assert_os}
 throws an error message, 
 \code{testOS}/\code{test_os}
 returns \code{FALSE},
 and \code{checkOS}/\code{check_os} 
 return a string with the error message.
 The function \code{expect_os} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check the operating system
}
\examples{
testOS("linux")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkInt.R
\name{checkInt}
\alias{checkInt}
\alias{check_int}
\alias{assertInt}
\alias{assert_int}
\alias{testInt}
\alias{test_int}
\alias{expect_int}
\title{Check if an argument is a single integerish value}
\usage{
checkInt(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

check_int(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

assertInt(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE,
  coerce = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_int(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE,
  coerce = FALSE,
  .var.name = vname(x),
  add = NULL
)

testInt(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

test_int(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

expect_int(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{lower}{[\code{numeric(1)}]\cr
Lower value all elements of \code{x} must be greater than or equal to.}

\item{upper}{[\code{numeric(1)}]\cr
Upper value all elements of \code{x} must be lower than or equal to.}

\item{tol}{[\code{double(1)}]\cr
Numerical tolerance used to check whether a double or complex can be converted.
Default is \code{sqrt(.Machine$double.eps)}.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{coerce}{[\code{logical(1)}]\cr
If \code{TRUE}, the input \code{x} is returned as integer after an successful assertion.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertInt}/\code{assert_int} return 
 \code{x} invisibly, whereas
 \code{checkInt}/\code{check_int} and 
 \code{testInt}/\code{test_int} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertInt}/\code{assert_int}
 throws an error message, 
 \code{testInt}/\code{test_int}
 returns \code{FALSE},
 and \code{checkInt}/\code{check_int} 
 return a string with the error message.
 The function \code{expect_int} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a single integerish value
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\note{
To perform an assertion and then convert to integer, use \code{\link{asInt}}.
\code{assertInt} will not convert numerics to integer.
}
\examples{
testInt(1)
testInt(-1, lower = 0)
}
\seealso{
Other scalars: 
\code{\link{checkCount}()},
\code{\link{checkFlag}()},
\code{\link{checkNumber}()},
\code{\link{checkScalarNA}()},
\code{\link{checkScalar}()},
\code{\link{checkString}()}
}
\concept{scalars}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/allMissing.R, R/anyMissing.R
\name{allMissing}
\alias{allMissing}
\alias{anyMissing}
\title{Check if an object contains missing values}
\usage{
allMissing(x)

anyMissing(x)
}
\arguments{
\item{x}{[\code{ANY}]\cr
Object to check.}
}
\value{
[\code{logical(1)}] Returns \code{TRUE} if any (\code{anyMissing}) or all (\code{allMissing})
 elements of \code{x} are missing (see details), \code{FALSE} otherwise.
}
\description{
\code{anyMissing} checks for the presence of at least one missing value,
\code{allMissing} checks for the presence of at least one non-missing value.
Supported are atomic types (see \code{\link[base]{is.atomic}}), lists and data frames.
Missingness is defined as \code{NA} or \code{NaN} for atomic types and data frame columns,
\code{NULL} is defined as missing for lists.\cr
\code{allMissing} applied to a \code{data.frame} returns \code{TRUE} if at least one column has
only non-missing values. If you want to perform the less frequent check that there is at least
a single non-missing observation present in the \code{data.frame}, use
\code{all(sapply(df, allMissing))} instead.
}
\examples{
allMissing(1:2)
allMissing(c(1, NA))
allMissing(c(NA, NA))
x = data.frame(a = 1:2, b = NA)
# Note how allMissing combines the results for data frames:
allMissing(x)
all(sapply(x, allMissing))
anyMissing(c(1, 1))
anyMissing(c(1, NA))
anyMissing(list(1, NULL))

x = iris
x[, "Species"] = NA
anyMissing(x)
allMissing(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkDisjunct.R
\name{checkDisjunct}
\alias{checkDisjunct}
\alias{check_disjunct}
\alias{assertDisjunct}
\alias{assert_disjunct}
\alias{testDisjunct}
\alias{test_disjunct}
\alias{expect_disjunct}
\title{Check if an argument is disjunct from a given set}
\usage{
checkDisjunct(x, y, fmatch = FALSE)

check_disjunct(x, y, fmatch = FALSE)

assertDisjunct(x, y, fmatch = FALSE, .var.name = vname(x), add = NULL)

assert_disjunct(x, y, fmatch = FALSE, .var.name = vname(x), add = NULL)

testDisjunct(x, y, fmatch = FALSE)

test_disjunct(x, y, fmatch = FALSE)

expect_disjunct(x, y, fmatch = FALSE, info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{y}{[\code{atomic}]\cr
Other Set.}

\item{fmatch}{[\code{logical(1)}]\cr
Use the set operations implemented in \code{\link[fastmatch]{fmatch}} in package \pkg{fastmatch}.
If \pkg{fastmatch} is not installed, this silently falls back to \code{\link[base]{match}}.
\code{\link[fastmatch]{fmatch}} modifies \code{y} by reference:
A hash table is added as attribute which is used in subsequent calls.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertDisjunct}/\code{assert_disjunct} return 
 \code{x} invisibly, whereas
 \code{checkDisjunct}/\code{check_disjunct} and 
 \code{testDisjunct}/\code{test_disjunct} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertDisjunct}/\code{assert_disjunct}
 throws an error message, 
 \code{testDisjunct}/\code{test_disjunct}
 returns \code{FALSE},
 and \code{checkDisjunct}/\code{check_disjunct} 
 return a string with the error message.
 The function \code{expect_disjunct} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is disjunct from a given set
}
\note{
The object \code{x} must be of the same type as the set w.r.t. \code{\link[base]{typeof}}.
Integers and doubles are both treated as numeric.
}
\examples{
testDisjunct(1L, letters)
testDisjunct(c("a", "z"), letters)

# x is not converted before the comparison (except for numerics)
testDisjunct(factor("a"), "a")
testDisjunct(1, "1")
testDisjunct(1, as.integer(1))
}
\seealso{
Other set: 
\code{\link{checkChoice}()},
\code{\link{checkPermutation}()},
\code{\link{checkSetEqual}()},
\code{\link{checkSubset}()}
}
\concept{set}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkArray.R
\name{checkArray}
\alias{checkArray}
\alias{check_array}
\alias{assertArray}
\alias{assert_array}
\alias{testArray}
\alias{test_array}
\alias{expect_array}
\title{Check if an argument is an array}
\usage{
checkArray(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE
)

check_array(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE
)

assertArray(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_array(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testArray(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE
)

test_array(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE
)

expect_array(
  x,
  mode = NULL,
  any.missing = TRUE,
  d = NULL,
  min.d = NULL,
  max.d = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{mode}{[\code{character(1)}]\cr
Storage mode of the array. Arrays can hold vectors, i.e. \dQuote{logical},
\dQuote{integer}, \dQuote{integerish}, \dQuote{double}, \dQuote{numeric}, \dQuote{complex},
\dQuote{character} and \dQuote{list}. You can also specify \dQuote{atomic}
here to explicitly prohibit lists. Default is \code{NULL} (no check).
If all values of \code{x} are missing, this check is skipped.}

\item{any.missing}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{TRUE}.}

\item{d}{[\code{integer(1)}]\cr
Exact number of dimensions of array \code{x}.
Default is \code{NULL} (no check).}

\item{min.d}{[\code{integer(1)}]\cr
Minimum number of dimensions of array \code{x}.
Default is \code{NULL} (no check).}

\item{max.d}{[\code{integer(1)}]\cr
Maximum number of dimensions of array \code{x}.
Default is \code{NULL} (no check).}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertArray}/\code{assert_array} return 
 \code{x} invisibly, whereas
 \code{checkArray}/\code{check_array} and 
 \code{testArray}/\code{test_array} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertArray}/\code{assert_array}
 throws an error message, 
 \code{testArray}/\code{test_array}
 returns \code{FALSE},
 and \code{checkArray}/\code{check_array} 
 return a string with the error message.
 The function \code{expect_array} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is an array
}
\examples{
checkArray(array(1:27, dim = c(3, 3, 3)), d = 3)
}
\seealso{
Other basetypes: 
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}

Other compound: 
\code{\link{checkDataFrame}()},
\code{\link{checkDataTable}()},
\code{\link{checkMatrix}()},
\code{\link{checkTibble}()}
}
\concept{basetypes}
\concept{compound}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkNumeric.R
\name{checkNumeric}
\alias{checkNumeric}
\alias{check_numeric}
\alias{assertNumeric}
\alias{assert_numeric}
\alias{testNumeric}
\alias{test_numeric}
\alias{expect_numeric}
\title{Check that an argument is a vector of type numeric}
\usage{
checkNumeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

check_numeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

assertNumeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_numeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testNumeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

test_numeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

expect_numeric(
  x,
  lower = -Inf,
  upper = Inf,
  finite = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{lower}{[\code{numeric(1)}]\cr
Lower value all elements of \code{x} must be greater than or equal to.}

\item{upper}{[\code{numeric(1)}]\cr
Upper value all elements of \code{x} must be lower than or equal to.}

\item{finite}{[\code{logical(1)}]\cr
Check for only finite values? Default is \code{FALSE}.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{sorted}{[\code{logical(1)}]\cr
Elements must be sorted in ascending order. Missing values are ignored.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{typed.missing}{[\code{logical(1)}]\cr
If set to \code{FALSE} (default), all types of missing values (\code{NA}, \code{NA_integer_},
\code{NA_real_}, \code{NA_character_} or \code{NA_character_}) as well as empty vectors are allowed
while type-checking atomic input.
Set to \code{FALSE} to enable strict type checking.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertNumeric}/\code{assert_numeric} return 
 \code{x} invisibly, whereas
 \code{checkNumeric}/\code{check_numeric} and 
 \code{testNumeric}/\code{test_numeric} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertNumeric}/\code{assert_numeric}
 throws an error message, 
 \code{testNumeric}/\code{test_numeric}
 returns \code{FALSE},
 and \code{checkNumeric}/\code{check_numeric} 
 return a string with the error message.
 The function \code{expect_numeric} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Vectors of storage type \dQuote{integer} and \dQuote{double} count as \dQuote{numeric}, c.f. \code{\link[base]{is.numeric}}.
To explicitly check for real integer or double vectors, see \code{\link{checkInteger}}, \code{\link{checkIntegerish}} or
\code{\link{checkDouble}}.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\examples{
testNumeric(1)
testNumeric(1, min.len = 1, lower = 0)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkPOSIXct.R
\name{checkPOSIXct}
\alias{checkPOSIXct}
\alias{check_posixct}
\alias{assertPOSIXct}
\alias{assert_posixct}
\alias{testPOSIXct}
\alias{test_posixct}
\alias{expect_posixct}
\title{Check that an argument is a date/time object in POSIXct format}
\usage{
checkPOSIXct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE
)

check_posixct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE
)

assertPOSIXct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_posixct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testPOSIXct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE
)

test_posixct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE
)

expect_posixct(
  x,
  lower = NULL,
  upper = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{lower}{[\code{\link[base]{Date}}]\cr
All non-missing dates in \code{x} must be >= this POSIXct time. Must be provided in the same timezone as \code{x}.}

\item{upper}{[\code{\link[base]{Date}}]\cr
All non-missing dates in \code{x} must be <= this POSIXct time. Must be provided in the same timezone as \code{x}.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{sorted}{[\code{logical(1)}]\cr
Elements must be sorted in ascending order. Missing values are ignored.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertAtomic}/\code{assert_atomic} return 
 \code{x} invisibly, whereas
 \code{checkAtomic}/\code{check_atomic} and 
 \code{testAtomic}/\code{test_atomic} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertAtomic}/\code{assert_atomic}
 throws an error message, 
 \code{testAtomic}/\code{test_atomic}
 returns \code{FALSE},
 and \code{checkAtomic}/\code{check_atomic} 
 return a string with the error message.
 The function \code{expect_atomic} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Checks that an object is of class \code{\link[base]{POSIXct}}.
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qassert.R
\name{qassert}
\alias{qassert}
\alias{qtest}
\alias{qexpect}
\title{Quick argument checks on (builtin) R types}
\usage{
qassert(x, rules, .var.name = vname(x))

qtest(x, rules)

qexpect(x, rules, info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object the check.}

\item{rules}{[\code{character}]\cr
Set of rules. See details.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in error messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
\code{qassert} throws an \code{R} exception if object \code{x} does
 not comply to at least one of the \code{rules} and returns the tested object invisibly
 otherwise.
 \code{qtest} behaves the same way but returns \code{FALSE} if none of the
 \code{rules} comply.
 \code{qexpect} is intended to be inside the unit test framework \code{\link[testthat]{testthat}} and
 returns an \code{\link[testthat]{expectation}}.
}
\description{
The provided functions parse rules which allow to express some of the most
frequent argument checks by typing just a few letters.
}
\details{
The rule is specified in up to three parts.
\enumerate{
 \item{
   Class and missingness check.
   The first letter is an abbreviation for the class. If it is
   provided uppercase, missing values are prohibited.
   Supported abbreviations:
   \tabular{rl}{
     \code{[bB]} \tab Bool / logical.\cr
     \code{[iI]} \tab Integer.\cr
     \code{[xX]} \tab Integerish (numeric convertible to integer, see \code{\link{checkIntegerish}}).\cr
     \code{[rR]} \tab Real / double.\cr
     \code{[cC]} \tab Complex.\cr
     \code{[nN]} \tab Numeric (integer or double).\cr
     \code{[sS]} \tab String / character.\cr
     \code{[fF]} \tab Factor\cr
     \code{[aA]} \tab Atomic.\cr
     \code{[vV]} \tab Atomic vector (see \code{\link{checkAtomicVector}}).\cr
     \code{[lL]} \tab List. Missingness is defined as \code{NULL} element.\cr
     \code{[mM]} \tab Matrix.\cr
     \code{[dD]} \tab Data.frame. Missingness is checked recursively on columns.\cr
     \code{[pP]} \tab POSIXct date.\cr
     \code{[e]}  \tab Environment.\cr
     \code{[0]}  \tab \code{NULL}.\cr
     \code{[*]}  \tab placeholder to allow any type.
   }
   Note that the check for missingness does not distinguish between
   \code{NaN} and \code{NA}. Infinite values are not treated as missing, but
   can be caught using boundary checks (part 3).
   }
 \item{
   Length definition. This can be one of
   \tabular{rl}{
     \code{[*]} \tab any length,\cr
     \code{[?]} \tab length of zero or one,\cr
     \code{[+]} \tab length of at least one, or\cr
     \code{[0-9]+} \tab exact length specified as integer.
   }
   Preceding the exact length with one of the comparison operators \code{=}/\code{==},
   \code{<}, \code{<=}, \code{>=} or \code{>} is also supported.
 }
 \item{
   Range check as two numbers separated by a comma, enclosed by square brackets
   (endpoint included) or parentheses (endpoint excluded).
   For example, \dQuote{[0, 3)} results in \code{all(x >= 0 & x < 3)}.
   The lower and upper bound may be omitted which is the equivalent of a negative or
   positive infinite bound, respectively.
   By definition \code{[0,]} contains \code{Inf}, while \code{[0,)} does not.
   The same holds for the left (lower) boundary and \code{-Inf}.
   E.g., the rule \dQuote{N1()} checks for a single finite numeric which is not NA,
   while \dQuote{N1[)} allows \code{-Inf}.
 }
}
}
\note{
The functions are inspired by the blog post of Bogumił Kamiński:
\url{http://rsnippets.blogspot.de/2013/06/testing-function-agruments-in-gnu-r.html}.
The implementation is mostly written in C to minimize the overhead.
}
\examples{
# logical of length 1
qtest(NA, "b1")

# logical of length 1, NA not allowed
qtest(NA, "B1")

# logical of length 0 or 1, NA not allowed
qtest(TRUE, "B?")

# numeric with length > 0
qtest(runif(10), "n+")

# integer with length > 0, NAs not allowed, all integers >= 0 and < Inf
qtest(1:3, "I+[0,)")

# either an emtpy list or a character vector with <=5 elements
qtest(1, c("l0", "s<=5"))

# data frame with at least one column and no missing value in any column
qtest(iris, "D+")
}
\seealso{
\code{\link{qtestr}} and \code{\link{qassertr}} for efficient checks
of list elements and data frame columns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matchArg.R
\name{matchArg}
\alias{matchArg}
\title{Partial Argument Matching}
\usage{
matchArg(x, choices, several.ok = FALSE, .var.name = vname(x), add = NULL)
}
\arguments{
\item{x}{[character]\cr
User provided argument to match.}

\item{choices}{[character()]\cr
Candidates to match \code{x} with.}

\item{several.ok}{[logical(1)]\cr
If \code{TRUE}, multiple matches are allowed, cf. \code{\link[base]{match.arg}}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in error messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertions. See \code{\link{AssertCollection}}.}
}
\value{
Subset of \code{choices}.
}
\description{
This is an extensions to \code{\link[base]{match.arg}} with support for \code{\link{AssertCollection}}.
The behavior is very similar to \code{\link[base]{match.arg}}, except that \code{NULL} is not
a valid value for \code{x}.
}
\examples{
matchArg("k", choices = c("kendall", "pearson"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{assert}
\alias{assert}
\title{Combine multiple checks into one assertion}
\usage{
assert(..., combine = "or", .var.name = NULL, add = NULL)
}
\arguments{
\item{...}{[any]\cr
List of calls to check functions.}

\item{combine}{[\code{character(1)}]\cr
\dQuote{or} or \dQuote{and} to combine the check functions with an OR
or AND, respectively.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}
}
\value{
Throws an error (or pushes the error message to an 
  \code{\link{AssertCollection}} if \code{add} is not \code{NULL}) 
  if the checks fail and invisibly returns \code{TRUE} otherwise.
}
\description{
You can call this function with an arbitrary number of of \code{check*}
functions, i.e. functions provided by this package or your own functions which
return \code{TRUE} on success and the error message as \code{character(1)} otherwise.
The resulting assertion is successful, if \code{combine} is
\dQuote{or} (default) and at least one check evaluates to \code{TRUE} or
\code{combine} is \dQuote{and} and all checks evaluate to \code{TRUE}.
Otherwise, \code{assert} throws an informative error message.
}
\examples{
x = 1:10
assert(checkNull(x), checkInteger(x, any.missing = FALSE))
collection <- makeAssertCollection()
assert(checkChoice(x, c("a", "b")), checkDataFrame(x), add = collection)
collection$getMessages()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVector.R
\name{checkVector}
\alias{checkVector}
\alias{check_vector}
\alias{assertVector}
\alias{assert_vector}
\alias{testVector}
\alias{test_vector}
\title{Check if an argument is a vector}
\usage{
checkVector(
  x,
  strict = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

check_vector(
  x,
  strict = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

assertVector(
  x,
  strict = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_vector(
  x,
  strict = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testVector(
  x,
  strict = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

test_vector(
  x,
  strict = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{strict}{[\code{logical(1)}]\cr
May the vector have additional attributes? If \code{TRUE}, mimics the behavior of
\code{\link[base]{is.vector}}.
Default is \code{FALSE} which allows e.g. \code{factor}s or \code{data.frame}s
to be recognized as vectors.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertVector}/\code{assert_vector} return 
 \code{x} invisibly, whereas
 \code{checkVector}/\code{check_vector} and 
 \code{testVector}/\code{test_vector} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertVector}/\code{assert_vector}
 throws an error message, 
 \code{testVector}/\code{test_vector}
 returns \code{FALSE},
 and \code{checkVector}/\code{check_vector} 
 return a string with the error message.
 The function \code{expect_vector} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a vector
}
\examples{
testVector(letters, min.len = 1L, any.missing = FALSE)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()}

Other atomicvector: 
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()}
}
\concept{atomicvector}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asInteger.R
\name{asInteger}
\alias{asInteger}
\alias{asCount}
\alias{asInt}
\title{Convert an argument to an integer}
\usage{
asInteger(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  .var.name = vname(x)
)

asCount(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  .var.name = vname(x)
)

asInt(
  x,
  na.ok = FALSE,
  lower = -Inf,
  upper = Inf,
  tol = sqrt(.Machine$double.eps),
  .var.name = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to convert.}

\item{tol}{[\code{double(1)}]\cr
Numerical tolerance used to check whether a double or complex can be converted.
Default is \code{sqrt(.Machine$double.eps)}.}

\item{lower}{[\code{numeric(1)}]\cr
Lower value all elements of \code{x} must be greater than or equal to.}

\item{upper}{[\code{numeric(1)}]\cr
Upper value all elements of \code{x} must be lower than or equal to.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{sorted}{[\code{logical(1)}]\cr
Elements must be sorted in ascending order. Missing values are ignored.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in error messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{positive}{[\code{logical(1)}]\cr
Must \code{x} be positive (>= 1)?
Default is \code{FALSE}.}
}
\value{
Converted \code{x}.
}
\description{
\code{asInteger} is intended to be used for vectors while \code{asInt} is
a specialization for scalar integers and \code{asCount} for scalar
non-negative integers.
Convertible are (a) atomic vectors with all elements \code{NA}
and (b) double vectors with all elements being within \code{tol}
range of an integer.

Note that these functions may be deprecated in the future.
Instead, it is advised to use \code{\link{assertCount}},
\code{\link{assertInt}} or \code{\link{assertIntegerish}} with
argument \code{coerce} set to \code{TRUE} instead.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\examples{
asInteger(c(1, 2, 3))
asCount(1)
asInt(1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFactor.R
\name{checkFactor}
\alias{checkFactor}
\alias{check_factor}
\alias{assertFactor}
\alias{assert_factor}
\alias{testFactor}
\alias{test_factor}
\alias{expect_factor}
\title{Check if an argument is a factor}
\usage{
checkFactor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

check_factor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

assertFactor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_factor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testFactor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

test_factor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

expect_factor(
  x,
  levels = NULL,
  ordered = NA,
  empty.levels.ok = TRUE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  n.levels = NULL,
  min.levels = NULL,
  max.levels = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{levels}{[\code{character}]\cr
Vector of allowed factor levels.}

\item{ordered}{[\code{logical(1)}]\cr
Check for an ordered factor? If \code{FALSE} or \code{TRUE}, checks explicitly
for an unordered or ordered factor, respectively.
Default is \code{NA} which does not perform any additional check.}

\item{empty.levels.ok}{[\code{logical(1)}]\cr
Are empty levels allowed?
Default is \code{TRUE}.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{n.levels}{[\code{integer(1)}]\cr
Exact number of factor levels.
Default is \code{NULL} (no check).}

\item{min.levels}{[\code{integer(1)}]\cr
Minimum number of factor levels.
Default is \code{NULL} (no check).}

\item{max.levels}{[\code{integer(1)}]\cr
Maximum number of factor levels.
Default is \code{NULL} (no check).}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertFactor}/\code{assert_factor} return 
 \code{x} invisibly, whereas
 \code{checkFactor}/\code{check_factor} and 
 \code{testFactor}/\code{test_factor} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertFactor}/\code{assert_factor}
 throws an error message, 
 \code{testFactor}/\code{test_factor}
 returns \code{FALSE},
 and \code{checkFactor}/\code{check_factor} 
 return a string with the error message.
 The function \code{expect_factor} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a factor
}
\examples{
x = factor("a", levels = c("a", "b"))
testFactor(x)
testFactor(x, empty.levels.ok = FALSE)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkSetEqual.R
\name{checkSetEqual}
\alias{checkSetEqual}
\alias{check_set_equal}
\alias{assertSetEqual}
\alias{assert_set_equal}
\alias{testSetEqual}
\alias{test_set_equal}
\alias{expect_set_equal}
\title{Check if an argument is equal to a given set}
\usage{
checkSetEqual(x, y, ordered = FALSE, fmatch = FALSE)

check_set_equal(x, y, ordered = FALSE, fmatch = FALSE)

assertSetEqual(
  x,
  y,
  ordered = FALSE,
  fmatch = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_set_equal(
  x,
  y,
  ordered = FALSE,
  fmatch = FALSE,
  .var.name = vname(x),
  add = NULL
)

testSetEqual(x, y, ordered = FALSE, fmatch = FALSE)

test_set_equal(x, y, ordered = FALSE, fmatch = FALSE)

expect_set_equal(
  x,
  y,
  ordered = FALSE,
  fmatch = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{y}{[\code{atomic}]\cr
Set to compare with.}

\item{ordered}{[\code{logical(1)}]\cr
Check \code{x} to have the same length and order as \code{y}, i.e.
check using \dQuote{==} while handling \code{NA}s nicely.
Default is \code{FALSE}.}

\item{fmatch}{[\code{logical(1)}]\cr
Use the set operations implemented in \code{\link[fastmatch]{fmatch}} in package \pkg{fastmatch}.
If \pkg{fastmatch} is not installed, this silently falls back to \code{\link[base]{match}}.
\code{\link[fastmatch]{fmatch}} modifies \code{y} by reference:
A hash table is added as attribute which is used in subsequent calls.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertSubset}/\code{assert_subset} return 
 \code{x} invisibly, whereas
 \code{checkSubset}/\code{check_subset} and 
 \code{testSubset}/\code{test_subset} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertSubset}/\code{assert_subset}
 throws an error message, 
 \code{testSubset}/\code{test_subset}
 returns \code{FALSE},
 and \code{checkSubset}/\code{check_subset} 
 return a string with the error message.
 The function \code{expect_subset} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is equal to a given set
}
\note{
The object \code{x} must be of the same type as the set w.r.t. \code{\link[base]{typeof}}.
Integers and doubles are both treated as numeric.
}
\examples{
testSetEqual(c("a", "b"), c("a", "b"))
testSetEqual(1:3, 1:4)

# x is not converted before the comparison (except for numerics)
testSetEqual(factor("a"), "a")
testSetEqual(1, "1")
testSetEqual(1, as.integer(1))
}
\seealso{
Other set: 
\code{\link{checkChoice}()},
\code{\link{checkDisjunct}()},
\code{\link{checkPermutation}()},
\code{\link{checkSubset}()}
}
\concept{set}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeTest.R
\name{makeTest}
\alias{makeTest}
\alias{makeTestFunction}
\title{Turn a Check into a Test}
\usage{
makeTest(res)

makeTestFunction(check.fun, c.fun = NULL, env = parent.frame())
}
\arguments{
\item{res}{[\code{TRUE} | \code{character(1)}]\cr
The result of a check function: \code{TRUE} for successful checks,
and an error message as string otherwise.}

\item{check.fun}{[\code{function}]\cr
Function which checks the input. Must return \code{TRUE} on success and a string with the error message otherwise.}

\item{c.fun}{[\code{character(1)}]\cr
If not \code{NULL}, instead of calling the function \code{check.fun}, use \code{.Call} to call a C function \dQuote{c.fun} with the identical
set of parameters. The C function must be registered as a native symbol, see \code{\link[base]{.Call}}.
Useful if \code{check.fun} is just a simple wrapper.}

\item{env}{[\code{environment}]\cr
The environment of the created function. Default is the \code{\link[base]{parent.frame}}.}
}
\value{
\code{makeTest} returns \code{TRUE} if the check is successful and \code{FALSE} otherwise.
 \code{makeTestFunction} returns a \code{function}.
}
\description{
\code{makeTest} is the internal function used to evaluate the result of a
check and throw an exception if necessary.
This function is currently only a stub and just calls \code{\link[base]{isTRUE}}.
\code{makeTestFunction} can be used to automatically create an assertion
function based on a check function (see example).
}
\examples{
# Simple custom check function
checkFalse = function(x) if (!identical(x, FALSE)) "Must be FALSE" else TRUE

# Create the respective test function
testFalse = function(x) {
  res = checkFalse(x)
  makeTest(res)
}

# Alternative: Automatically create such a function
testFalse = makeTestFunction(checkFalse)
print(testFalse)
}
\seealso{
Other CustomConstructors: 
\code{\link{makeAssertion}()},
\code{\link{makeExpectation}()}
}
\concept{CustomConstructors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkChoice.R
\name{checkChoice}
\alias{checkChoice}
\alias{check_choice}
\alias{assertChoice}
\alias{assert_choice}
\alias{testChoice}
\alias{test_choice}
\alias{expect_choice}
\title{Check if an object is an element of a given set}
\usage{
checkChoice(x, choices, null.ok = FALSE, fmatch = FALSE)

check_choice(x, choices, null.ok = FALSE, fmatch = FALSE)

assertChoice(
  x,
  choices,
  null.ok = FALSE,
  fmatch = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_choice(
  x,
  choices,
  null.ok = FALSE,
  fmatch = FALSE,
  .var.name = vname(x),
  add = NULL
)

testChoice(x, choices, null.ok = FALSE, fmatch = FALSE)

test_choice(x, choices, null.ok = FALSE, fmatch = FALSE)

expect_choice(
  x,
  choices,
  null.ok = FALSE,
  fmatch = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{choices}{[\code{atomic}]\cr
Set of possible values.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{fmatch}{[\code{logical(1)}]\cr
Use the set operations implemented in \code{\link[fastmatch]{fmatch}} in package \pkg{fastmatch}.
If \pkg{fastmatch} is not installed, this silently falls back to \code{\link[base]{match}}.
\code{\link[fastmatch]{fmatch}} modifies \code{y} by reference:
A hash table is added as attribute which is used in subsequent calls.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertChoice}/\code{assert_choice} return 
 \code{x} invisibly, whereas
 \code{checkChoice}/\code{check_choice} and 
 \code{testChoice}/\code{test_choice} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertChoice}/\code{assert_choice}
 throws an error message, 
 \code{testChoice}/\code{test_choice}
 returns \code{FALSE},
 and \code{checkChoice}/\code{check_choice} 
 return a string with the error message.
 The function \code{expect_choice} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an object is an element of a given set
}
\note{
The object \code{x} must be of the same type as the set w.r.t. \code{\link[base]{typeof}}.
Integers and doubles are both treated as numeric.
}
\examples{
testChoice("x", letters)

# x is not converted before the comparison (except for numerics)
testChoice(factor("a"), "a")
testChoice(1, "1")
testChoice(1, as.integer(1))
}
\seealso{
Other set: 
\code{\link{checkDisjunct}()},
\code{\link{checkPermutation}()},
\code{\link{checkSetEqual}()},
\code{\link{checkSubset}()}
}
\concept{set}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkTRUE.R
\name{checkTRUE}
\alias{checkTRUE}
\alias{check_true}
\alias{assertTRUE}
\alias{assert_true}
\alias{testTRUE}
\alias{test_true}
\title{Check if an argument is TRUE}
\usage{
checkTRUE(x, na.ok = FALSE)

check_true(x, na.ok = FALSE)

assertTRUE(x, na.ok = FALSE, .var.name = vname(x), add = NULL)

assert_true(x, na.ok = FALSE, .var.name = vname(x), add = NULL)

testTRUE(x, na.ok = FALSE)

test_true(x, na.ok = FALSE)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertTRUE.}/\code{assert_true.} return 
 \code{x} invisibly, whereas
 \code{checkTRUE.}/\code{check_true.} and 
 \code{testTRUE.}/\code{test_true.} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertTRUE.}/\code{assert_true.}
 throws an error message, 
 \code{testTRUE.}/\code{test_true.}
 returns \code{FALSE},
 and \code{checkTRUE.}/\code{check_true.} 
 return a string with the error message.
 The function \code{expect_true.} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Simply checks if an argument is \code{TRUE}.
}
\examples{
testTRUE(TRUE)
testTRUE(FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFALSE.R
\name{checkFALSE}
\alias{checkFALSE}
\alias{check_false}
\alias{assertFALSE}
\alias{assert_false}
\alias{testFALSE}
\alias{test_false}
\title{Check if an argument is FALSE}
\usage{
checkFALSE(x, na.ok = FALSE)

check_false(x, na.ok = FALSE)

assertFALSE(x, na.ok = FALSE, .var.name = vname(x), add = NULL)

assert_false(x, na.ok = FALSE, .var.name = vname(x), add = NULL)

testFALSE(x, na.ok = FALSE)

test_false(x, na.ok = FALSE)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertFALSE.}/\code{assert_false.} return 
 \code{x} invisibly, whereas
 \code{checkFALSE.}/\code{check_false.} and 
 \code{testFALSE.}/\code{test_false.} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertFALSE.}/\code{assert_false.}
 throws an error message, 
 \code{testFALSE.}/\code{test_false.}
 returns \code{FALSE},
 and \code{checkFALSE.}/\code{check_false.} 
 return a string with the error message.
 The function \code{expect_false.} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Simply checks if an argument is \code{FALSE}.
}
\examples{
testFALSE(FALSE)
testFALSE(TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/anyInfinite.R
\name{anyInfinite}
\alias{anyInfinite}
\title{Check if an object contains infinite values}
\usage{
anyInfinite(x)
}
\arguments{
\item{x}{[\code{ANY}]\cr
Object to check.}
}
\value{
[\code{logical(1)}] Returns \code{TRUE} if any element is \code{-Inf} or \code{Inf}.
}
\description{
Supported are atomic types (see \code{\link[base]{is.atomic}}), lists and data frames.
}
\examples{
anyInfinite(1:10)
anyInfinite(c(1:10, Inf))
iris[3, 3] = Inf
anyInfinite(iris)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeExpectation.R
\name{register_test_backend}
\alias{register_test_backend}
\title{Select Backend for Unit Tests}
\usage{
register_test_backend(name)
}
\arguments{
\item{name}{[\code{character(1)}]\cr
\code{"testthat"} or \code{"tinytest"}.}
}
\value{
\code{NULL} (invisibly).
}
\description{
Allows to explicitly select a backend for the unit tests.
Currently supported are \code{"testthat"} and \code{"tinytest"}.
The respective package must be installed and are loaded (but not attached).

If this function is not explicitly called, defaults to \code{"testthat"} unless
the \code{"tinytest"}'s namespace is loaded.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkR6.R
\name{checkR6}
\alias{checkR6}
\alias{check_r6}
\alias{assertR6}
\alias{assert_r6}
\alias{testR6}
\alias{test_r6}
\alias{expect_r6}
\title{Check if an argument is an R6 class}
\usage{
checkR6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE
)

check_r6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE
)

assertR6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_r6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testR6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE
)

test_r6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE
)

expect_r6(
  x,
  classes = NULL,
  ordered = FALSE,
  cloneable = NULL,
  public = NULL,
  private = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{classes}{[\code{character}]\cr
Class names to check for inheritance with \code{\link[base]{inherits}}.
\code{x} must inherit from all specified classes.}

\item{ordered}{[\code{logical(1)}]\cr
Expect \code{x} to be specialized in provided order.
Default is \code{FALSE}.}

\item{cloneable}{[\code{logical(1)}]\cr
If \code{TRUE}, check that \code{x} has a \code{clone} method. If \code{FALSE}, ensure that
\code{x} is not cloneable.}

\item{public}{[\code{character}]\cr
Names of expected public slots. This includes active bindings.}

\item{private}{[\code{character}]\cr
Names of expected private slots.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertClass}/\code{assert_class} return 
 \code{x} invisibly, whereas
 \code{checkClass}/\code{check_class} and 
 \code{testClass}/\code{test_class} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertClass}/\code{assert_class}
 throws an error message, 
 \code{testClass}/\code{test_class}
 returns \code{FALSE},
 and \code{checkClass}/\code{check_class} 
 return a string with the error message.
 The function \code{expect_class} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is an R6 class
}
\examples{
library(R6)
generator = R6Class("Bar",
  public = list(a = 5),
  private = list(b = 42),
  active = list(c = function() 99)
)
x = generator$new()
checkR6(x, "Bar", cloneable = TRUE, public = "a")
}
\seealso{
Other classes: 
\code{\link{checkClass}()},
\code{\link{checkMultiClass}()}
}
\concept{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFileExists.R
\name{checkFileExists}
\alias{checkFileExists}
\alias{check_file_exists}
\alias{assertFileExists}
\alias{assert_file_exists}
\alias{testFileExists}
\alias{test_file_exists}
\alias{expect_file_exists}
\alias{checkFile}
\alias{assertFile}
\alias{assert_file}
\alias{testFile}
\alias{expect_file}
\title{Check existence and access rights of files}
\usage{
checkFileExists(x, access = "", extension = NULL)

check_file_exists(x, access = "", extension = NULL)

assertFileExists(
  x,
  access = "",
  extension = NULL,
  .var.name = vname(x),
  add = NULL
)

assert_file_exists(
  x,
  access = "",
  extension = NULL,
  .var.name = vname(x),
  add = NULL
)

testFileExists(x, access = "", extension = NULL)

test_file_exists(x, access = "", extension = NULL)

expect_file_exists(
  x,
  access = "",
  extension = NULL,
  info = NULL,
  label = vname(x)
)

checkFile(x, access = "", extension = NULL)

assertFile(x, access = "", extension = NULL, .var.name = vname(x), add = NULL)

assert_file(x, access = "", extension = NULL, .var.name = vname(x), add = NULL)

testFile(x, access = "", extension = NULL)

expect_file(x, access = "", extension = NULL, info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{access}{[\code{character(1)}]\cr
Single string containing possible characters \sQuote{r}, \sQuote{w} and \sQuote{x} to
force a check for read, write or execute access rights, respectively.
Write and executable rights are not checked on Windows.}

\item{extension}{[\code{character}]\cr
Vector of allowed file extensions, matched case insensitive.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertFileExists}/\code{assert_file_exists} return 
 \code{x} invisibly, whereas
 \code{checkFileExists}/\code{check_file_exists} and 
 \code{testFileExists}/\code{test_file_exists} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertFileExists}/\code{assert_file_exists}
 throws an error message, 
 \code{testFileExists}/\code{test_file_exists}
 returns \code{FALSE},
 and \code{checkFileExists}/\code{check_file_exists} 
 return a string with the error message.
 The function \code{expect_file_exists} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check existence and access rights of files
}
\note{
The functions without the suffix \dQuote{exists} are deprecated and will be removed
from the package in a future version due to name clashes.
\code{test_file} has been unexported already.
}
\examples{
# Check if R's COPYING file is readable
testFileExists(file.path(R.home(), "COPYING"), access = "r")

# Check if R's COPYING file is readable and writable
testFileExists(file.path(R.home(), "COPYING"), access = "rw")
}
\seealso{
Other filesystem: 
\code{\link{checkAccess}()},
\code{\link{checkDirectoryExists}()},
\code{\link{checkPathForOutput}()}
}
\concept{filesystem}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkPathForOutput.R
\name{checkPathForOutput}
\alias{checkPathForOutput}
\alias{check_path_for_output}
\alias{assertPathForOutput}
\alias{assert_path_for_output}
\alias{testPathForOutput}
\alias{test_path_for_output}
\alias{expect_path_for_output}
\title{Check if a path is suited for creating an output file}
\usage{
checkPathForOutput(x, overwrite = FALSE, extension = NULL)

check_path_for_output(x, overwrite = FALSE, extension = NULL)

assertPathForOutput(
  x,
  overwrite = FALSE,
  extension = NULL,
  .var.name = vname(x),
  add = NULL
)

assert_path_for_output(
  x,
  overwrite = FALSE,
  extension = NULL,
  .var.name = vname(x),
  add = NULL
)

testPathForOutput(x, overwrite = FALSE, extension = NULL)

test_path_for_output(x, overwrite = FALSE, extension = NULL)

expect_path_for_output(
  x,
  overwrite = FALSE,
  extension = NULL,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{overwrite}{[\code{logical(1)}]\cr
If \code{TRUE}, an existing file in place is allowed if it
it is both readable and writable.
Default is \code{FALSE}.}

\item{extension}{[\code{character(1)}]\cr
Extension of the file, e.g. \dQuote{txt} or \dQuote{tar.gz}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertPathForOutput}/\code{assert_path_for_output} return 
 \code{x} invisibly, whereas
 \code{checkPathForOutput}/\code{check_path_for_output} and 
 \code{testPathForOutput}/\code{test_path_for_output} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertPathForOutput}/\code{assert_path_for_output}
 throws an error message, 
 \code{testPathForOutput}/\code{test_path_for_output}
 returns \code{FALSE},
 and \code{checkPathForOutput}/\code{check_path_for_output} 
 return a string with the error message.
 The function \code{expect_path_for_output} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if a file path can be used safely to create a file and write to it.

This is checked:
\itemize{
 \item{Does \code{dirname(x)} exist?}
 \item{Does no file under path \code{x} exist?}
 \item{Is \code{dirname(x)} writable?}
}
Paths are relative to the current working directory.
}
\examples{
# Can we create a file in the tempdir?
testPathForOutput(file.path(tempdir(), "process.log"))
}
\seealso{
Other filesystem: 
\code{\link{checkAccess}()},
\code{\link{checkDirectoryExists}()},
\code{\link{checkFileExists}()}
}
\concept{filesystem}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkPermutation.R
\name{checkPermutation}
\alias{checkPermutation}
\alias{check_permutation}
\alias{assertPermutation}
\alias{assert_permutation}
\alias{testPermutation}
\alias{test_permutation}
\alias{expect_permutation}
\title{Check if the arguments are permutations of each other.}
\usage{
checkPermutation(x, y, na.ok = TRUE)

check_permutation(x, y, na.ok = TRUE)

assertPermutation(x, y, na.ok = TRUE, .var.name = vname(x), add = NULL)

assert_permutation(x, y, na.ok = TRUE, .var.name = vname(x), add = NULL)

testPermutation(x, y, na.ok = TRUE)

test_permutation(x, y, na.ok = TRUE)

expect_permutation(x, y, na.ok = TRUE, info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{y}{[\code{atomic}]\cr
Vector to compare with. Atomic vector of type other than raw.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{TRUE}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertPermutation}/\code{assert_permutation} return 
 \code{x} invisibly, whereas
 \code{checkPermutation}/\code{check_permutation} and 
 \code{testPermutation}/\code{test_permutation} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertPermutation}/\code{assert_permutation}
 throws an error message, 
 \code{testPermutation}/\code{test_permutation}
 returns \code{FALSE},
 and \code{checkPermutation}/\code{check_permutation} 
 return a string with the error message.
 The function \code{expect_permutation} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
In contrast to \code{\link{checkSetEqual}}, the function tests for a true
permutation of the two vectors and also considers duplicated values.
Missing values are being treated as actual values by default.
Does not work on raw values.
}
\note{
The object \code{x} must be of the same type as the set w.r.t. \code{\link[base]{typeof}}.
Integers and doubles are both treated as numeric.
}
\examples{
testPermutation(letters[1:2], letters[2:1])
testPermutation(letters[c(1, 1, 2)], letters[1:2])
testPermutation(c(NA, 1, 2), c(1, 2, NA))
testPermutation(c(NA, 1, 2), c(1, 2, NA), na.ok = FALSE)
}
\seealso{
Other set: 
\code{\link{checkChoice}()},
\code{\link{checkDisjunct}()},
\code{\link{checkSetEqual}()},
\code{\link{checkSubset}()}
}
\concept{set}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkMatrix.R
\name{checkMatrix}
\alias{checkMatrix}
\alias{check_matrix}
\alias{assertMatrix}
\alias{assert_matrix}
\alias{testMatrix}
\alias{test_matrix}
\alias{expect_matrix}
\title{Check if an argument is a matrix}
\usage{
checkMatrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

check_matrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

assertMatrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_matrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testMatrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

test_matrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

expect_matrix(
  x,
  mode = NULL,
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{mode}{[\code{character(1)}]\cr
Storage mode of the array. Arrays can hold vectors, i.e. \dQuote{logical},
\dQuote{integer}, \dQuote{integerish}, \dQuote{double}, \dQuote{numeric}, \dQuote{complex},
\dQuote{character} and \dQuote{list}. You can also specify \dQuote{atomic}
here to explicitly prohibit lists. Default is \code{NULL} (no check).
If all values of \code{x} are missing, this check is skipped.}

\item{any.missing}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are matrices with only missing values allowed? Default is \code{TRUE}.}

\item{min.rows}{[\code{integer(1)}]\cr
Minimum number of rows.}

\item{max.rows}{[\code{integer(1)}]\cr
Maximum number of rows.}

\item{min.cols}{[\code{integer(1)}]\cr
Minimum number of columns.}

\item{max.cols}{[\code{integer(1)}]\cr
Maximum number of columns.}

\item{nrows}{[\code{integer(1)}]\cr
Exact number of rows.}

\item{ncols}{[\code{integer(1)}]\cr
Exact number of columns.}

\item{row.names}{[\code{character(1)}]\cr
Check for row names. Default is \dQuote{NULL} (no check).
See \code{\link{checkNamed}} for possible values.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{col.names}{[\code{character(1)}]\cr
Check for column names. Default is \dQuote{NULL} (no check).
See \code{\link{checkNamed}} for possible values.
Note that you can use \code{\link{checkSubset}} to test for a specific set of names.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertMatrix}/\code{assert_matrix} return 
 \code{x} invisibly, whereas
 \code{checkMatrix}/\code{check_matrix} and 
 \code{testMatrix}/\code{test_matrix} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertMatrix}/\code{assert_matrix}
 throws an error message, 
 \code{testMatrix}/\code{test_matrix}
 returns \code{FALSE},
 and \code{checkMatrix}/\code{check_matrix} 
 return a string with the error message.
 The function \code{expect_matrix} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a matrix
}
\examples{
x = matrix(1:9, 3)
colnames(x) = letters[1:3]
testMatrix(x, nrows = 3, min.cols = 1, col.names = "named")
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}

Other compound: 
\code{\link{checkArray}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDataTable}()},
\code{\link{checkTibble}()}
}
\concept{basetypes}
\concept{compound}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AssertCollection.R
\name{AssertCollection}
\alias{AssertCollection}
\alias{makeAssertCollection}
\alias{reportAssertions}
\title{Collect multiple assertions}
\usage{
makeAssertCollection()

reportAssertions(collection)
}
\arguments{
\item{collection}{[\code{AssertCollection}]\cr
Object of type \dQuote{AssertCollection} (constructed via \code{makeAssertCollection}).}
}
\value{
\code{makeAssertCollection()} returns an object of class \dQuote{AssertCollection} and
 \code{reportCollection} returns invisibly \code{TRUE} if no error is thrown (i.e., no message was
 collected).
}
\description{
The function \code{makeAssertCollection()} returns a simple stack-like
closure you can pass to all functions of the \code{assert*}-family.
All messages get collected and can be reported with \code{reportAssertions()}.
Alternatively, you can easily write your own report function or customize the the output of
the report function to a certain degree.
See the example on how to push custom messages or retrieve all stored messages.
}
\examples{
x = "a"
coll = makeAssertCollection()

print(coll$isEmpty())
assertNumeric(x, add = coll)
coll$isEmpty()
coll$push("Custom error message")
coll$getMessages()
\dontrun{
  reportAssertions(coll)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coalesce.R
\name{\%??\%}
\alias{\%??\%}
\title{Coalesce operator}
\usage{
lhs \%??\% rhs
}
\arguments{
\item{lhs}{[any]\cr
Left hand side of the operator. Is returned if not missing or \code{NULL}.}

\item{rhs}{[any]\cr
Right hand side of the operator. Is returned if \code{lhs} is missing or \code{NULL}.}
}
\value{
Either \code{lhs} or \code{rhs}.
}
\description{
Returns the left hand side if not missing nor \code{NULL}, and
the right hand side otherwise.
}
\examples{
print(NULL \%??\% 1 \%??\% 2)
print(names(iris) \%??\% letters[seq_len(ncol(iris))])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkNull.R
\name{checkNull}
\alias{checkNull}
\alias{check_null}
\alias{assertNull}
\alias{assert_null}
\alias{testNull}
\alias{test_null}
\title{Check if an argument is NULL}
\usage{
checkNull(x)

check_null(x)

assertNull(x, .var.name = vname(x), add = NULL)

assert_null(x, .var.name = vname(x), add = NULL)

testNull(x)

test_null(x)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertNull}/\code{assert_null} return 
 \code{x} invisibly, whereas
 \code{checkNull}/\code{check_null} and 
 \code{testNull}/\code{test_null} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertNull}/\code{assert_null}
 throws an error message, 
 \code{testNull}/\code{test_null}
 returns \code{FALSE},
 and \code{checkNull}/\code{check_null} 
 return a string with the error message.
 The function \code{expect_null} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is NULL
}
\examples{
testNull(NULL)
testNull(1)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vname.R
\name{vname}
\alias{vname}
\title{Lookup a variable name}
\usage{
vname(x)
}
\arguments{
\item{x}{[ANY]\cr
Object.}
}
\value{
[\code{character(1)}] Variable name.
}
\description{
Tries to heuristically determine the variable name of \code{x} in the parent frame
with a combination of \code{\link[base]{deparse}} and \code{\link[base]{substitute}}.
Used for checkmate's error messages.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeAssertion.R
\name{makeAssertion}
\alias{makeAssertion}
\alias{makeAssertionFunction}
\title{Turn a Check into an Assertion}
\usage{
makeAssertion(x, res, var.name, collection)

makeAssertionFunction(
  check.fun,
  c.fun = NULL,
  use.namespace = TRUE,
  coerce = FALSE,
  env = parent.frame()
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{res}{[\code{TRUE} | \code{character(1)}]\cr
The result of a check function: \code{TRUE} for successful checks,
and an error message as string otherwise.}

\item{var.name}{[\code{character(1)}]\cr
The custom name for \code{x} as passed to any \code{assert*} function.
Defaults to a heuristic name lookup.}

\item{collection}{[\code{\link{AssertCollection}}]\cr
If an \code{\link{AssertCollection}} is provided, the error message is stored
in it. If \code{NULL}, an exception is raised if \code{res} is not
\code{TRUE}.}

\item{check.fun}{[\code{function}]\cr
Function which checks the input. Must return \code{TRUE} on success and a string with the error message otherwise.}

\item{c.fun}{[\code{character(1)}]\cr
If not \code{NULL}, instead of calling the function \code{check.fun}, use \code{.Call} to call a C function \dQuote{c.fun} with the identical
set of parameters. The C function must be registered as a native symbol, see \code{\link[base]{.Call}}.
Useful if \code{check.fun} is just a simple wrapper.}

\item{use.namespace}{[\code{logical(1)}]\cr
Call functions of \pkg{checkmate} using its namespace explicitly.
Can be set to \code{FALSE} so save some microseconds, but the checkmate package needs to be imported.
Default is \code{TRUE}.}

\item{coerce}{[\code{logical(1)}]\cr
If \code{TRUE}, injects some lines of code to convert numeric values to integer after an successful assertion.
Currently used in \code{\link{assertCount}}, \code{\link{assertInt}} and \code{\link{assertIntegerish}}.}

\item{env}{[\code{environment}]\cr
The environment of the created function. Default is the \code{\link[base]{parent.frame}}.}
}
\value{
\code{makeAssertion} invisibly returns the checked object if the check was successful,
 and an exception is raised (or its message stored in the collection) otherwise.
 \code{makeAssertionFunction} returns a \code{function}.
}
\description{
\code{makeAssertion} is the internal function used to evaluate the result of a
check and throw an exception if necessary.
\code{makeAssertionFunction} can be used to automatically create an assertion
function based on a check function (see example).
}
\examples{
# Simple custom check function
checkFalse = function(x) if (!identical(x, FALSE)) "Must be FALSE" else TRUE

# Create the respective assert function
assertFalse = function(x, .var.name = vname(x), add = NULL) {
  res = checkFalse(x)
  makeAssertion(x, res, .var.name, add)
}

# Alternative: Automatically create such a function
assertFalse = makeAssertionFunction(checkFalse)
print(assertFalse)
}
\seealso{
Other CustomConstructors: 
\code{\link{makeExpectation}()},
\code{\link{makeTest}()}
}
\concept{CustomConstructors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkAtomic.R
\name{checkAtomic}
\alias{checkAtomic}
\alias{check_atomic}
\alias{assertAtomic}
\alias{assert_atomic}
\alias{testAtomic}
\alias{test_atomic}
\alias{expect_atomic}
\title{Check that an argument is an atomic vector}
\usage{
checkAtomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL
)

check_atomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL
)

assertAtomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  .var.name = vname(x),
  add = NULL
)

assert_atomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  .var.name = vname(x),
  add = NULL
)

testAtomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL
)

test_atomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL
)

expect_atomic(
  x,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertAtomic}/\code{assert_atomic} return 
 \code{x} invisibly, whereas
 \code{checkAtomic}/\code{check_atomic} and 
 \code{testAtomic}/\code{test_atomic} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertAtomic}/\code{assert_atomic}
 throws an error message, 
 \code{testAtomic}/\code{test_atomic}
 returns \code{FALSE},
 and \code{checkAtomic}/\code{check_atomic} 
 return a string with the error message.
 The function \code{expect_atomic} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
For the definition of \dQuote{atomic}, see \code{\link[base]{is.atomic}}.
}
\examples{
testAtomic(letters, min.len = 1L, any.missing = FALSE)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}

Other atomicvector: 
\code{\link{checkAtomicVector}()},
\code{\link{checkVector}()}
}
\concept{atomicvector}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkString.R
\name{checkString}
\alias{checkString}
\alias{check_string}
\alias{assertString}
\alias{assert_string}
\alias{testString}
\alias{test_string}
\alias{expect_string}
\title{Check if an argument is a string}
\usage{
checkString(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE
)

check_string(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE
)

assertString(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_string(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testString(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE
)

test_string(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE
)

expect_string(
  x,
  na.ok = FALSE,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{n.chars}{[\code{integer(1)}]\cr
Exact number of characters for each element of \code{x}.}

\item{min.chars}{[\code{integer(1)}]\cr
Minimum number of characters for each element of \code{x}.}

\item{max.chars}{[\code{integer(1)}]\cr
Maximum number of characters for each element of \code{x}.}

\item{pattern}{[\code{character(1L)}]\cr
Regular expression as used in \code{\link[base]{grepl}}.
All non-missing elements of \code{x} must comply to this pattern.}

\item{fixed}{[\code{character(1)}]\cr
Substring to detect in \code{x}. Will be used as \code{pattern} in \code{\link[base]{grepl}}
with option \code{fixed} set to \code{TRUE}.
All non-missing elements of \code{x} must contain this substring.}

\item{ignore.case}{[\code{logical(1)}]\cr
See \code{\link[base]{grepl}}. Default is \code{FALSE}.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertString}/\code{assert_string} return 
 \code{x} invisibly, whereas
 \code{checkString}/\code{check_string} and 
 \code{testString}/\code{test_string} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertString}/\code{assert_string}
 throws an error message, 
 \code{testString}/\code{test_string}
 returns \code{FALSE},
 and \code{checkString}/\code{check_string} 
 return a string with the error message.
 The function \code{expect_string} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
A string is defined as a scalar character vector.
To check for vectors of arbitrary length, see \code{\link{checkCharacter}}.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\examples{
testString("a")
testString(letters)
}
\seealso{
Other scalars: 
\code{\link{checkCount}()},
\code{\link{checkFlag}()},
\code{\link{checkInt}()},
\code{\link{checkNumber}()},
\code{\link{checkScalarNA}()},
\code{\link{checkScalar}()}
}
\concept{scalars}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeExpectation.R
\name{makeExpectation}
\alias{makeExpectation}
\alias{makeExpectationFunction}
\title{Turn a Check into an Expectation}
\usage{
makeExpectation(x, res, info, label)

makeExpectationFunction(
  check.fun,
  c.fun = NULL,
  use.namespace = FALSE,
  env = parent.frame()
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{res}{[\code{TRUE} | \code{character(1)}]\cr
The result of a check function: \code{TRUE} for successful checks,
and an error message as string otherwise.}

\item{info}{[\code{character(1)}]\cr
See \code{\link[testthat]{expect_that}}}

\item{label}{[\code{character(1)}]\cr
See \code{\link[testthat]{expect_that}}}

\item{check.fun}{[\code{function}]\cr
Function which checks the input. Must return \code{TRUE} on success and a string with the error message otherwise.}

\item{c.fun}{[\code{character(1)}]\cr
If not \code{NULL}, instead of calling the function \code{check.fun}, use \code{.Call} to call a C function \dQuote{c.fun} with the identical
set of parameters. The C function must be registered as a native symbol, see \code{\link[base]{.Call}}.
Useful if \code{check.fun} is just a simple wrapper.}

\item{use.namespace}{[\code{logical(1)}]\cr
Call functions of \pkg{checkmate} using its namespace explicitly.
Can be set to \code{FALSE} so save some microseconds, but the checkmate package needs to be imported.
Default is \code{TRUE}.}

\item{env}{[\code{environment}]\cr
The environment of the created function. Default is the \code{\link[base]{parent.frame}}.}
}
\value{
\code{makeExpectation} invisibly returns the checked object.
  \code{makeExpectationFunction} returns a \code{function}.
}
\description{
\code{makeExpectation} is the internal function used to evaluate the result of a
check and turn it into an \code{\link[testthat]{expectation}}.
\code{makeExceptionFunction} can be used to automatically create an expectation
function based on a check function (see example).
}
\examples{
# Simple custom check function
checkFalse = function(x) if (!identical(x, FALSE)) "Must be FALSE" else TRUE

# Create the respective expect function
expect_false = function(x, info = NULL, label = vname(x)) {
  res = checkFalse(x)
  makeExpectation(x, res, info = info, label = label)
}

# Alternative: Automatically create such a function
expect_false = makeExpectationFunction(checkFalse)
print(expect_false)
}
\seealso{
Other CustomConstructors: 
\code{\link{makeAssertion}()},
\code{\link{makeTest}()}
}
\concept{CustomConstructors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wfwl.R
\name{wf}
\alias{wf}
\alias{wl}
\title{Get the index of the first/last TRUE}
\usage{
wf(x, use.names = TRUE)

wl(x, use.names = TRUE)
}
\arguments{
\item{x}{[\code{logical}]\cr
Logical vector.}

\item{use.names}{[\code{logical(1)}]\cr
If \code{TRUE} and \code{x} is named, the result is also
named.}
}
\value{
[\code{integer(1)} | \code{integer(0)}].
 Returns the index of the first/last \code{TRUE} value in \code{x} or
 an empty integer vector if none is found. NAs are ignored.
}
\description{
A quick C implementation for \dQuote{which.first} (\code{head(which(x), 1)}) and
\dQuote{which.last} (\code{tail(which(x), 1)}).
}
\examples{
wf(c(FALSE, TRUE))
wl(c(FALSE, FALSE))
wf(NA)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkDataTable.R
\name{checkDataTable}
\alias{checkDataTable}
\alias{check_data_table}
\alias{assertDataTable}
\alias{assert_data_table}
\alias{testDataTable}
\alias{test_data_table}
\alias{expect_data_table}
\title{Check if an argument is a data table}
\usage{
checkDataTable(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

check_data_table(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

assertDataTable(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_data_table(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testDataTable(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

test_data_table(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE
)

expect_data_table(
  x,
  key = NULL,
  index = NULL,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  min.rows = NULL,
  max.rows = NULL,
  min.cols = NULL,
  max.cols = NULL,
  nrows = NULL,
  ncols = NULL,
  row.names = NULL,
  col.names = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{key}{[\code{character}]\cr
Expected primary key(s) of the data table.}

\item{index}{[\code{character}]\cr
Expected secondary key(s) of the data table.}

\item{types}{[\code{character}]\cr
Character vector of class names. Each list element must inherit
from at least one of the provided types.
The types \dQuote{logical}, \dQuote{integer}, \dQuote{integerish}, \dQuote{double},
\dQuote{numeric}, \dQuote{complex}, \dQuote{character}, \dQuote{factor}, \dQuote{atomic}, \dQuote{vector}
\dQuote{atomicvector}, \dQuote{array}, \dQuote{matrix}, \dQuote{list}, \dQuote{function},
\dQuote{environment} and \dQuote{null} are supported.
For other types \code{\link[base]{inherits}} is used as a fallback to check \code{x}'s inheritance.
Defaults to \code{character(0)} (no check).}

\item{any.missing}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are matrices with only missing values allowed? Default is \code{TRUE}.}

\item{min.rows}{[\code{integer(1)}]\cr
Minimum number of rows.}

\item{max.rows}{[\code{integer(1)}]\cr
Maximum number of rows.}

\item{min.cols}{[\code{integer(1)}]\cr
Minimum number of columns.}

\item{max.cols}{[\code{integer(1)}]\cr
Maximum number of columns.}

\item{nrows}{[\code{integer(1)}]\cr
Exact number of rows.}

\item{ncols}{[\code{integer(1)}]\cr
Exact number of columns.}

\item{row.names}{[\code{character(1)}]\cr
Check for row names. Default is \dQuote{NULL} (no check).
See \code{\link{checkNamed}} for possible values.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{col.names}{[\code{character(1)}]\cr
Check for column names. Default is \dQuote{NULL} (no check).
See \code{\link{checkNamed}} for possible values.
Note that you can use \code{\link{checkSubset}} to test for a specific set of names.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertDataTable}/\code{assert_data_table} return 
 \code{x} invisibly, whereas
 \code{checkDataTable}/\code{check_data_table} and 
 \code{testDataTable}/\code{test_data_table} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertDataTable}/\code{assert_data_table}
 throws an error message, 
 \code{testDataTable}/\code{test_data_table}
 returns \code{FALSE},
 and \code{checkDataTable}/\code{check_data_table} 
 return a string with the error message.
 The function \code{expect_data_table} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a data table
}
\examples{
library(data.table)
dt = as.data.table(iris)
setkeyv(dt, "Species")
setkeyv(dt, "Sepal.Length", physical = FALSE)
testDataTable(dt)
testDataTable(dt, key = "Species", index = "Sepal.Length", any.missing = FALSE)
}
\seealso{
Other compound: 
\code{\link{checkArray}()},
\code{\link{checkDataFrame}()},
\code{\link{checkMatrix}()},
\code{\link{checkTibble}()}
}
\concept{compound}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkCharacter.R
\name{checkCharacter}
\alias{checkCharacter}
\alias{check_character}
\alias{assertCharacter}
\alias{assert_character}
\alias{testCharacter}
\alias{test_character}
\alias{expect_character}
\title{Check if an argument is a vector of type character}
\usage{
checkCharacter(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

check_character(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

assertCharacter(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_character(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testCharacter(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

test_character(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

expect_character(
  x,
  n.chars = NULL,
  min.chars = NULL,
  max.chars = NULL,
  pattern = NULL,
  fixed = NULL,
  ignore.case = FALSE,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{n.chars}{[\code{integer(1)}]\cr
Exact number of characters for each element of \code{x}.}

\item{min.chars}{[\code{integer(1)}]\cr
Minimum number of characters for each element of \code{x}.}

\item{max.chars}{[\code{integer(1)}]\cr
Maximum number of characters for each element of \code{x}.}

\item{pattern}{[\code{character(1L)}]\cr
Regular expression as used in \code{\link[base]{grepl}}.
All non-missing elements of \code{x} must comply to this pattern.}

\item{fixed}{[\code{character(1)}]\cr
Substring to detect in \code{x}. Will be used as \code{pattern} in \code{\link[base]{grepl}}
with option \code{fixed} set to \code{TRUE}.
All non-missing elements of \code{x} must contain this substring.}

\item{ignore.case}{[\code{logical(1)}]\cr
See \code{\link[base]{grepl}}. Default is \code{FALSE}.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{sorted}{[\code{logical(1)}]\cr
Elements must be sorted in ascending order. Missing values are ignored.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{typed.missing}{[\code{logical(1)}]\cr
If set to \code{FALSE} (default), all types of missing values (\code{NA}, \code{NA_integer_},
\code{NA_real_}, \code{NA_character_} or \code{NA_character_}) as well as empty vectors are allowed
while type-checking atomic input.
Set to \code{TRUE} to enable strict type checking.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertCharacter}/\code{assert_character} return 
 \code{x} invisibly, whereas
 \code{checkCharacter}/\code{check_character} and 
 \code{testCharacter}/\code{test_character} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertCharacter}/\code{assert_character}
 throws an error message, 
 \code{testCharacter}/\code{test_character}
 returns \code{FALSE},
 and \code{checkCharacter}/\code{check_character} 
 return a string with the error message.
 The function \code{expect_character} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
To check for scalar strings, see \code{\link{checkString}}.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\examples{
testCharacter(letters, min.len = 1, any.missing = FALSE)
testCharacter(letters, min.chars = 2)
testCharacter("example", pattern = "xa")
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkAccess.R
\name{checkAccess}
\alias{checkAccess}
\alias{check_access}
\alias{assertAccess}
\alias{assert_access}
\alias{testAccess}
\alias{test_access}
\alias{expect_access}
\title{Check file system access rights}
\usage{
checkAccess(x, access = "")

check_access(x, access = "")

assertAccess(x, access = "", .var.name = vname(x), add = NULL)

assert_access(x, access = "", .var.name = vname(x), add = NULL)

testAccess(x, access = "")

test_access(x, access = "")

expect_access(x, access = "", info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{access}{[\code{character(1)}]\cr
Single string containing possible characters \sQuote{r}, \sQuote{w} and \sQuote{x} to
force a check for read, write or execute access rights, respectively.
Write and executable rights are not checked on Windows.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertAccess}/\code{assert_access} return 
 \code{x} invisibly, whereas
 \code{checkAccess}/\code{check_access} and 
 \code{testAccess}/\code{test_access} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertAccess}/\code{assert_access}
 throws an error message, 
 \code{testAccess}/\code{test_access}
 returns \code{FALSE},
 and \code{checkAccess}/\code{check_access} 
 return a string with the error message.
 The function \code{expect_access} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check file system access rights
}
\examples{
# Is R's home directory readable?
testAccess(R.home(), "r")

# Is R's home directory writeable?
testAccess(R.home(), "w")
}
\seealso{
Other filesystem: 
\code{\link{checkDirectoryExists}()},
\code{\link{checkFileExists}()},
\code{\link{checkPathForOutput}()}
}
\concept{filesystem}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkNames.R
\name{checkNames}
\alias{checkNames}
\alias{check_names}
\alias{assertNames}
\alias{assert_names}
\alias{testNames}
\alias{test_names}
\alias{expect_names}
\title{Check names to comply to specific rules}
\usage{
checkNames(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names"
)

check_names(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names"
)

assertNames(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names",
  .var.name = vname(x),
  add = NULL
)

assert_names(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names",
  .var.name = vname(x),
  add = NULL
)

testNames(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names"
)

test_names(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names"
)

expect_names(
  x,
  type = "named",
  subset.of = NULL,
  must.include = NULL,
  permutation.of = NULL,
  identical.to = NULL,
  disjunct.from = NULL,
  what = "names",
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[\code{character} || \code{NULL}]\cr
Names to check using rules defined via \code{type}.}

\item{type}{[character(1)]\cr
Type of formal check(s) to perform on the names.
\describe{
\item{unnamed:}{Checks \code{x} to be \code{NULL}.}
\item{named:}{Checks \code{x} for regular names which excludes names to be \code{NA} or empty (\code{""}).}
\item{unique:}{Performs checks like with \dQuote{named} and additionally tests for non-duplicated names.}
\item{strict:}{Performs checks like with \dQuote{unique} and additionally fails for names with UTF-8 characters and names which do not comply to R's variable name restrictions. As regular expression, this is \dQuote{^[.]*[a-zA-Z]+[a-zA-Z0-9._]*$}.}
\item{ids:}{Same as \dQuote{strict}, but does not enforce uniqueness.}
}
Note that for zero-length \code{x}, all these name checks evaluate to \code{TRUE}.}

\item{subset.of}{[\code{character}]\cr
Names provided in \code{x} must be subset of the set \code{subset.of}.}

\item{must.include}{[\code{character}]\cr
Names provided in \code{x} must be a superset of the set \code{must.include}.}

\item{permutation.of}{[\code{character}]\cr
Names provided in \code{x} must be a permutation of the set \code{permutation.of}.
Duplicated names in \code{permutation.of} are stripped out and duplicated names in \code{x}
thus lead to a failed check.
Use this argument instead of \code{identical.to} if the order of the names is not relevant.}

\item{identical.to}{[\code{character}]\cr
Names provided in \code{x} must be identical to the vector \code{identical.to}.
Use this argument instead of \code{permutation.of} if the order of the names is relevant.}

\item{disjunct.from}{[\code{character}]\cr
Names provided in \code{x} must may not be present in the vector \code{disjunct.from}.}

\item{what}{[\code{character(1)}]\cr
Type of name vector to check, e.g. \dQuote{names} (default), \dQuote{colnames} or \dQuote{rownames}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertNames}/\code{assert_names} return 
 \code{x} invisibly, whereas
 \code{checkNames}/\code{check_names} and 
 \code{testNames}/\code{test_names} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertNames}/\code{assert_names}
 throws an error message, 
 \code{testNames}/\code{test_names}
 returns \code{FALSE},
 and \code{checkNames}/\code{check_names} 
 return a string with the error message.
 The function \code{expect_names} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Performs various checks on character vectors, usually names.
}
\examples{
x = 1:3
testNames(names(x), "unnamed")
names(x) = letters[1:3]
testNames(names(x), "unique")

cn = c("Species", "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width")
assertNames(names(iris), permutation.of = cn)
}
\seealso{
Other attributes: 
\code{\link{checkClass}()},
\code{\link{checkMultiClass}()},
\code{\link{checkNamed}()}
}
\concept{attributes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkClass.R
\name{checkClass}
\alias{checkClass}
\alias{check_class}
\alias{assertClass}
\alias{assert_class}
\alias{testClass}
\alias{test_class}
\alias{expect_class}
\title{Check the class membership of an argument}
\usage{
checkClass(x, classes, ordered = FALSE, null.ok = FALSE)

check_class(x, classes, ordered = FALSE, null.ok = FALSE)

assertClass(
  x,
  classes,
  ordered = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_class(
  x,
  classes,
  ordered = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testClass(x, classes, ordered = FALSE, null.ok = FALSE)

test_class(x, classes, ordered = FALSE, null.ok = FALSE)

expect_class(
  x,
  classes,
  ordered = FALSE,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{classes}{[\code{character}]\cr
Class names to check for inheritance with \code{\link[base]{inherits}}.
\code{x} must inherit from all specified classes.}

\item{ordered}{[\code{logical(1)}]\cr
Expect \code{x} to be specialized in provided order.
Default is \code{FALSE}.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertClass}/\code{assert_class} return 
 \code{x} invisibly, whereas
 \code{checkClass}/\code{check_class} and 
 \code{testClass}/\code{test_class} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertClass}/\code{assert_class}
 throws an error message, 
 \code{testClass}/\code{test_class}
 returns \code{FALSE},
 and \code{checkClass}/\code{check_class} 
 return a string with the error message.
 The function \code{expect_class} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check the class membership of an argument
}
\examples{
# Create an object with classes "foo" and "bar"
x = 1
class(x) = c("foo", "bar")

# is x of class "foo"?
testClass(x, "foo")

# is x of class "foo" and "bar"?
testClass(x, c("foo", "bar"))

# is x of class "foo" or "bar"?
\dontrun{
assert(
  checkClass(x, "foo"),
  checkClass(x, "bar")
)
}
# is x most specialized as "bar"?
testClass(x, "bar", ordered = TRUE)
}
\seealso{
Other attributes: 
\code{\link{checkMultiClass}()},
\code{\link{checkNamed}()},
\code{\link{checkNames}()}

Other classes: 
\code{\link{checkMultiClass}()},
\code{\link{checkR6}()}
}
\concept{attributes}
\concept{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qassertr.R
\name{qassertr}
\alias{qassertr}
\alias{qtestr}
\alias{qexpectr}
\title{Quick recursive arguments checks on lists and data frames}
\usage{
qassertr(x, rules, .var.name = vname(x))

qtestr(x, rules, depth = 1L)

qexpectr(x, rules, info = NULL, label = vname(x))
}
\arguments{
\item{x}{[\code{list} or \code{data.frame}]\cr
List or data frame to check for compliance with at least one of \code{rules}.
See details of \code{\link{qtest}} for rule explanation.}

\item{rules}{[\code{character}]\cr
Set of rules. See \code{\link{qtest}}}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in error messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{depth}{[\code{integer(1)}]\cr
Maximum recursion depth. Defaults to \dQuote{1} to directly check list elements or
data frame columns. Set to a higher value to check lists of lists of elements.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
See \code{\link{qassert}}.
}
\description{
These functions are the tuned counterparts of \code{\link{qtest}},
\code{\link{qassert}} and \code{\link{qexpect}} tailored for recursive
checks of list elements or data frame columns.
}
\examples{
# All list elements are integers with length >= 1?
qtestr(as.list(1:10), "i+")

# All list elements (i.e. data frame columns) are numeric?
qtestr(iris, "n")

# All list elements are numeric, w/o NAs?
qtestr(list(a = 1:3, b = rnorm(1), c = letters), "N+")

# All list elements are numeric OR character
qtestr(list(a = 1:3, b = rnorm(1), c = letters), c("N+", "S+"))
}
\seealso{
\code{\link{qtest}}, \code{\link{qassert}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkCount.R
\name{checkCount}
\alias{checkCount}
\alias{check_count}
\alias{assertCount}
\alias{assert_count}
\alias{testCount}
\alias{test_count}
\alias{expect_count}
\title{Check if an argument is a count}
\usage{
checkCount(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

check_count(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

assertCount(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE,
  coerce = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_count(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE,
  coerce = FALSE,
  .var.name = vname(x),
  add = NULL
)

testCount(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

test_count(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE
)

expect_count(
  x,
  na.ok = FALSE,
  positive = FALSE,
  tol = sqrt(.Machine$double.eps),
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{positive}{[\code{logical(1)}]\cr
Must \code{x} be positive (>= 1)?
Default is \code{FALSE}, allowing 0.}

\item{tol}{[\code{double(1)}]\cr
Numerical tolerance used to check whether a double or complex can be converted.
Default is \code{sqrt(.Machine$double.eps)}.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{coerce}{[\code{logical(1)}]\cr
If \code{TRUE}, the input \code{x} is returned as integer after an successful assertion.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertCount}/\code{assert_count} return 
 \code{x} invisibly, whereas
 \code{checkCount}/\code{check_count} and 
 \code{testCount}/\code{test_count} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertCount}/\code{assert_count}
 throws an error message, 
 \code{testCount}/\code{test_count}
 returns \code{FALSE},
 and \code{checkCount}/\code{check_count} 
 return a string with the error message.
 The function \code{expect_count} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
A count is defined as non-negative integerish value.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\note{
To perform an assertion and then convert to integer, use \code{\link{asCount}}.
\code{assertCount} will not convert numerics to integer.
}
\examples{
testCount(1)
testCount(-1)
}
\seealso{
Other scalars: 
\code{\link{checkFlag}()},
\code{\link{checkInt}()},
\code{\link{checkNumber}()},
\code{\link{checkScalarNA}()},
\code{\link{checkScalar}()},
\code{\link{checkString}()}
}
\concept{scalars}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkIntegerish.R
\name{checkIntegerish}
\alias{checkIntegerish}
\alias{check_integerish}
\alias{assertIntegerish}
\alias{assert_integerish}
\alias{testIntegerish}
\alias{test_integerish}
\alias{expect_integerish}
\title{Check if an object is an integerish vector}
\usage{
checkIntegerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

check_integerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

assertIntegerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  coerce = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_integerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  coerce = FALSE,
  .var.name = vname(x),
  add = NULL
)

testIntegerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

test_integerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE
)

expect_integerish(
  x,
  tol = sqrt(.Machine$double.eps),
  lower = -Inf,
  upper = Inf,
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  sorted = FALSE,
  names = NULL,
  typed.missing = FALSE,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{tol}{[\code{double(1)}]\cr
Numerical tolerance used to check whether a double or complex can be converted.
Default is \code{sqrt(.Machine$double.eps)}.}

\item{lower}{[\code{numeric(1)}]\cr
Lower value all elements of \code{x} must be greater than or equal to.}

\item{upper}{[\code{numeric(1)}]\cr
Upper value all elements of \code{x} must be lower than or equal to.}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{sorted}{[\code{logical(1)}]\cr
Elements must be sorted in ascending order. Missing values are ignored.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{typed.missing}{[\code{logical(1)}]\cr
If set to \code{FALSE} (default), all types of missing values (\code{NA}, \code{NA_integer_},
\code{NA_real_}, \code{NA_character_} or \code{NA_character_}) as well as empty vectors are allowed
while type-checking atomic input.
Set to \code{TRUE} to enable strict type checking.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{coerce}{[\code{logical(1)}]\cr
If \code{TRUE}, the input \code{x} is returned as integer after an successful assertion.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertIntegerish}/\code{assert_integerish} return 
 \code{x} invisibly, whereas
 \code{checkIntegerish}/\code{check_integerish} and 
 \code{testIntegerish}/\code{test_integerish} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertIntegerish}/\code{assert_integerish}
 throws an error message, 
 \code{testIntegerish}/\code{test_integerish}
 returns \code{FALSE},
 and \code{checkIntegerish}/\code{check_integerish} 
 return a string with the error message.
 The function \code{expect_integerish} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
An integerish value is defined as value safely convertible to integer.
This includes integers and numeric values which sufficiently close to an
integer w.r.t. a numeric tolerance `tol`.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\note{
To convert from integerish to integer, use \code{\link{asInteger}}.
}
\examples{
testIntegerish(1L)
testIntegerish(1.)
testIntegerish(1:2, lower = 1L, upper = 2L, any.missing = FALSE)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkNamed.R
\name{checkNamed}
\alias{checkNamed}
\alias{check_named}
\alias{assertNamed}
\alias{assert_named}
\alias{testNamed}
\alias{test_named}
\title{Check if an argument is named}
\usage{
checkNamed(x, type = "named")

check_named(x, type = "named")

assertNamed(x, type = "named", .var.name = vname(x), add = NULL)

assert_named(x, type = "named", .var.name = vname(x), add = NULL)

testNamed(x, type = "named")

test_named(x, type = "named")
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{type}{[character(1)]\cr
Select the check(s) to perform.
\dQuote{unnamed} checks \code{x} to be unnamed.
\dQuote{named} (default) checks \code{x} to be named which excludes names to be \code{NA} or empty (\code{""}).
\dQuote{unique} additionally tests for non-duplicated names.
\dQuote{strict} checks for unique names which comply to R's variable name restrictions.
Note that for zero-length \code{x} every name check evaluates to \code{TRUE}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertNamed}/\code{assert_named} return 
 \code{x} invisibly, whereas
 \code{checkNamed}/\code{check_named} and 
 \code{testNamed}/\code{test_named} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertNamed}/\code{assert_named}
 throws an error message, 
 \code{testNamed}/\code{test_named}
 returns \code{FALSE},
 and \code{checkNamed}/\code{check_named} 
 return a string with the error message.
 The function \code{expect_named} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is named
}
\note{
These function are deprecated and will be removed in a future version.
Please use \code{\link{checkNames}} instead.
}
\examples{
x = 1:3
testNamed(x, "unnamed")
names(x) = letters[1:3]
testNamed(x, "unique")
}
\seealso{
Other attributes: 
\code{\link{checkClass}()},
\code{\link{checkMultiClass}()},
\code{\link{checkNames}()}
}
\concept{attributes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkDirectoryExists.R
\name{checkDirectoryExists}
\alias{checkDirectoryExists}
\alias{check_directory_exists}
\alias{assertDirectoryExists}
\alias{assert_directory_exists}
\alias{testDirectoryExists}
\alias{test_directory_exists}
\alias{expect_directory_exists}
\alias{checkDirectory}
\alias{assertDirectory}
\alias{assert_directory}
\alias{testDirectory}
\alias{test_directory}
\alias{expect_directory}
\title{Check for existence and access rights of directories}
\usage{
checkDirectoryExists(x, access = "")

check_directory_exists(x, access = "")

assertDirectoryExists(x, access = "", .var.name = vname(x), add = NULL)

assert_directory_exists(x, access = "", .var.name = vname(x), add = NULL)

testDirectoryExists(x, access = "")

test_directory_exists(x, access = "")

expect_directory_exists(x, access = "", info = NULL, label = vname(x))

checkDirectory(x, access = "")

assertDirectory(x, access = "", .var.name = vname(x), add = NULL)

assert_directory(x, access = "", .var.name = vname(x), add = NULL)

testDirectory(x, access = "")

test_directory(x, access = "")

expect_directory(x, access = "", info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{access}{[\code{character(1)}]\cr
Single string containing possible characters \sQuote{r}, \sQuote{w} and \sQuote{x} to
force a check for read, write or execute access rights, respectively.
Write and executable rights are not checked on Windows.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertDirectoryExists}/\code{assert_directory_exists} return 
 \code{x} invisibly, whereas
 \code{checkDirectoryExists}/\code{check_directory_exists} and 
 \code{testDirectoryExists}/\code{test_directory_exists} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertDirectoryExists}/\code{assert_directory_exists}
 throws an error message, 
 \code{testDirectoryExists}/\code{test_directory_exists}
 returns \code{FALSE},
 and \code{checkDirectoryExists}/\code{check_directory_exists} 
 return a string with the error message.
 The function \code{expect_directory_exists} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check for existence and access rights of directories
}
\note{
The functions without the suffix \dQuote{exists} are deprecated and will be removed
from the package in a future version due to name clashes.
}
\examples{
# Is R's home directory readable?
testDirectory(R.home(), "r")

# Is R's home directory readable and writable?
testDirectory(R.home(), "rw")
}
\seealso{
Other filesystem: 
\code{\link{checkAccess}()},
\code{\link{checkFileExists}()},
\code{\link{checkPathForOutput}()}
}
\concept{filesystem}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFlag.R
\name{checkFlag}
\alias{checkFlag}
\alias{check_flag}
\alias{assertFlag}
\alias{assert_flag}
\alias{testFlag}
\alias{test_flag}
\alias{expect_flag}
\title{Check if an argument is a flag}
\usage{
checkFlag(x, na.ok = FALSE, null.ok = FALSE)

check_flag(x, na.ok = FALSE, null.ok = FALSE)

assertFlag(x, na.ok = FALSE, null.ok = FALSE, .var.name = vname(x), add = NULL)

assert_flag(
  x,
  na.ok = FALSE,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testFlag(x, na.ok = FALSE, null.ok = FALSE)

test_flag(x, na.ok = FALSE, null.ok = FALSE)

expect_flag(x, na.ok = FALSE, null.ok = FALSE, info = NULL, label = vname(x))
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertFlag}/\code{assert_flag} return 
 \code{x} invisibly, whereas
 \code{checkFlag}/\code{check_flag} and 
 \code{testFlag}/\code{test_flag} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertFlag}/\code{assert_flag}
 throws an error message, 
 \code{testFlag}/\code{test_flag}
 returns \code{FALSE},
 and \code{checkFlag}/\code{check_flag} 
 return a string with the error message.
 The function \code{expect_flag} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
A flag is defined as single logical value.
}
\details{
This function does not distinguish between
\code{NA}, \code{NA_integer_}, \code{NA_real_}, \code{NA_complex_}
\code{NA_character_} and \code{NaN}.
}
\examples{
testFlag(TRUE)
testFlag(1)
}
\seealso{
Other scalars: 
\code{\link{checkCount}()},
\code{\link{checkInt}()},
\code{\link{checkNumber}()},
\code{\link{checkScalarNA}()},
\code{\link{checkScalar}()},
\code{\link{checkString}()}
}
\concept{scalars}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{checkmate-package}
\alias{checkmate}
\alias{checkmate-package}
\title{checkmate: Fast and Versatile Argument Checks}
\description{
Tests and assertions to perform frequent argument checks. A substantial part of the package was written in C to minimize any worries about execution time overhead.
}
\section{Check scalars}{

\itemize{
  \item{\code{\link{checkFlag}}}
  \item{\code{\link{checkCount}}}
  \item{\code{\link{checkNumber}}}
  \item{\code{\link{checkInt}}}
  \item{\code{\link{checkString}}}
  \item{\code{\link{checkScalar}}}
  \item{\code{\link{checkScalarNA}}}
}
}

\section{Check vectors}{

\itemize{
  \item{\code{\link{checkLogical}}}
  \item{\code{\link{checkNumeric}}}
  \item{\code{\link{checkDouble}}}
  \item{\code{\link{checkInteger}}}
  \item{\code{\link{checkIntegerish}}}
  \item{\code{\link{checkCharacter}}}
  \item{\code{\link{checkComplex}}}
  \item{\code{\link{checkFactor}}}
  \item{\code{\link{checkList}}}
  \item{\code{\link{checkPOSIXct}}}
  \item{\code{\link{checkVector}}}
  \item{\code{\link{checkAtomic}}}
  \item{\code{\link{checkAtomicVector}}}
  \item{\code{\link{checkRaw}}}
}
}

\section{Check attributes}{

\itemize{
  \item{\code{\link{checkClass}}}
  \item{\code{\link{checkMultiClass}}}
  \item{\code{\link{checkNames}}}
  \item{\code{\link{checkNamed}}} (deprecated)
}
}

\section{Check compound types}{

\itemize{
  \item{\code{\link{checkArray}}}
  \item{\code{\link{checkDataFrame}}}
  \item{\code{\link{checkMatrix}}}
}
}

\section{Check other built-in R types}{

\itemize{
  \item{\code{\link{checkDate}}}
  \item{\code{\link{checkEnvironment}}}
  \item{\code{\link{checkFunction}}}
  \item{\code{\link{checkFormula}}}
  \item{\code{\link{checkNull}}}
}
}

\section{Check sets}{

\itemize{
  \item{\code{\link{checkChoice}}}
  \item{\code{\link{checkSubset}}}
  \item{\code{\link{checkSetEqual}}}
  \item{\code{\link{checkDisjunct}}}
  \item{\code{\link{checkPermutation}}}
}
}

\section{File IO}{

\itemize{
  \item{\code{\link{checkFileExists}}}
  \item{\code{\link{checkDirectoryExists}}}
  \item{\code{\link{checkPathForOutput}}}
  \item{\code{\link{checkAccess}}}
}
}

\section{Popular data types of third party packages}{

\itemize{
  \item{\code{\link{checkDataTable}}}
  \item{\code{\link{checkR6}}}
  \item{\code{\link{checkTibble}}}
}
}

\section{Safe coercion to integer}{

\itemize{
  \item{\code{\link{asCount}}}
  \item{\code{\link{asInt}}}
  \item{\code{\link{asInteger}}}
}
}

\section{Quick argument checks using a DSL}{

\itemize{
  \item{\code{\link{qassert}}}
  \item{\code{\link{qassertr}}}
}
}

\section{Misc}{

\itemize{
  \item{\code{\link{checkOS}} (check operating system)}
  \item{\code{\link{assert}} (combine multiple checks into an assertion)}
  \item{\code{\link{anyMissing}}}
  \item{\code{\link{allMissing}}}
  \item{\code{\link{anyNaN}}}
  \item{\code{\link{wf}} (which.first and which.last)}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://mllg.github.io/checkmate/}
  \item \url{https://github.com/mllg/checkmate}
  \item Report bugs at \url{https://github.com/mllg/checkmate/issues}
}

}
\author{
\strong{Maintainer}: Michel Lang \email{michellang@gmail.com} (\href{https://orcid.org/0000-0001-9754-0393}{ORCID})

Other contributors:
\itemize{
  \item Bernd Bischl \email{bernd_bischl@gmx.net} [contributor]
  \item Dénes Tóth \email{toth.denes@kogentum.hu} (\href{https://orcid.org/0000-0003-4262-3217}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFunction.R
\name{checkFunction}
\alias{checkFunction}
\alias{check_function}
\alias{assertFunction}
\alias{assert_function}
\alias{testFunction}
\alias{test_function}
\alias{expect_function}
\title{Check if an argument is a function}
\usage{
checkFunction(x, args = NULL, ordered = FALSE, nargs = NULL, null.ok = FALSE)

check_function(x, args = NULL, ordered = FALSE, nargs = NULL, null.ok = FALSE)

assertFunction(
  x,
  args = NULL,
  ordered = FALSE,
  nargs = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_function(
  x,
  args = NULL,
  ordered = FALSE,
  nargs = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testFunction(x, args = NULL, ordered = FALSE, nargs = NULL, null.ok = FALSE)

test_function(x, args = NULL, ordered = FALSE, nargs = NULL, null.ok = FALSE)

expect_function(
  x,
  args = NULL,
  ordered = FALSE,
  nargs = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{args}{[\code{character}]\cr
Expected formal arguments. Checks that a function has no arguments if
set to \code{character(0)}.
Default is \code{NULL} (no check).}

\item{ordered}{[\code{logical(1)}]\cr
Flag whether the arguments provided in \code{args} must be the first
\code{length(args)} arguments of the function in the specified order.
Default is \code{FALSE}.}

\item{nargs}{[\code{integer(1)}]\cr
Required number of arguments, without \code{...}.
Default is \code{NULL} (no check).}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertFunction}/\code{assert_function} return 
 \code{x} invisibly, whereas
 \code{checkFunction}/\code{check_function} and 
 \code{testFunction}/\code{test_function} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertFunction}/\code{assert_function}
 throws an error message, 
 \code{testFunction}/\code{test_function}
 returns \code{FALSE},
 and \code{checkFunction}/\code{check_function} 
 return a string with the error message.
 The function \code{expect_function} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a function
}
\examples{
testFunction(mean)
testFunction(mean, args = "x")
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkList}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkList.R
\name{checkList}
\alias{checkList}
\alias{check_list}
\alias{assertList}
\alias{assert_list}
\alias{testList}
\alias{test_list}
\alias{expect_list}
\title{Check if an argument is a list}
\usage{
checkList(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

check_list(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

assertList(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

assert_list(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  .var.name = vname(x),
  add = NULL
)

testList(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

test_list(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE
)

expect_list(
  x,
  types = character(0L),
  any.missing = TRUE,
  all.missing = TRUE,
  len = NULL,
  min.len = NULL,
  max.len = NULL,
  unique = FALSE,
  names = NULL,
  null.ok = FALSE,
  info = NULL,
  label = vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{types}{[\code{character}]\cr
Character vector of class names. Each list element must inherit
from at least one of the provided types.
The types \dQuote{logical}, \dQuote{integer}, \dQuote{integerish}, \dQuote{double},
\dQuote{numeric}, \dQuote{complex}, \dQuote{character}, \dQuote{factor}, \dQuote{atomic}, \dQuote{vector}
\dQuote{atomicvector}, \dQuote{array}, \dQuote{matrix}, \dQuote{list}, \dQuote{function},
\dQuote{environment} and \dQuote{null} are supported.
For other types \code{\link[base]{inherits}} is used as a fallback to check \code{x}'s inheritance.
Defaults to \code{character(0)} (no check).}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{all.missing}{[\code{logical(1)}]\cr
Are vectors with no non-missing values allowed? Default is \code{TRUE}.
Note that empty vectors do not have non-missing values.}

\item{len}{[\code{integer(1)}]\cr
Exact expected length of \code{x}.}

\item{min.len}{[\code{integer(1)}]\cr
Minimal length of \code{x}.}

\item{max.len}{[\code{integer(1)}]\cr
Maximal length of \code{x}.}

\item{unique}{[\code{logical(1)}]\cr
Must all values be unique? Default is \code{FALSE}.}

\item{names}{[\code{character(1)}]\cr
Check for names. See \code{\link{checkNamed}} for possible values.
Default is \dQuote{any} which performs no check at all.
Note that you can use \code{\link{checkSubset}} to check for a specific set of names.}

\item{null.ok}{[\code{logical(1)}]\cr
If set to \code{TRUE}, \code{x} may also be \code{NULL}.
In this case only a type check of \code{x} is performed, all additional checks are disabled.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link{vname}}.}

\item{add}{[\code{AssertCollection}]\cr
Collection to store assertion messages. See \code{\link{AssertCollection}}.}

\item{info}{[\code{character(1)}]\cr
Extra information to be included in the message for the testthat reporter.
See \code{\link[testthat]{expect_that}}.}

\item{label}{[\code{character(1)}]\cr
Name of the checked object to print in messages. Defaults to
the heuristic implemented in \code{\link{vname}}.}
}
\value{
Depending on the function prefix:
 If the check is successful, the functions 
 \code{assertList}/\code{assert_list} return 
 \code{x} invisibly, whereas
 \code{checkList}/\code{check_list} and 
 \code{testList}/\code{test_list} return 
 \code{TRUE}.
 If the check is not successful, 
 \code{assertList}/\code{assert_list}
 throws an error message, 
 \code{testList}/\code{test_list}
 returns \code{FALSE},
 and \code{checkList}/\code{check_list} 
 return a string with the error message.
 The function \code{expect_list} always returns an
 \code{\link[testthat]{expectation}}.
}
\description{
Check if an argument is a list
}
\note{
Contrary to R's \code{\link[base]{is.list}}, objects of type \code{\link[base]{data.frame}}
and \code{\link[base]{pairlist}} are not recognized as list.

Missingness is defined here as elements of the list being \code{NULL}, analogously to \code{\link{anyMissing}}.

The test for uniqueness does differentiate between the different NA types which are built-in in R.
This is required to be consistent with \code{\link[base]{unique}} while checking
scalar missing values. Also see the example.
}
\examples{
testList(list())
testList(as.list(iris), types = c("numeric", "factor"))

# Missingness
testList(list(1, NA), any.missing = FALSE)
testList(list(1, NULL), any.missing = FALSE)

# Uniqueness differentiates between different NA types:
testList(list(NA, NA), unique = TRUE)
testList(list(NA, NA_real_), unique = TRUE)
}
\seealso{
Other basetypes: 
\code{\link{checkArray}()},
\code{\link{checkAtomicVector}()},
\code{\link{checkAtomic}()},
\code{\link{checkCharacter}()},
\code{\link{checkComplex}()},
\code{\link{checkDataFrame}()},
\code{\link{checkDate}()},
\code{\link{checkDouble}()},
\code{\link{checkEnvironment}()},
\code{\link{checkFactor}()},
\code{\link{checkFormula}()},
\code{\link{checkFunction}()},
\code{\link{checkIntegerish}()},
\code{\link{checkInteger}()},
\code{\link{checkLogical}()},
\code{\link{checkMatrix}()},
\code{\link{checkNull}()},
\code{\link{checkNumeric}()},
\code{\link{checkPOSIXct}()},
\code{\link{checkRaw}()},
\code{\link{checkVector}()}
}
\concept{basetypes}
